@@ -24,6 +24,17 @@
  * \brief Answering machine detection
  *
  * \author Claude Klimos (claude.klimos@aheeva.com)
+ *
+ * \ingroup applications
+ */
+
+/*! \li \ref app_amd.c uses the configuration file \ref amd.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*! 
+ * \page amd.conf amd.conf
+ * \verbinclude amd.conf.sample
  */
 
 /*** MODULEINFO
@@ -32,7 +43,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 375004 $")
 
 #include "asterisk/module.h"
 #include "asterisk/lock.h"
@@ -50,19 +61,19 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		<syntax>
 			<parameter name="initialSilence" required="false">
 				<para>Is maximum initial silence duration before greeting.</para>
-				<para>If this is exceeded, the result is detection as a MACHINE</para>
+				<para>If this is exceeded set as MACHINE</para>
 			</parameter>
 			<parameter name="greeting" required="false">
 				<para>is the maximum length of a greeting.</para>
-				<para>If this is exceeded, the result is detection as a MACHINE</para>
+				<para>If this is exceeded set as MACHINE</para>
 			</parameter>
 			<parameter name="afterGreetingSilence" required="false">
 				<para>Is the silence after detecting a greeting.</para>
-				<para>If this is exceeded, the result is detection as a HUMAN</para>
+				<para>If this is exceeded set as HUMAN</para>
 			</parameter>
 			<parameter name="totalAnalysis Time" required="false">
 				<para>Is the maximum time allowed for the algorithm</para>
-				<para>to decide on whether the audio represents a HUMAN, or a MACHINE</para>
+				<para>to decide HUMAN or MACHINE</para>
 			</parameter>
 			<parameter name="miniumWordLength" required="false">
 				<para>Is the minimum duration of Voice considered to be a word</para>
@@ -73,14 +84,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			</parameter>
 			<parameter name="maximumNumberOfWords" required="false">
 				<para>Is the maximum number of words in a greeting</para>
-				<para>If this is REACHED, then the result is detection as a MACHINE</para>
+				<para>If this is exceeded set as MACHINE</para>
 			</parameter>
 			<parameter name="silenceThreshold" required="false">
-				<para>What is the average level of noise from 0 to 32767 which if not exceeded, should be considered silence?</para>
+				<para>How long do we consider silence</para>
 			</parameter>
 			<parameter name="maximumWordLength" required="false">
 				<para>Is the maximum duration of a word to accept.</para>
-				<para>If exceeded, then the result is detection as a MACHINE</para>
+				<para>If exceeded set as MACHINE</para>
 			</parameter>
 		</syntax>
 		<description>
@@ -114,11 +125,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 						Voice Duration - Greeting.
 					</value>
 					<value name="MAXWORDLENGTH">
-						Word Length - max length of a single word.
-					</value>
-					<value name="MAXWORDS">
 						Word Count - maximum number of words.
-					</value>
+					</value>	
 				</variable>
 			</variablelist>
 		</description>
@@ -490,6 +498,16 @@ static int unload_module(void)
 	return ast_unregister_application(app);
 }
 
+/*!
+ * \brief Load the module
+ *
+ * Module loading including tests for configuration or dependencies.
+ * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
+ * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
+ * configuration file or other non-critical problem return 
+ * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
+ */
 static int load_module(void)
 {
 	if (load_config(0))
