@@ -33,7 +33,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 410157 $")
 
 #include <sys/ioctl.h>
 
@@ -56,10 +56,10 @@ static const char app[] = "ChanIsAvail";
 			<parameter name="Technology/Resource" required="true" argsep="&amp;">
 				<argument name="Technology2/Resource2" multiple="true">
 					<para>Optional extra devices to check</para>
-					<para>If you need more than one enter them as
-					Technology2/Resource2&amp;Technology3/Resource3&amp;.....</para>
+					<para>If you need more then one enter them as
+					Technology2/Resource2&amp;Technology3/Resourse3&amp;.....</para>
 				</argument>
-				<para>Specification of the device(s) to check.  These must be in the format of
+				<para>Specification of the device(s) to check.  These must be in the format of 
 				<literal>Technology/Resource</literal>, where <replaceable>Technology</replaceable>
 				represents a particular channel driver, and <replaceable>Resource</replaceable>
 				represents a resource available to that particular channel driver.</para>
@@ -171,7 +171,7 @@ static int chanavail_exec(struct ast_channel *chan, const char *data)
 			}
 			snprintf(tmp, sizeof(tmp), "%d", status);
 			ast_str_append(&tmp_availstat, 0, "%s%s", ast_str_strlen(tmp_availstat) ? "&" : "", tmp);
-			if ((inuse <= 1) && (tempchan = ast_request(tech, ast_channel_nativeformats(chan), chan, number, &status))) {
+			if ((inuse <= 1) && (tempchan = ast_request(tech, ast_channel_nativeformats(chan), NULL, chan, number, &status))) {
 					ast_str_append(&tmp_availchan, 0, "%s%s", ast_str_strlen(tmp_availchan) ? "&" : "", ast_channel_name(tempchan));
 					
 					snprintf(tmp, sizeof(tmp), "%s/%s", tech, number);
