@@ -32,7 +32,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 422718 $")
 
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -243,11 +243,12 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 	int setmacrocontext=0;
 	int autoloopflag, inhangup = 0;
 	struct ast_str *tmp_subst = NULL;
-	const char *my_macro_exten = NULL;
+  
 	char *save_macro_exten;
 	char *save_macro_context;
 	char *save_macro_priority;
 	char *save_macro_offset;
+	int save_in_subroutine;
 	struct ast_datastore *macro_store = ast_channel_datastore_find(chan, &macro_ds_info, NULL);
 
 	if (ast_strlen_zero(data)) {
@@ -303,32 +304,12 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 	}
 
 	snprintf(fullmacro, sizeof(fullmacro), "macro-%s", macro);
-
-	/* first search for the macro */
-	if (!ast_context_find(fullmacro)) {
-		ast_log(LOG_WARNING, "No such context '%s' for macro '%s'. Was called by %s@%s\n",
-			fullmacro, macro, ast_channel_exten(chan), ast_channel_context(chan));
-		return 0;
-	}
-
-	/* now search for the right extension */
-	if (ast_exists_extension(chan, fullmacro, "s", 1,
-		S_COR(ast_channel_caller(chan)->id.number.valid,
-			ast_channel_caller(chan)->id.number.str, NULL))) {
-		/* We have a normal macro */
-		my_macro_exten = "s";
-	} else if (ast_exists_extension(chan, fullmacro, "~~s~~", 1,
-		S_COR(ast_channel_caller(chan)->id.number.valid,
-			ast_channel_caller(chan)->id.number.str, NULL))) {
-		/* We have an AEL generated macro */
-		my_macro_exten = "~~s~~";
-	}
-
-	/* do we have a valid exten? */
-	if (!my_macro_exten) {
-		ast_log(LOG_WARNING,
-			"Context '%s' for macro '%s' lacks 's' extension, priority 1\n",
-			fullmacro, macro);
+	if (!ast_exists_extension(chan, fullmacro, "s", 1,
+		S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, NULL))) {
+		if (!ast_context_find(fullmacro)) 
+			ast_log(LOG_WARNING, "No such context '%s' for macro '%s'. Was called by %s@%s\n", fullmacro, macro, ast_channel_exten(chan), ast_channel_context(chan));
+		else
+			ast_log(LOG_WARNING, "Context '%s' for macro '%s' lacks 's' extension, priority 1\n", fullmacro, macro);
 		return 0;
 	}
 
@@ -376,8 +357,11 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 
 	pbx_builtin_setvar_helper(chan, "MACRO_DEPTH", depthc);
 
+	save_in_subroutine = ast_test_flag(ast_channel_flags(chan), AST_FLAG_SUBROUTINE_EXEC);
+	ast_set_flag(ast_channel_flags(chan), AST_FLAG_SUBROUTINE_EXEC);
+
 	/* Setup environment for new run */
-	ast_channel_exten_set(chan, my_macro_exten);
+	ast_channel_exten_set(chan, "s");
 	ast_channel_context_set(chan, fullmacro);
 	ast_channel_priority_set(chan, 1);
 
@@ -392,10 +376,9 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 		pbx_builtin_setvar_helper(chan, varname, cur);
 		argc++;
 	}
+	ast_channel_unlock(chan);
 	autoloopflag = ast_test_flag(ast_channel_flags(chan), AST_FLAG_IN_AUTOLOOP);
 	ast_set_flag(ast_channel_flags(chan), AST_FLAG_IN_AUTOLOOP);
-	ast_channel_unlock(chan);
-
 	while (ast_exists_extension(chan, ast_channel_context(chan), ast_channel_exten(chan), ast_channel_priority(chan),
 		S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, NULL))) {
 		struct ast_context *c;
@@ -520,10 +503,7 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 
 		/* don't stop executing extensions when we're in "h" */
 		if (ast_check_hangup(chan) && !inhangup) {
-			ast_debug(1, "Extension %s, macroexten %s, priority %d returned normally even though call was hung up\n",
-				ast_channel_exten(chan),
-				ast_channel_macroexten(chan),
-				ast_channel_priority(chan));
+			ast_debug(1, "Extension %s, macroexten %s, priority %d returned normally even though call was hung up\n", ast_channel_exten(chan), ast_channel_macroexten(chan), ast_channel_priority(chan));
 			goto out;
 		}
 		ast_channel_priority_set(chan, ast_channel_priority(chan) + 1);
@@ -537,21 +517,29 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 	snprintf(depthc, sizeof(depthc), "%d", depth);
 	pbx_builtin_setvar_helper(chan, "MACRO_DEPTH", depthc);
 	ast_set2_flag(ast_channel_flags(chan), autoloopflag, AST_FLAG_IN_AUTOLOOP);
+	ast_set2_flag(ast_channel_flags(chan), save_in_subroutine, AST_FLAG_SUBROUTINE_EXEC);
 
   	for (x = 1; x < argc; x++) {
   		/* Restore old arguments and delete ours */
 		snprintf(varname, sizeof(varname), "ARG%d", x);
-		pbx_builtin_setvar_helper(chan, varname, oldargs[x]);
-		ast_free(oldargs[x]);
+  		if (oldargs[x]) {
+			pbx_builtin_setvar_helper(chan, varname, oldargs[x]);
+			ast_free(oldargs[x]);
+		} else {
+			pbx_builtin_setvar_helper(chan, varname, NULL);
+		}
   	}
 
 	/* Restore macro variables */
 	pbx_builtin_setvar_helper(chan, "MACRO_EXTEN", save_macro_exten);
 	pbx_builtin_setvar_helper(chan, "MACRO_CONTEXT", save_macro_context);
 	pbx_builtin_setvar_helper(chan, "MACRO_PRIORITY", save_macro_priority);
-	ast_free(save_macro_exten);
-	ast_free(save_macro_context);
-	ast_free(save_macro_priority);
+	if (save_macro_exten)
+		ast_free(save_macro_exten);
+	if (save_macro_context)
+		ast_free(save_macro_context);
+	if (save_macro_priority)
+		ast_free(save_macro_priority);
 
 	if (setmacrocontext) {
 		ast_channel_macrocontext_set(chan, "");
@@ -559,8 +547,7 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 		ast_channel_macropriority_set(chan, 0);
 	}
 
-	if (!strcasecmp(ast_channel_context(chan), fullmacro)
-		&& !(ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_ASYNCGOTO)) {
+	if (!strcasecmp(ast_channel_context(chan), fullmacro)) {
 		const char *offsets;
 
   		/* If we're leaving the macro normally, restore original information */
@@ -581,7 +568,8 @@ static int _macro_exec(struct ast_channel *chan, const char *data, int exclusive
 	}
 
 	pbx_builtin_setvar_helper(chan, "MACRO_OFFSET", save_macro_offset);
-	ast_free(save_macro_offset);
+	if (save_macro_offset)
+		ast_free(save_macro_offset);
 
 	/* Unlock the macro */
 	if (exclusive) {
