@@ -32,7 +32,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 410157 $")
 
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -60,7 +60,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 				<argument name="Technology2/Resource2" multiple="true">
 					<para>Optional extra devices to dial in parallel</para>
 					<para>If you need more than one, enter them as Technology2/Resource2&amp;
-					Technology3/Resource3&amp;.....</para>
+					Technology3/Resourse3&amp;.....</para>
 				</argument>
 			</parameter>
 			<parameter name="options">
@@ -318,7 +318,7 @@ static int page_exec(struct ast_channel *chan, const char *data)
 		}
 
 		/* Append technology and resource */
-		if (ast_dial_append(dial, tech, resource) == -1) {
+		if (ast_dial_append(dial, tech, resource, NULL) == -1) {
 			ast_log(LOG_ERROR, "Failed to add %s to outbound dial\n", tech);
 			ast_dial_destroy(dial);
 			continue;
