@@ -25,14 +25,23 @@
  * \brief PostgreSQL CDR logger
  *
  * \author Matthew D. Hardeman <mhardemn@papersoft.com>
- * \extref PostgreSQL http://www.postgresql.org/
+ * PostgreSQL http://www.postgresql.org/
  *
  * See also
  * \arg \ref Config_cdr
- * \extref PostgreSQL http://www.postgresql.org/
+ * PostgreSQL http://www.postgresql.org/
  * \ingroup cdr_drivers
  */
 
+/*! \li \ref cdr_pgsql.c uses the configuration file \ref cdr_pgsql.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*!
+ * \page cdr_pgsql.conf cdr_pgsql.conf
+ * \verbinclude cdr_pgsql.conf.sample
+ */
+
 /*** MODULEINFO
 	<depend>pgsql</depend>
 	<support_level>extended</support_level>
@@ -40,7 +49,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 402081 $")
 
 #include <libpq-fe.h>
 
@@ -124,7 +133,7 @@ static char *handle_cdr_pgsql_status(struct ast_cli_entry *e, int cmd, struct as
 		return NULL;
 	}
 
-	if (a->argc != e->args)
+	if (a->argc != 3)
 		return CLI_SHOWUSAGE;
 
 	if (connected) {
@@ -213,9 +222,9 @@ static int pgsql_log(struct ast_cdr *cdr)
 		AST_RWLIST_RDLOCK(&psql_columns);
 		AST_RWLIST_TRAVERSE(&psql_columns, cur, list) {
 			/* For fields not set, simply skip them */
-			ast_cdr_getvar(cdr, cur->name, &value, buf, sizeof(buf), 0, 0);
+			ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 			if (strcmp(cur->name, "calldate") == 0 && !value) {
-				ast_cdr_getvar(cdr, "start", &value, buf, sizeof(buf), 0, 0);
+				ast_cdr_format_var(cdr, "start", &value, buf, sizeof(buf), 0);
 			}
 			if (!value) {
 				if (cur->notnull && !cur->hasdefault) {
@@ -277,7 +286,7 @@ static int pgsql_log(struct ast_cdr *cdr)
 			} else if (strcmp(cur->name, "duration") == 0 || strcmp(cur->name, "billsec") == 0) {
 				if (cur->type[0] == 'i') {
 					/* Get integer, no need to escape anything */
-					ast_cdr_getvar(cdr, cur->name, &value, buf, sizeof(buf), 0, 0);
+					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(13);
 					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
@@ -293,18 +302,18 @@ static int pgsql_log(struct ast_cdr *cdr)
 			} else if (strcmp(cur->name, "disposition") == 0 || strcmp(cur->name, "amaflags") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					/* Integer, no need to escape anything */
-					ast_cdr_getvar(cdr, cur->name, &value, buf, sizeof(buf), 0, 1);
+					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 1);
 					LENGTHEN_BUF2(13);
 					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
 				} else {
 					/* Although this is a char field, there are no special characters in the values for these fields */
-					ast_cdr_getvar(cdr, cur->name, &value, buf, sizeof(buf), 0, 0);
+					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(31);
 					ast_str_append(&sql2, 0, "%s'%s'", first ? "" : ",", value);
 				}
 			} else {
 				/* Arbitrary field, could be anything */
-				ast_cdr_getvar(cdr, cur->name, &value, buf, sizeof(buf), 0, 0);
+				ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 				if (strncmp(cur->type, "int", 3) == 0) {
 					long long whatever;
 					if (value && sscanf(value, "%30lld", &whatever) == 1) {
@@ -427,7 +436,10 @@ static void empty_columns(void)
 
 static int unload_module(void)
 {
-	ast_cdr_unregister(name);
+	if (ast_cdr_unregister(name)) {
+		return -1;
+	}
+
 	ast_cli_unregister_multiple(cdr_pgsql_status_cli, ARRAY_LEN(cdr_pgsql_status_cli));
 
 	PQfinish(conn);
@@ -603,20 +615,42 @@ static int config_module(int reload)
 		version = PQserverVersion(conn);
 
 		if (version >= 70300) {
-			char *schemaname, *tablename, *tmp_schemaname, *tmp_tablename;
+			char *schemaname, *tablename;
 			if (strchr(table, '.')) {
-				tmp_schemaname = ast_strdupa(table);
-				tmp_tablename = strchr(tmp_schemaname, '.');
-				*tmp_tablename++ = '\0';
+				schemaname = ast_strdupa(table);
+				tablename = strchr(schemaname, '.');
+				*tablename++ = '\0';
 			} else {
-				tmp_schemaname = "";
-				tmp_tablename = table;
+				schemaname = "";
+				tablename = table;
 			}
-			tablename = ast_alloca(strlen(tmp_tablename) * 2 + 1);
-			PQescapeStringConn(conn, tablename, tmp_tablename, strlen(tmp_tablename), NULL);
 
-			schemaname = ast_alloca(strlen(tmp_schemaname) * 2 + 1);
-			PQescapeStringConn(conn, schemaname, tmp_schemaname, strlen(tmp_schemaname), NULL);
+			/* Escape special characters in schemaname */
+			if (strchr(schemaname, '\\') || strchr(schemaname, '\'')) {
+				char *tmp = schemaname, *ptr;
+
+				ptr = schemaname = ast_alloca(strlen(tmp) * 2 + 1);
+				for (; *tmp; tmp++) {
+					if (strchr("\\'", *tmp)) {
+						*ptr++ = *tmp;
+					}
+					*ptr++ = *tmp;
+				}
+				*ptr = '\0';
+			}
+			/* Escape special characters in tablename */
+			if (strchr(tablename, '\\') || strchr(tablename, '\'')) {
+				char *tmp = tablename, *ptr;
+
+				ptr = tablename = ast_alloca(strlen(tmp) * 2 + 1);
+				for (; *tmp; tmp++) {
+					if (strchr("\\'", *tmp)) {
+						*ptr++ = *tmp;
+					}
+					*ptr++ = *tmp;
+				}
+				*ptr = '\0';
+			}
 
 			snprintf(sqlcmd, sizeof(sqlcmd), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
 				tablename,
