@@ -21,12 +21,17 @@
  * \brief RADIUS CDR Support
  *
  * \author Philippe Sultan
- * \extref The Radius Client Library - http://developer.berlios.de/projects/radiusclient-ng/
+ * The Radius Client Library
+ * 	http://developer.berlios.de/projects/radiusclient-ng/
  *
  * \arg See also \ref AstCDR
  * \ingroup cdr_drivers
  */
 
+/*! \li \ref cdr_radius.c uses the configuration file \ref cdr.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
 /*** MODULEINFO
 	<depend>radius</depend>
 	<support_level>extended</support_level>
@@ -34,7 +39,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 406803 $")
 
 #ifdef FREERADIUS_CLIENT
 #include <freeradius-client.h>
@@ -173,12 +178,12 @@ static int build_radius_record(VALUE_PAIR **tosend, struct ast_cdr *cdr)
 		return -1;
 
 	/* Disposition */
-	tmp = ast_cdr_disp2str(cdr->disposition);
+	tmp = ast_strdupa(ast_cdr_disp2str(cdr->disposition));
 	if (!rc_avpair_add(rh, tosend, PW_AST_DISPOSITION, tmp, strlen(tmp), VENDOR_CODE))
 		return -1;
 
 	/* AMA Flags */
-	tmp = ast_cdr_flags2str(cdr->amaflags);
+	tmp = ast_strdupa(ast_channel_amaflags2string(cdr->amaflags));
 	if (!rc_avpair_add(rh, tosend, PW_AST_AMA_FLAGS, tmp, strlen(tmp), VENDOR_CODE))
 		return -1;
 
@@ -195,7 +200,8 @@ static int build_radius_record(VALUE_PAIR **tosend, struct ast_cdr *cdr)
 	}
 
 	/* Setting Acct-Session-Id & User-Name attributes for proper generation
-	   of Acct-Unique-Session-Id on server side */
+	 * of Acct-Unique-Session-Id on server side 
+	 */
 	/* Channel */
 	if (!rc_avpair_add(rh, tosend, PW_USER_NAME, &cdr->channel, strlen(cdr->channel), 0))
 		return -1;
@@ -232,7 +238,10 @@ return_cleanup:
 
 static int unload_module(void)
 {
-	ast_cdr_unregister(name);
+	if (ast_cdr_unregister(name)) {
+		return -1;
+	}
+
 	if (rh) {
 		rc_destroy(rh);
 		rh = NULL;
