@@ -29,11 +29,11 @@
  * \brief PostgreSQL CEL logger 
  * 
  * \author Steve Murphy <murf@digium.com>
- * \extref PostgreSQL http://www.postgresql.org/
+ * PostgreSQL http://www.postgresql.org/
  *
  * See also
  * \arg \ref Config_cel
- * \extref PostgreSQL http://www.postgresql.org/
+ * PostgreSQL http://www.postgresql.org/
  * \ingroup cel_drivers
  */
 
@@ -44,7 +44,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include <libpq-fe.h>
 
@@ -58,6 +58,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #define DATE_FORMAT "%Y-%m-%d %T.%6q"
 
+#define PGSQL_BACKEND_NAME "CEL PGSQL backend"
+
 static char *config = "cel_pgsql.conf";
 static char *pghostname = NULL, *pgdbname = NULL, *pgdbuser = NULL, *pgpassword = NULL, *pgdbport = NULL, *table = NULL;
 static int connected = 0;
@@ -73,7 +75,6 @@ AST_MUTEX_DEFINE_STATIC(pgsql_lock);
 
 static PGconn	*conn = NULL;
 static PGresult	*result = NULL;
-static struct ast_event_sub *event_sub = NULL;
 
 struct columns {
 	char *name;
@@ -91,7 +92,7 @@ static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
 		/* Lengthen buffer, if necessary */ \
 		if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) { \
 			if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 3) / 512 + 1) * 512) != 0) { \
-				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", pghostname, table); \
+				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n"); \
 				ast_free(sql); \
 				ast_free(sql2); \
 				AST_RWLIST_UNLOCK(&psql_columns); \
@@ -104,7 +105,7 @@ static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
 	do { \
 		if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) { \
 			if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) { \
-				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", pghostname, table); \
+				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n"); \
 				ast_free(sql); \
 				ast_free(sql2); \
 				AST_RWLIST_UNLOCK(&psql_columns); \
@@ -113,7 +114,7 @@ static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
 		} \
 	} while (0)
 
-static void pgsql_log(const struct ast_event *event, void *userdata)
+static void pgsql_log(struct ast_event *event)
 {
 	struct ast_tm tm;
 	char timestr[128];
@@ -346,10 +347,7 @@ static int my_unload_module(void)
 {
 	struct columns *current;
 
-	if (event_sub) {
-		event_sub = ast_event_unsubscribe(event_sub);
-	}
-
+	ast_cel_backend_unregister(PGSQL_BACKEND_NAME);
 	AST_RWLIST_WRLOCK(&psql_columns);
 	if (conn) {
 		PQfinish(conn);
@@ -566,9 +564,7 @@ static int my_load_module(int reload)
 	process_my_load_module(cfg);
 	ast_config_destroy(cfg);
 
-	event_sub = ast_event_subscribe(AST_EVENT_CEL, pgsql_log, "CEL PGSQL backend", NULL, AST_EVENT_IE_END);
-
-	if (!event_sub) {
+	if (ast_cel_backend_register(PGSQL_BACKEND_NAME, pgsql_log)) {
 		ast_log(LOG_WARNING, "Unable to subscribe to CEL events for pgsql\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
