@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Version Info: $Id$
+ * Version Info: $Id: chan_h323.h 404348 2013-12-19 16:37:56Z rmudgett $
  */
 
 #ifndef CHAN_H323_H
@@ -31,6 +31,7 @@
 
 #include <arpa/inet.h>
 #include "asterisk/format.h"
+#include "asterisk/app.h"
 
 /*
  * Enable support for sending/reception of tunnelled Q.SIG messages and
@@ -94,7 +95,7 @@ struct oh323_user {
    All peers are registered to a GK if there is one */
 struct oh323_peer {
 	ASTOBJ_COMPONENTS(struct oh323_peer);
-	char mailbox[80];
+	char mailbox[AST_MAX_MAILBOX_UNIQUEID];
 	int delme;
 	struct sockaddr_in addr;
 	struct ast_ha *ha;
