@@ -42,6 +42,7 @@
 #include "asterisk/musiconhold.h"
 #include "asterisk/cli.h"
 #include "asterisk/transcap.h"
+#include "asterisk/stasis_channels.h"
 
 #include "sig_ss7.h"
 #if defined(LIBSS7_ABI_COMPATIBILITY)
@@ -257,12 +258,15 @@ static void sig_ss7_loopback(struct sig_ss7_chan *p, int enable)
 	}
 }
 
-static struct ast_channel *sig_ss7_new_ast_channel(struct sig_ss7_chan *p, int state, int ulaw, int transfercapability, char *exten, const struct ast_channel *requestor)
+static struct ast_channel *sig_ss7_new_ast_channel(struct sig_ss7_chan *p, int state,
+	int ulaw, int transfercapability, char *exten,
+	const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor)
 {
 	struct ast_channel *ast;
 
 	if (sig_ss7_callbacks.new_ast_channel) {
-		ast = sig_ss7_callbacks.new_ast_channel(p->chan_pvt, state, ulaw, exten, requestor);
+		ast = sig_ss7_callbacks.new_ast_channel(p->chan_pvt, state, ulaw, exten,
+			assignedids, requestor);
 	} else {
 		return NULL;
 	}
@@ -603,7 +607,7 @@ static void ss7_start_call(struct sig_ss7_chan *p, struct sig_ss7_linkset *links
 	 */
 	ast_mutex_unlock(&linkset->lock);
 	sig_ss7_unlock_private(p);
-	c = sig_ss7_new_ast_channel(p, AST_STATE_RING, law, 0, p->exten, NULL);
+	c = sig_ss7_new_ast_channel(p, AST_STATE_RING, law, 0, p->exten, NULL, NULL);
 	if (!c) {
 		ast_log(LOG_WARNING, "Unable to start PBX on CIC %d\n", p->cic);
 		ast_mutex_lock(&linkset->lock);
@@ -621,6 +625,8 @@ static void ss7_start_call(struct sig_ss7_chan *p, struct sig_ss7_linkset *links
 
 	sig_ss7_set_echocanceller(p, 1);
 
+	ast_channel_stage_snapshot(c);
+
 	/*
 	 * It is reasonably safe to set the following
 	 * channel variables while the channel private
@@ -700,6 +706,8 @@ static void ss7_start_call(struct sig_ss7_chan *p, struct sig_ss7_linkset *links
 		p->generic_name[0] = 0;
 	}
 
+	ast_channel_stage_snapshot_done(c);
+
 	sig_ss7_unlock_private(p);
 	ast_channel_unlock(c);
 
@@ -1884,7 +1892,9 @@ int sig_ss7_indicate(struct sig_ss7_chan *p, struct ast_channel *chan, int condi
  * \retval ast_channel on success.
  * \retval NULL on error.
  */
-struct ast_channel *sig_ss7_request(struct sig_ss7_chan *p, enum sig_ss7_law law, const struct ast_channel *requestor, int transfercapability)
+struct ast_channel *sig_ss7_request(struct sig_ss7_chan *p, enum sig_ss7_law law,
+	const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor,
+	int transfercapability)
 {
 	struct ast_channel *ast;
 
@@ -1896,7 +1906,8 @@ struct ast_channel *sig_ss7_request(struct sig_ss7_chan *p, enum sig_ss7_law law
 	}
 
 	sig_ss7_set_outgoing(p, 1);
-	ast = sig_ss7_new_ast_channel(p, AST_STATE_RESERVED, law, transfercapability, p->exten, requestor);
+	ast = sig_ss7_new_ast_channel(p, AST_STATE_RESERVED, law, transfercapability,
+		p->exten, assignedids, requestor);
 	if (!ast) {
 		sig_ss7_set_outgoing(p, 0);
 
