@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include "include/sip.h"
 #include "include/config_parser.h"
@@ -40,7 +40,7 @@ int sip_parse_register_line(struct sip_registry *reg, int default_expiry, const
 {
 	int portnum = 0;
 	int domainport = 0;
-	enum sip_transport transport = SIP_TRANSPORT_UDP;
+	enum ast_transport transport = AST_TRANSPORT_UDP;
 	char buf[256] = "";
 	char *userpart = NULL, *hostpart = NULL;
 	/* register => [peer?][transport://]user[@domain][:secret[:authuser]]@host[:port][/extension][~expiry] */
@@ -79,17 +79,13 @@ int sip_parse_register_line(struct sip_registry *reg, int default_expiry, const
 		AST_APP_ARG(port);
 	);
 
-	if (!reg) {
-		return -1;
-	}
-
-	reg->expire = -1;
-	reg->timeout = -1;
-
 	if (!value) {
 		return -1;
 	}
 
+	if (!reg) {
+		return -1;
+	}
 	ast_copy_string(buf, value, sizeof(buf));
 
 	/*! register => [peer?][transport://]user[@domain][:secret[:authuser]]@host[:port][/extension][~expiry]
@@ -234,21 +230,21 @@ int sip_parse_register_line(struct sip_registry *reg, int default_expiry, const
 
 	/* set transport type */
 	if (!pre2.transport) {
-		transport = SIP_TRANSPORT_UDP;
+		transport = AST_TRANSPORT_UDP;
 	} else if (!strncasecmp(pre2.transport, "tcp", 3)) {
-		transport = SIP_TRANSPORT_TCP;
+		transport = AST_TRANSPORT_TCP;
 	} else if (!strncasecmp(pre2.transport, "tls", 3)) {
-		transport = SIP_TRANSPORT_TLS;
+		transport = AST_TRANSPORT_TLS;
 	} else if (!strncasecmp(pre2.transport, "udp", 3)) {
-		transport = SIP_TRANSPORT_UDP;
+		transport = AST_TRANSPORT_UDP;
 	} else {
-		transport = SIP_TRANSPORT_UDP;
+		transport = AST_TRANSPORT_UDP;
 		ast_log(LOG_NOTICE, "'%.3s' is not a valid transport type on line %d of sip.conf. defaulting to udp.\n", pre2.transport, lineno);
 	}
 
 	/* if no portnum specified, set default for transport */
 	if (!portnum) {
-		if (transport == SIP_TRANSPORT_TLS) {
+		if (transport == AST_TRANSPORT_TLS) {
 			portnum = STANDARD_TLS_PORT;
 		} else {
 			portnum = STANDARD_SIP_PORT;
@@ -265,6 +261,7 @@ int sip_parse_register_line(struct sip_registry *reg, int default_expiry, const
 	ast_string_field_set(reg, regdomain, ast_strip_quoted(S_OR(user2.domain, ""), "\"", "\""));
 
 	reg->transport = transport;
+	reg->timeout = reg->expire = -1;
 	reg->portno = portnum;
 	reg->regdomainport = domainport;
 	reg->callid_valid = FALSE;
@@ -274,7 +271,6 @@ int sip_parse_register_line(struct sip_registry *reg, int default_expiry, const
 	return 0;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(sip_parse_register_line_test)
 {
 	int res = AST_TEST_PASS;
@@ -318,7 +314,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "")           ||
 		strcmp(reg->secret, "")             ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -347,7 +343,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -376,7 +372,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -405,7 +401,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -434,7 +430,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_TCP ||
+		reg->transport != AST_TRANSPORT_TCP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -463,7 +459,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_TLS ||
+		reg->transport != AST_TRANSPORT_TLS ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != 111 ||
@@ -492,7 +488,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "peer")           ||
-		reg->transport != SIP_TRANSPORT_TCP ||
+		reg->transport != AST_TRANSPORT_TCP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != 111 ||
@@ -521,7 +517,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "peer")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != 111 ||
@@ -591,7 +587,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "authuser")           ||
 		strcmp(reg->secret, "pass")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -620,7 +616,7 @@ AST_TEST_DEFINE(sip_parse_register_line_test)
 		strcmp(reg->authuser, "")           ||
 		strcmp(reg->secret, "")         ||
 		strcmp(reg->peername, "")           ||
-		reg->transport != SIP_TRANSPORT_UDP ||
+		reg->transport != AST_TRANSPORT_UDP ||
 		reg->timeout != -1                  ||
 		reg->expire != -1                   ||
 		reg->refresh != default_expiry ||
@@ -644,9 +640,8 @@ alloc_fail:
 	ast_test_status_update(test, "Out of memory. \n");
 	return res;
 }
-#endif
 
-int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum sip_transport *transport)
+int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum ast_transport *transport)
 {
 	char *port;
 
@@ -658,11 +653,11 @@ int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum s
 		*hostname += 3;
 
 		if (!strncasecmp(line, "tcp", 3)) {
-			*transport = SIP_TRANSPORT_TCP;
+			*transport = AST_TRANSPORT_TCP;
 		} else if (!strncasecmp(line, "tls", 3)) {
-			*transport = SIP_TRANSPORT_TLS;
+			*transport = AST_TRANSPORT_TLS;
 		} else if (!strncasecmp(line, "udp", 3)) {
-			*transport = SIP_TRANSPORT_UDP;
+			*transport = AST_TRANSPORT_UDP;
 		} else if (lineno) {
 			ast_log(LOG_NOTICE, "'%.3s' is not a valid transport type on line %d of sip.conf. defaulting to udp.\n", line, lineno);
 		} else {
@@ -670,7 +665,7 @@ int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum s
 		}
 	} else {
 		*hostname = line;
-		*transport = SIP_TRANSPORT_UDP;
+		*transport = AST_TRANSPORT_UDP;
 	}
 
 	if ((line = strrchr(*hostname, '@')))
@@ -700,7 +695,7 @@ int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum s
 	}
 
 	if (!port) {
-		if (*transport & SIP_TRANSPORT_TLS) {
+		if (*transport & AST_TRANSPORT_TLS) {
 			*portnum = STANDARD_TLS_PORT;
 		} else {
 			*portnum = STANDARD_SIP_PORT;
@@ -710,13 +705,12 @@ int sip_parse_host(char *line, int lineno, char **hostname, int *portnum, enum s
 	return 0;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(sip_parse_host_line_test)
 {
 	int res = AST_TEST_PASS;
 	char *host;
 	int port;
-	enum sip_transport transport;
+	enum ast_transport transport;
 	char host1[] = "www.blah.com";
 	char host2[] = "tcp://www.blah.com";
 	char host3[] = "tls://10.10.10.10";
@@ -740,7 +734,7 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	sip_parse_host(host1, 1, &host, &port, &transport);
 	if (port != STANDARD_SIP_PORT ||
 			ast_strlen_zero(host) || strcmp(host, "www.blah.com") ||
-			transport != SIP_TRANSPORT_UDP) {
+			transport != AST_TRANSPORT_UDP) {
 		ast_test_status_update(test, "Test 1: simple host failed.\n");
 		res = AST_TEST_FAIL;
 	}
@@ -749,7 +743,7 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	sip_parse_host(host2, 1, &host, &port, &transport);
 	if (port != STANDARD_SIP_PORT ||
 			ast_strlen_zero(host) || strcmp(host, "www.blah.com") ||
-			transport != SIP_TRANSPORT_TCP) {
+			transport != AST_TRANSPORT_TCP) {
 		ast_test_status_update(test, "Test 2: tcp host failed.\n");
 		res = AST_TEST_FAIL;
 	}
@@ -758,7 +752,7 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	sip_parse_host(host3, 1, &host, &port, &transport);
 	if (port != STANDARD_TLS_PORT ||
 			ast_strlen_zero(host) || strcmp(host, "10.10.10.10") ||
-			transport != SIP_TRANSPORT_TLS) {
+			transport != AST_TRANSPORT_TLS) {
 		ast_test_status_update(test, "Test 3: tls host failed. \n");
 		res = AST_TEST_FAIL;
 	}
@@ -767,7 +761,7 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	sip_parse_host(host4, 1, &host, &port, &transport);
 	if (port != 1234 || ast_strlen_zero(host) ||
 			strcmp(host, "10.10.10.10") ||
-			transport != SIP_TRANSPORT_TLS) {
+			transport != AST_TRANSPORT_TLS) {
 		ast_test_status_update(test, "Test 4: tls host with custom port failed.\n");
 		res = AST_TEST_FAIL;
 	}
@@ -776,7 +770,7 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	sip_parse_host(host5, 1, &host, &port, &transport);
 	if (port != 1234 || ast_strlen_zero(host) ||
 			strcmp(host, "10.10.10.10") ||
-			transport != SIP_TRANSPORT_UDP) {
+			transport != AST_TRANSPORT_UDP) {
 		ast_test_status_update(test, "Test 5: simple host with custom port failed.\n");
 		res = AST_TEST_FAIL;
 	}
@@ -790,7 +784,6 @@ AST_TEST_DEFINE(sip_parse_host_line_test)
 	return res;
 
 }
-#endif
 
 /*! \brief Parse the comma-separated nat= option values */
 void sip_parse_nat_option(const char *value, struct ast_flags *mask, struct ast_flags *flags)
@@ -838,7 +831,6 @@ void sip_parse_nat_option(const char *value, struct ast_flags *mask, struct ast_
 	}
 }
 
-#ifdef TEST_FRAMEWORK
 #define TEST_FORCE_RPORT      1 << 0
 #define TEST_COMEDIA          1 << 1
 #define TEST_AUTO_FORCE_RPORT 1 << 2
@@ -909,8 +901,6 @@ AST_TEST_DEFINE(sip_parse_nat_test)
 
 	return res;
 }
-#endif
-
 /*! \brief SIP test registration */
 void sip_config_parser_register_tests(void)
 {
