@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 426863 $")
 
 #include "include/sip.h"
 #include "include/sip_utils.h"
@@ -220,7 +220,7 @@ int parse_uri_full(char *uri, const char *scheme, char **user, char **pass,
 	return error;
 }
 
-#ifdef TEST_FRAMEWORK
+
 AST_TEST_DEFINE(sip_parse_uri_full_test)
 {
 	int res = AST_TEST_PASS;
@@ -429,7 +429,7 @@ AST_TEST_DEFINE(sip_parse_uri_full_test)
 
 	return res;
 }
-#endif
+
 
 int parse_uri(char *uri, const char *scheme, char **user, char **pass,
 	      char **hostport, char **transport) {
@@ -445,7 +445,6 @@ int parse_uri(char *uri, const char *scheme, char **user, char **pass,
 	return ret;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(sip_parse_uri_test)
 {
 	int res = AST_TEST_PASS;
@@ -591,7 +590,6 @@ AST_TEST_DEFINE(sip_parse_uri_test)
 
 	return res;
 }
-#endif
 
 /*! \brief  Get caller id name from SIP headers, copy into output buffer
  *
@@ -722,7 +720,6 @@ const char *get_calleridname(const char *input, char *output, size_t outputsize)
 	return input;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(get_calleridname_test)
 {
 	int res = AST_TEST_PASS;
@@ -805,7 +802,6 @@ AST_TEST_DEFINE(get_calleridname_test)
 
 	return res;
 }
-#endif
 
 int get_name_and_number(const char *hdr, char **name, char **number)
 {
@@ -847,7 +843,6 @@ int get_name_and_number(const char *hdr, char **name, char **number)
 	return 0;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(get_name_and_number_test)
 {
 	int res = AST_TEST_PASS;
@@ -952,7 +947,6 @@ AST_TEST_DEFINE(get_name_and_number_test)
 
 	return res;
 }
-#endif
 
 int get_in_brackets_const(const char *src,const char **start,int *length)
 {
@@ -1085,7 +1079,6 @@ char *get_in_brackets(char *tmp)
 	return out;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(get_in_brackets_test)
 {
 	int res = AST_TEST_PASS;
@@ -1162,7 +1155,7 @@ AST_TEST_DEFINE(get_in_brackets_test)
 
 	return res;
 }
-#endif
+
 
 int parse_name_andor_addr(char *uri, const char *scheme, char **name,
 			  char **user, char **pass, char **hostport,
@@ -1208,7 +1201,6 @@ int parse_name_andor_addr(char *uri, const char *scheme, char **name,
 	return parse_uri_full(uri, scheme, user, pass, hostport, params, headers, residue2);
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(parse_name_andor_addr_test)
 {
 	int res = AST_TEST_PASS;
@@ -1338,7 +1330,6 @@ AST_TEST_DEFINE(parse_name_andor_addr_test)
 
 	return res;
 }
-#endif
 
 int get_comma(char *in, char **out)
 {
@@ -1435,7 +1426,6 @@ int parse_contact_header(char *contactheader, struct contactliststruct *contactl
 	return last;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(parse_contact_header_test)
 {
 	int res = AST_TEST_PASS;
@@ -1581,7 +1571,6 @@ AST_TEST_DEFINE(parse_contact_header_test)
 
 	return res;
 }
-#endif
 
 /*!
  * \brief Parse supported header in incoming packet
@@ -1591,9 +1580,9 @@ AST_TEST_DEFINE(parse_contact_header_test)
  * item is found that is not supported, it is copied to the unsupported
  * out buffer.
  *
- * \param option list
+ * \param options list
  * \param unsupported out buffer (optional)
- * \param unsupported out buffer length (optional)
+ * \param unsupported_len out buffer length (optional)
  */
 unsigned int parse_sip_options(const char *options, char *unsupported, size_t unsupported_len)
 {
@@ -1669,7 +1658,6 @@ unsigned int parse_sip_options(const char *options, char *unsupported, size_t un
 	return profile;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(sip_parse_options_test)
 {
 	int res = AST_TEST_PASS;
@@ -1807,7 +1795,6 @@ AST_TEST_DEFINE(sip_parse_options_test)
 
 	return res;
 }
-#endif
 
 /*! \brief helper routine for sip_uri_cmp to compare URI parameters
  *
@@ -2162,7 +2149,6 @@ int sip_uri_cmp(const char *input1, const char *input2)
 #define URI_CMP_MATCH 0
 #define URI_CMP_NOMATCH 1
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(sip_uri_cmp_test)
 {
 	static const struct {
@@ -2279,7 +2265,6 @@ AST_TEST_DEFINE(sip_uri_cmp_test)
 
 	return test_res;
 }
-#endif
 
 void free_via(struct sip_via *v)
 {
@@ -2338,7 +2323,7 @@ struct sip_via *parse_via(const char *header)
 
 	/* store the port, we have to handle ipv6 addresses containing ':'
 	 * characters gracefully */
-	if (((parm = strchr(v->sent_by, ']')) && *(++parm) == ':') || (!(parm = strchr(v->sent_by, ']')) && (parm = strchr(v->sent_by, ':')))) {
+	if (((parm = strchr(v->sent_by, ']')) && *(++parm) == ':') || (parm = strchr(v->sent_by, ':'))) {
 		char *endptr;
 
 		v->port = strtol(++parm, &endptr, 10);
@@ -2366,7 +2351,6 @@ struct sip_via *parse_via(const char *header)
 	return v;
 }
 
-#ifdef TEST_FRAMEWORK
 AST_TEST_DEFINE(parse_via_test)
 {
 	int res = AST_TEST_PASS;
@@ -2544,7 +2528,6 @@ AST_TEST_DEFINE(parse_via_test)
 	}
 	return res;
 }
-#endif
 
 void sip_request_parser_register_tests(void)
 {
@@ -2559,7 +2542,6 @@ void sip_request_parser_register_tests(void)
 	AST_TEST_REGISTER(sip_uri_cmp_test);
 	AST_TEST_REGISTER(parse_via_test);
 }
-
 void sip_request_parser_unregister_tests(void)
 {
 	AST_TEST_UNREGISTER(sip_parse_uri_test);
