@@ -30,29 +30,16 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 428245 $")
 
 #include "include/sip.h"
 #include "include/security_events.h"
 
 /*! \brief Determine transport type used to receive request*/
 
-static enum ast_security_event_transport_type security_event_get_transport(const struct sip_pvt *p)
+static enum ast_transport security_event_get_transport(const struct sip_pvt *p)
 {
-	int res = 0;
-
-	switch (p->socket.type) {
-	case SIP_TRANSPORT_UDP:
-		return AST_SECURITY_EVENT_TRANSPORT_UDP;
-	case SIP_TRANSPORT_TCP:
-	case SIP_TRANSPORT_WS:
-		return AST_SECURITY_EVENT_TRANSPORT_TCP;
-	case SIP_TRANSPORT_TLS:
-	case SIP_TRANSPORT_WSS:
-		return AST_SECURITY_EVENT_TRANSPORT_TLS;
-	}
-
-	return res;
+	return p->socket.type;
 }
 
 void sip_report_invalid_peer(const struct sip_pvt *p)
@@ -283,8 +270,7 @@ void sip_report_inval_transport(const struct sip_pvt *p, const char *transport)
         ast_security_event_report(AST_SEC_EVT(&inval_transport));
 }
 
-int sip_report_security_event(const char *peer, struct ast_sockaddr *addr, const struct sip_pvt *p,
-			      const struct sip_request *req, const int res) {
+int sip_report_security_event(const struct sip_pvt *p, const struct sip_request *req, const int res) {
 
 	struct sip_peer *peer_report;
 	enum check_auth_result res_report = res;
@@ -302,7 +288,7 @@ int sip_report_security_event(const char *peer, struct ast_sockaddr *addr, const
 		[K_LAST]  = { NULL, NULL}
 	};
 
-	peer_report = sip_find_peer(peer, addr, TRUE, FINDPEERS, FALSE, p->socket.type);
+	peer_report = sip_find_peer(p->exten, NULL, TRUE, FINDPEERS, FALSE, 0);
 
 	switch(res_report) {
 	case AUTH_DONT_KNOW:
