@@ -26,7 +26,7 @@
  *
  * \ingroup codecs
  *
- * \extref The Speex library - http://www.speex.org
+ * The Speex library - http://www.speex.org
  *
  */
 
@@ -39,12 +39,13 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 373330 $")
 
 #include <speex/speex.h>
 
 /* We require a post 1.1.8 version of Speex to enable preprocessing
-   and better type handling */   
+ * and better type handling
+ */   
 #ifdef _SPEEX_TYPES_H
 #include <speex/speex_preprocess.h>
 #endif
@@ -53,8 +54,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/module.h"
 #include "asterisk/config.h"
 #include "asterisk/utils.h"
-#include "asterisk/frame.h"
-#include "asterisk/linkedlists.h"
 
 /* codec variables */
 static int quality = 3;
@@ -260,16 +259,15 @@ static int lintospeex_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
 static struct ast_frame *lintospeex_frameout(struct ast_trans_pvt *pvt)
 {
 	struct speex_coder_pvt *tmp = pvt->pvt;
-	struct ast_frame *result = NULL;
-	struct ast_frame *last = NULL;
-	int samples = 0; /* output samples */
+	int is_speech=1;
+	int datalen = 0;	/* output bytes */
+	int samples = 0;	/* output samples */
 
+	/* We can't work on anything less than a frame in size */
+	if (pvt->samples < tmp->framesize)
+		return NULL;
+	speex_bits_reset(&tmp->bits);
 	while (pvt->samples >= tmp->framesize) {
-		struct ast_frame *current;
-		int is_speech = 1;
-
-		speex_bits_reset(&tmp->bits);
-
 #ifdef _SPEEX_TYPES_H
 		/* Preprocess audio */
 		if (preproc)
@@ -295,44 +293,32 @@ static struct ast_frame *lintospeex_frameout(struct ast_trans_pvt *pvt)
 #endif
 		samples += tmp->framesize;
 		pvt->samples -= tmp->framesize;
+	}
 
-		/* Use AST_FRAME_CNG to signify the start of any silence period */
-		if (is_speech) {
-			int datalen = 0; /* output bytes */
+	/* Move the data at the end of the buffer to the front */
+	if (pvt->samples)
+		memmove(tmp->buf, tmp->buf + samples, pvt->samples * 2);
 
-			tmp->silent_state = 0;
-			/* Terminate bit stream */
-			speex_bits_pack(&tmp->bits, 15, 5);
-			datalen = speex_bits_write(&tmp->bits, pvt->outbuf.c, pvt->t->buf_size);
-			current = ast_trans_frameout(pvt, datalen, tmp->framesize);
-		} else if (tmp->silent_state) {
-			current = NULL;
+	/* Use AST_FRAME_CNG to signify the start of any silence period */
+	if (is_speech) {
+		tmp->silent_state = 0;
+	} else {
+		if (tmp->silent_state) {
+			return NULL;
 		} else {
 			tmp->silent_state = 1;
 			speex_bits_reset(&tmp->bits);
 			memset(&pvt->f, 0, sizeof(pvt->f));
 			pvt->f.frametype = AST_FRAME_CNG;
-			pvt->f.samples = tmp->framesize;
+			pvt->f.samples = samples;
 			/* XXX what now ? format etc... */
-			current = ast_frisolate(&pvt->f);
 		}
-
-		if (!current) {
-			continue;
-		} else if (last) {
-			AST_LIST_NEXT(last, frame_list) = current;
-		} else {
-			result = current;
-		}
-		last = current;
 	}
 
-	/* Move the data at the end of the buffer to the front */
-	if (samples) {
-		memmove(tmp->buf, tmp->buf + samples, pvt->samples * 2);
-	}
-
-	return result;
+	/* Terminate bit stream */
+	speex_bits_pack(&tmp->bits, 15, 5);
+	datalen = speex_bits_write(&tmp->bits, pvt->outbuf.c, pvt->t->buf_size);
+	return ast_trans_frameout(pvt, datalen, samples);
 }
 
 static void speextolin_destroy(struct ast_trans_pvt *arg)
