@@ -33,7 +33,7 @@
  
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 396412 $")
 
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
@@ -73,14 +73,14 @@ struct wavg_desc {
 #define ltohs(b) (b)
 #else
 #if __BYTE_ORDER == __BIG_ENDIAN
-#define htoll(b) \
-	(((((b)      ) & 0xFF) << 24) | \
-	((( (b) >>  8) & 0xFF) << 16) | \
-	((( (b) >> 16) & 0xFF) <<  8) | \
-	((( (b) >> 24) & 0xFF)      ))
+#define htoll(b)  \
+          (((((b)      ) & 0xFF) << 24) | \
+	       ((((b) >>  8) & 0xFF) << 16) | \
+		   ((((b) >> 16) & 0xFF) <<  8) | \
+		   ((((b) >> 24) & 0xFF)      ))
 #define htols(b) \
-	(((((b)      ) & 0xFF) <<  8) | \
-	((( (b) >>  8) & 0xFF)      ))
+          (((((b)      ) & 0xFF) << 8) | \
+		   ((((b) >> 8) & 0xFF)      ))
 #define ltohl(b) htoll(b)
 #define ltohs(b) htols(b)
 #else
@@ -104,9 +104,7 @@ static int check_header(FILE *f)
 		ast_log(LOG_WARNING, "Read failed (size)\n");
 		return -1;
 	}
-#if __BYTE_ORDER == __BIG_ENDIAN
 	size = ltohl(size);
-#endif
 	if (fread(&formtype, 1, 4, f) != 4) {
 		ast_log(LOG_WARNING, "Read failed (formtype)\n");
 		return -1;
@@ -395,6 +393,17 @@ static int wav_rewrite(struct ast_filestream *s, const char *comment)
 	return 0;
 }
 
+static void wav_close(struct ast_filestream *s)
+{
+	if (s->mode == O_RDONLY) {
+		return;
+	}
+
+	if (s->filename) {
+		update_header(s->f);
+	}
+}
+
 static struct ast_frame *wav_read(struct ast_filestream *s, int *whennext)
 {
 	/* Send a frame from the file to the appropriate channel */
@@ -470,7 +479,6 @@ static int wav_write(struct ast_filestream *s, struct ast_frame *f)
 			ast_log(LOG_WARNING, "Bad write (%d/65): %s\n", res, strerror(errno));
 			return -1;
 		}
-		update_header(s->f); /* XXX inefficient! */
 	}
 	return 0;
 }
@@ -562,6 +570,7 @@ static struct ast_format_def wav49_f = {
 	.trunc = wav_trunc,
 	.tell = wav_tell,
 	.read = wav_read,
+	.close = wav_close,
 	.buf_size = 2*GSM_FRAME_SIZE + AST_FRIENDLY_OFFSET,
 	.desc_size = sizeof(struct wavg_desc),
 };
