@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 411315 $")
 
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
@@ -40,11 +40,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/callerid.h"
 
 /*
- * The CALLERID(pres) datatype is shorthand for getting/setting the
- * combined value of name-pres and num-pres.  Some channel drivers
- * don't make a distinction, so it makes sense to only use one property
- * to get/set it.  The same applies to CONNECTEDLINE(pres),
- * REDIRECTING(orig-pres), REDIRECTING(from-pres) and REDIRECTING(to-pres).
+ * Do not document the CALLERID(pres) datatype.
+ * The name and number now have their own presentation value.  The pres
+ * option will simply live on as a historical relic with as best
+ * as can be managed backward compatible meaning.
  *
  * Do not document the CALLERID(ton) datatype.
  * It is an alias for num-plan.
@@ -56,6 +55,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
  * It has turned out to not be needed.  The source value is really
  * only useful as a possible tracing aid.
  *
+ * Do not document the CONNECTEDLINE(pres) datatype.
+ * The name and number now have their own presentation value.  The pres
+ * option will simply live on as a historical relic with as best
+ * as can be managed backward compatible meaning.
+ *
  * Do not document the CONNECTEDLINE(ton) datatype.
  * It is an alias for num-plan.
  *
@@ -65,6 +69,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
  * they are active at the same time.  The plain pres option will simply
  * live on as a historical relic.
  *
+ * Do not document the REDIRECTING(orig-pres), REDIRECTING(from-pres),
+ * or REDIRECTING(to-pres) datatypes.
+ * The name and number now have their own presentation value.  The orig-pres,
+ * from-pres, and to-pres options will simply live on as a historical relic
+ * with as best as can be managed backward compatible meaning.
+ *
  * Do not document the REDIRECTING(orig-ton), REDIRECTING(from-ton),
  * or REDIRECTING(to-ton) datatypes.
  * They are aliases for orig-num-plan, from-num-plan, and to-num-plan
@@ -88,7 +98,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name = "num-valid" />
 					<enum name = "num-plan" />
 					<enum name = "num-pres" />
-					<enum name = "pres" />
 					<enum name = "subaddr" />
 					<enum name = "subaddr-valid" />
 					<enum name = "subaddr-type" />
@@ -135,9 +144,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		<description>
 			<para>Gets or sets Caller*ID data on the channel. Uses channel callerid by
 			default or optional callerid, if specified.</para>
-			<para>The <replaceable>pres</replaceable> field gets/sets a combined value
-			for <replaceable>name-pres</replaceable> and
-			<replaceable>num-pres</replaceable>.</para>
 			<para>The allowable values for the <replaceable>name-charset</replaceable>
 			field are the following:</para>
 			<enumlist>
@@ -162,8 +168,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		<description>
 			<para>Gets or sets Caller*ID presentation on the channel.
 			This function is deprecated in favor of CALLERID(num-pres)
-			and CALLERID(name-pres) or CALLERID(pres) to get/set both
-			at once.
+			and CALLERID(name-pres).
 			The following values are valid:</para>
 			<enumlist>
 				<enum name="allowed_not_screened">
@@ -213,7 +218,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name = "num-valid" />
 					<enum name = "num-plan" />
 					<enum name = "num-pres" />
-					<enum name = "pres" />
 					<enum name = "subaddr" />
 					<enum name = "subaddr-valid" />
 					<enum name = "subaddr-type" />
@@ -242,9 +246,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		</syntax>
 		<description>
 			<para>Gets or sets Connected Line data on the channel.</para>
-			<para>The <replaceable>pres</replaceable> field gets/sets a combined value
-			for <replaceable>name-pres</replaceable> and
-			<replaceable>num-pres</replaceable>.</para>
 			<para>The allowable values for the <replaceable>name-charset</replaceable>
 			field are the following:</para>
 			<enumlist>
@@ -278,7 +279,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name = "orig-num-valid" />
 					<enum name = "orig-num-plan" />
 					<enum name = "orig-num-pres" />
-					<enum name = "orig-pres" />
 					<enum name = "orig-subaddr" />
 					<enum name = "orig-subaddr-valid" />
 					<enum name = "orig-subaddr-type" />
@@ -294,7 +294,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name = "from-num-valid" />
 					<enum name = "from-num-plan" />
 					<enum name = "from-num-pres" />
-					<enum name = "from-pres" />
 					<enum name = "from-subaddr" />
 					<enum name = "from-subaddr-valid" />
 					<enum name = "from-subaddr-type" />
@@ -309,7 +308,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name = "to-num-valid" />
 					<enum name = "to-num-plan" />
 					<enum name = "to-num-pres" />
-					<enum name = "to-pres" />
 					<enum name = "to-subaddr" />
 					<enum name = "to-subaddr-valid" />
 					<enum name = "to-subaddr-type" />
@@ -368,11 +366,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		</syntax>
 		<description>
 			<para>Gets or sets Redirecting data on the channel.</para>
-			<para>The <replaceable>orig-pres</replaceable>,
-			<replaceable>from-pres</replaceable> and <replaceable>to-pres</replaceable>
-			fields get/set a combined value for the corresponding
-			<replaceable>...-name-pres</replaceable> and <replaceable>...-num-pres</replaceable>
-			fields.</para>
 			<para>The allowable values for the <replaceable>reason</replaceable>
 			and <replaceable>orig-reason</replaceable> fields are the following:</para>
 			<enumlist>
@@ -1159,9 +1152,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 		ast_channel_redirecting(chan)->from.number.valid = 1;
 		ast_free(ast_channel_redirecting(chan)->from.number.str);
 		ast_channel_redirecting(chan)->from.number.str = ast_strdup(value);
-		if (ast_channel_cdr(chan)) {
-			ast_cdr_setcid(ast_channel_cdr(chan), chan);
-		}
 	} else if (!strcasecmp("dnid", member.argv[0])) {
 		ast_party_dialed_set_init(&dialed, ast_channel_dialed(chan));
 		if (member.argc == 1) {
@@ -1179,9 +1169,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 				dialed.number.str = ast_strdup(value);
 				ast_trim_blanks(dialed.number.str);
 				ast_party_dialed_set(ast_channel_dialed(chan), &dialed);
-				if (ast_channel_cdr(chan)) {
-					ast_cdr_setcid(ast_channel_cdr(chan), chan);
-				}
 			} else if (member.argc == 3 && !strcasecmp("plan", member.argv[2])) {
 				/* dnid-num-plan */
 				val = ast_strdupa(value);
@@ -1189,9 +1176,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 
 				if (('0' <= val[0]) && (val[0] <= '9')) {
 					ast_channel_dialed(chan)->number.plan = atoi(val);
-					if (ast_channel_cdr(chan)) {
-						ast_cdr_setcid(ast_channel_cdr(chan), chan);
-					}
 				} else {
 					ast_log(LOG_ERROR,
 						"Unknown type-of-number/numbering-plan '%s', value unchanged\n", val);
@@ -1209,9 +1193,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 			switch (status) {
 			case ID_FIELD_VALID:
 				ast_party_dialed_set(ast_channel_dialed(chan), &dialed);
-				if (ast_channel_cdr(chan)) {
-					ast_cdr_setcid(ast_channel_cdr(chan), chan);
-				}
 				break;
 			case ID_FIELD_INVALID:
 				break;
@@ -1229,9 +1210,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 
 		if (('0' <= val[0]) && (val[0] <= '9')) {
 			ast_channel_caller(chan)->ani2 = atoi(val);
-			if (ast_channel_cdr(chan)) {
-				ast_cdr_setcid(ast_channel_cdr(chan), chan);
-			}
 		} else {
 			ast_log(LOG_ERROR, "Unknown callerid ani2 '%s', value unchanged\n", val);
 		}
@@ -1246,9 +1224,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 		switch (status) {
 		case ID_FIELD_VALID:
 			ast_party_caller_set(ast_channel_caller(chan), &caller, NULL);
-			if (ast_channel_cdr(chan)) {
-				ast_cdr_setcid(ast_channel_cdr(chan), chan);
-			}
 			break;
 		case ID_FIELD_INVALID:
 			break;
@@ -1263,9 +1238,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 		switch (status) {
 		case ID_FIELD_VALID:
 			ast_party_caller_set(ast_channel_caller(chan), &caller, NULL);
-			if (ast_channel_cdr(chan)) {
-				ast_cdr_setcid(ast_channel_cdr(chan), chan);
-			}
 			break;
 		case ID_FIELD_INVALID:
 			break;
@@ -1280,9 +1252,6 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 		switch (status) {
 		case ID_FIELD_VALID:
 			ast_channel_set_caller_event(chan, &caller, NULL);
-			if (ast_channel_cdr(chan)) {
-				ast_cdr_setcid(ast_channel_cdr(chan), chan);
-			}
 			break;
 		case ID_FIELD_INVALID:
 			break;
@@ -1510,7 +1479,7 @@ static int redirecting_read(struct ast_channel *chan, const char *cmd, char *dat
 	if (!strcasecmp("orig", member.argv[0])) {
 		if (member.argc == 2 && !strcasecmp("reason", member.argv[1])) {
 			ast_copy_string(buf,
-				ast_redirecting_reason_name(ast_redirecting->orig_reason), len);
+				ast_redirecting_reason_name(&ast_redirecting->orig_reason), len);
 		} else {
 			status = party_id_read(buf, len, member.argc - 1, member.argv + 1,
 				&ast_redirecting->orig);
@@ -1554,7 +1523,7 @@ static int redirecting_read(struct ast_channel *chan, const char *cmd, char *dat
 			ast_named_caller_presentation(
 				ast_party_id_presentation(&ast_redirecting->from)), len);
 	} else if (member.argc == 1 && !strcasecmp("reason", member.argv[0])) {
-		ast_copy_string(buf, ast_redirecting_reason_name(ast_redirecting->reason), len);
+		ast_copy_string(buf, ast_redirecting_reason_name(&ast_redirecting->reason), len);
 	} else if (member.argc == 1 && !strcasecmp("count", member.argv[0])) {
 		snprintf(buf, len, "%d", ast_redirecting->count);
 	} else if (1 < member.argc && !strcasecmp("priv", member.argv[0])) {
@@ -1676,10 +1645,16 @@ static int redirecting_write(struct ast_channel *chan, const char *cmd, char *da
 			}
 
 			if (reason < 0) {
-				ast_log(LOG_ERROR,
-					"Unknown redirecting orig reason '%s', value unchanged\n", val);
+			/* The argument passed into the function does not correspond to a pre-defined
+			 * reason, so we can just set the reason string to what was given and set the
+			 * code to be unknown
+			 */
+				redirecting.orig_reason.code = AST_REDIRECTING_REASON_UNKNOWN;
+				redirecting.orig_reason.str = val;
+				set_it(chan, &redirecting, NULL);
 			} else {
-				redirecting.orig_reason = reason;
+				redirecting.orig_reason.code = reason;
+				redirecting.orig_reason.str = "";
 				set_it(chan, &redirecting, NULL);
 			}
 		} else {
@@ -1759,9 +1734,16 @@ static int redirecting_write(struct ast_channel *chan, const char *cmd, char *da
 		}
 
 		if (reason < 0) {
-			ast_log(LOG_ERROR, "Unknown redirecting reason '%s', value unchanged\n", val);
+			/* The argument passed into the function does not correspond to a pre-defined
+			 * reason, so we can just set the reason string to what was given and set the
+			 * code to be unknown
+			 */
+			redirecting.reason.code = AST_REDIRECTING_REASON_UNKNOWN;
+			redirecting.reason.str = val;
+			set_it(chan, &redirecting, NULL);
 		} else {
-			redirecting.reason = reason;
+			redirecting.reason.code = reason;
+			redirecting.reason.str = "";
 			set_it(chan, &redirecting, NULL);
 		}
 	} else if (member.argc == 1 && !strcasecmp("count", member.argv[0])) {
