@@ -30,19 +30,22 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include <regex.h>
 #include <ctype.h>
 
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
+#include "asterisk/bridge.h"
 #include "asterisk/pbx.h"
 #include "asterisk/utils.h"
 #include "asterisk/app.h"
 #include "asterisk/indications.h"
 #include "asterisk/stringfields.h"
 #include "asterisk/global_datastores.h"
+#include "asterisk/bridge_basic.h"
+#include "asterisk/bridge_after.h"
 
 /*** DOCUMENTATION
 	<function name="CHANNELS" language="en_US">
@@ -100,6 +103,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name="audiowriteformat">
 						<para>R/O format currently being written.</para>
 					</enum>
+					<enum name="dtmf_features">
+						<para>R/W The channel's DTMF bridge features.
+						May include one or more of 'T' 'K' 'H' 'W' and 'X' in a similar manner to options
+						in the <literal>Dial</literal> application. When setting it, the features string
+						must be all upper case.</para>
+					</enum>
 					<enum name="callgroup">
 						<para>R/W numeric call pickup groups that this channel is a member.</para>
 					</enum>
@@ -118,6 +127,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name="checkhangup">
 						<para>R/O Whether the channel is hanging up (1/0)</para>
 					</enum>
+					<enum name="after_bridge_goto">
+						<para>R/W the parseable goto string indicating where the channel is
+						expected to return to in the PBX after exiting the next bridge it joins
+						on the condition that it doesn't hang up. The parseable goto string uses
+						the same syntax as the <literal>Goto</literal> application.</para>
+					</enum>
 					<enum name="hangup_handler_pop">
 						<para>W/O Replace the most recently added hangup handler
 						with a new hangup handler on the channel if supplied.  The
@@ -265,6 +280,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 						<para>   Defaults to <literal>audio</literal> if unspecified.</para>
 					</enum>
 				</enumlist>
+				<xi:include xpointer="xpointer(/docs/info[@name='PJSIPCHANNEL'])" />
 				<para><emphasis>chan_iax2</emphasis> provides the following additional options:</para>
 				<enumlist>
 					<enum name="osptoken">
@@ -434,9 +450,13 @@ static int func_channel_read(struct ast_channel *chan, const char *function,
 		ast_copy_string(buf, ast_channel_trace_is_enabled(chan) ? "1" : "0", len);
 		ast_channel_unlock(chan);
 #endif
-	} else if (!strcasecmp(data, "tonezone") && ast_channel_zone(chan))
+	} else if (!strcasecmp(data, "tonezone") && ast_channel_zone(chan)) {
 		locked_copy_string(chan, buf, ast_channel_zone(chan)->country, len);
-	else if (!strcasecmp(data, "language"))
+	} else if (!strcasecmp(data, "dtmf_features")) {
+		if (ast_bridge_features_ds_get_string(chan, buf, len)) {
+			buf[0] = '\0';
+		}
+	} else if (!strcasecmp(data, "language"))
 		locked_copy_string(chan, buf, ast_channel_language(chan), len);
 	else if (!strcasecmp(data, "musicclass"))
 		locked_copy_string(chan, buf, ast_channel_musicclass(chan), len);
@@ -481,11 +501,11 @@ static int func_channel_read(struct ast_channel *chan, const char *function,
 		}
 		ast_channel_unlock(chan);
 	} else if (!strcasecmp(data, "peer")) {
-		struct ast_channel *p;
+		RAII_VAR(struct ast_channel *, p, NULL, ast_channel_cleanup);
 
 		ast_channel_lock(chan);
-		p = ast_bridged_channel(chan);
-		if (p || ast_channel_tech(chan) || ast_channel_cdr(chan)) /* dummy channel? if so, we hid the peer name in the language */
+		p = ast_channel_bridge_peer(chan);
+		if (p || ast_channel_tech(chan)) /* dummy channel? if so, we hid the peer name in the language */
 			ast_copy_string(buf, (p ? ast_channel_name(p) : ""), len);
 		else {
 			/* a dummy channel can still pass along bridged peer info via
@@ -517,9 +537,11 @@ static int func_channel_read(struct ast_channel *chan, const char *function,
 		struct ast_str *tmp_str = ast_str_alloca(1024);
 
 		locked_copy_string(chan, buf,  ast_print_namedgroups(&tmp_str, ast_channel_named_pickupgroups(chan)), len);
+	} else if (!strcasecmp(data, "after_bridge_goto")) {
+		ast_bridge_read_after_goto(chan, buf, len);
 	} else if (!strcasecmp(data, "amaflags")) {
 		ast_channel_lock(chan);
-		snprintf(buf, len, "%d", ast_channel_amaflags(chan));
+		snprintf(buf, len, "%u", ast_channel_amaflags(chan));
 		ast_channel_unlock(chan);
 	} else if (!strncasecmp(data, "secure_bridge_", 14)) {
 		struct ast_datastore *ds;
@@ -558,9 +580,15 @@ static int func_channel_write_real(struct ast_channel *chan, const char *functio
 		locked_string_field_set(chan, accountcode, value);
 	else if (!strcasecmp(data, "userfield"))
 		locked_string_field_set(chan, userfield, value);
-	else if (!strcasecmp(data, "amaflags")) {
+	else if (!strcasecmp(data, "after_bridge_goto")) {
+		if (ast_strlen_zero(value)) {
+			ast_bridge_discard_after_goto(chan);
+		} else {
+			ast_bridge_set_after_go_on(chan, ast_channel_context(chan), ast_channel_exten(chan), ast_channel_priority(chan), value);
+		}
+	} else if (!strcasecmp(data, "amaflags")) {
 		ast_channel_lock(chan);
-		if(isdigit(*value)) {
+		if (isdigit(*value)) {
 			int amaflags;
 			sscanf(value, "%30d", &amaflags);
 			ast_channel_amaflags_set(chan, amaflags);
@@ -605,6 +633,8 @@ static int func_channel_write_real(struct ast_channel *chan, const char *functio
 			ast_channel_unlock(chan);
 			new_zone = ast_tone_zone_unref(new_zone);
 		}
+	} else if (!strcasecmp(data, "dtmf_features")) {
+		ret = ast_bridge_features_ds_set_string(chan, value);
 	} else if (!strcasecmp(data, "callgroup")) {
 		ast_channel_lock(chan);
 		ast_channel_callgroup_set(chan, ast_get_group(value));
