@@ -27,7 +27,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 433173 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413597 $")
 
 #include <sys/stat.h>   /* stat(2) */
 
@@ -561,7 +561,7 @@ static int file_read(struct ast_channel *chan, const char *cmd, char *data, stru
 
 			/* Don't go past the length requested */
 			if (off_i + toappend > offset + length) {
-				toappend = MIN(offset + length - off_i, flength - off_i);
+				toappend = length - off_i;
 			}
 
 			ast_str_append_substr(buf, len, fbuf, toappend);
