@@ -29,6 +29,7 @@
 #include "asterisk/astobj2.h"
 #include "asterisk/silk.h"
 #include "asterisk/celt.h"
+#include "asterisk/opus.h"
 #define AST_FORMAT_ATTR_SIZE 64
 #define AST_FORMAT_INC 100000
 
@@ -101,6 +102,8 @@ enum ast_format_id {
 	AST_FORMAT_SLINEAR192       = 27 + AST_FORMAT_TYPE_AUDIO,
 	AST_FORMAT_SPEEX32          = 28 + AST_FORMAT_TYPE_AUDIO,
 	AST_FORMAT_CELT             = 29 + AST_FORMAT_TYPE_AUDIO,
+	/*! Opus */
+	AST_FORMAT_OPUS             = 30 + AST_FORMAT_TYPE_AUDIO,
 
 	/*! H.261 Video */
 	AST_FORMAT_H261             = 1 + AST_FORMAT_TYPE_VIDEO,
@@ -112,6 +115,8 @@ enum ast_format_id {
 	AST_FORMAT_H264             = 4 + AST_FORMAT_TYPE_VIDEO,
 	/*! MPEG4 Video */
 	AST_FORMAT_MP4_VIDEO        = 5 + AST_FORMAT_TYPE_VIDEO,
+	/*! VP8 */
+	AST_FORMAT_VP8              = 6 + AST_FORMAT_TYPE_VIDEO,
 
 	/*! JPEG Images */
 	AST_FORMAT_JPEG             = 1 + AST_FORMAT_TYPE_IMAGE,
@@ -264,9 +269,9 @@ void ast_format_sdp_generate(const struct ast_format *format, unsigned int paylo
  * with optional format attributes represented by format specific key value pairs.
  *
  * \param format to set
- * \param id, format id to set on format
- * \param set_attributes, are there attributes to set on this format. 0 == false, 1 == True.
- * \param var list of attribute key value pairs, must end with AST_FORMAT_ATTR_END;
+ * \param id format id to set on format
+ * \param set_attributes are there attributes to set on this format. 0 == false, 1 == True.
+ * \param ... var list of attribute key value pairs, must end with AST_FORMAT_ATTR_END;
  *
  * \details Example usage.
  * ast_format_set(format, AST_FORMAT_ULAW, 0); // no capability attributes are needed for ULAW
@@ -290,7 +295,7 @@ struct ast_format *ast_format_set(struct ast_format *format, enum ast_format_id
  * set additional format attributes to the structure.
  *
  * \param format to set
- * \param var list of attribute key value pairs, must end with AST_FORMAT_ATTR_END;
+ * \param ... var list of attribute key value pairs, must end with AST_FORMAT_ATTR_END;
  *
  * \details Example usage.
  * ast_format_set(format, AST_FORMAT_SILK, 0);
@@ -421,7 +426,7 @@ const char* ast_getformatname(const struct ast_format *format);
 /*! \brief Returns a string containing all formats pertaining to an format id.
  * \param buf a buffer for the output string
  * \param size size of buf (bytes)
- * \param format id.
+ * \param id format id.
  * \return The return value is buf.
  */
 char* ast_getformatname_multiple_byid(char *buf, size_t size, enum ast_format_id id);
@@ -471,4 +476,22 @@ int ast_format_is_slinear(const struct ast_format *format);
  * \brief Get the best slinear format id for a given sample rate
  */
 enum ast_format_id ast_format_slin_by_rate(unsigned int rate);
+
+/*!
+ * \since 12
+ * \brief Get the message type used for signaling a format registration
+ *
+ * \retval Stasis message type for format registration
+ * \retval NULL on error
+ */
+struct stasis_message_type *ast_format_register_type(void);
+
+/*!
+ * \since 12
+ * \brief Get the message type used for signaling a format unregistration
+ *
+ * \retval Stasis message type for format unregistration
+ * \retval NULL on error
+ */
+struct stasis_message_type *ast_format_unregister_type(void);
 #endif /* _AST_FORMAT_H */
