@@ -25,6 +25,14 @@
  * provide and some other module related functions.
  */
 
+/*! \li \ref module.h uses the configuration file \ref modules.conf
+ * \addtogroup configuration_file
+ */
+
+/*! \page modules.conf modules.conf
+ * \verbinclude modules.conf.sample
+ */
+
 #ifndef _ASTERISK_MODULE_H
 #define _ASTERISK_MODULE_H
 
@@ -102,17 +110,17 @@ int ast_unload_resource(const char *resource_name, enum ast_module_unload_mode);
  */
 void ast_update_use_count(void);
 
-/*! 
- * \brief Ask for a list of modules, descriptions, and use counts.
+/*!
+ * \brief Ask for a list of modules, descriptions, use counts and status.
  * \param modentry A callback to an updater function.
  * \param like
  *
  * For each of the modules loaded, modentry will be executed with the resource,
  * description, and usecount values of each particular module.
- * 
+ *
  * \return the number of modules loaded
  */
-int ast_update_module_list(int (*modentry)(const char *module, const char *description, int usecnt, const char *like),
+int ast_update_module_list(int (*modentry)(const char *module, const char *description, int usecnt, const char *status, const char *like),
 			   const char *like);
 
 /*!
@@ -269,31 +277,8 @@ void __ast_module_user_hangup_all(struct ast_module *);
 #define ast_module_user_remove(user) __ast_module_user_remove(ast_module_info->self, user)
 #define ast_module_user_hangup_all() __ast_module_user_hangup_all(ast_module_info->self)
 
-struct ast_module *__ast_module_ref(struct ast_module *mod, const char *file, int line, const char *func);
-void __ast_module_shutdown_ref(struct ast_module *mod, const char *file, int line, const char *func);
-void __ast_module_unref(struct ast_module *mod, const char *file, int line, const char *func);
-
-/*!
- * \brief Hold a reference to the module
- * \param mod Module to reference
- * \return mod
- *
- * \note A module reference will prevent the module
- * from being unloaded.
- */
-#define ast_module_ref(mod)           __ast_module_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-/*!
- * \brief Prevent unload of the module before shutdown
- * \param mod Module to hold
- *
- * \note This should not be balanced by a call to ast_module_unref.
- */
-#define ast_module_shutdown_ref(mod)  __ast_module_shutdown_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-/*!
- * \brief Release a reference to the module
- * \param mod Module to release
- */
-#define ast_module_unref(mod)         __ast_module_unref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+struct ast_module *ast_module_ref(struct ast_module *);
+void ast_module_unref(struct ast_module *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 #define AST_MODULE_INFO(keystr, flags_to_set, desc, load_func, unload_func, reload_func, load_pri)	\
@@ -493,6 +478,9 @@ int ast_register_application2(const char *app, int (*execute)(struct ast_channel
  */
 int ast_unregister_application(const char *app);
 
+/*! Macro to safely ref and unref the self module for the current scope */
+#define SCOPED_MODULE_USE(module) \
+	RAII_VAR(struct ast_module *, __self__ ## __LINE__, ast_module_ref(module), ast_module_unref)
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
