@@ -32,7 +32,7 @@
 
 #if defined(__AST_DEBUG_MALLOC)
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 398726 $")
 
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
 #include <stddef.h>
@@ -42,6 +42,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/lock.h"
 #include "asterisk/strings.h"
 #include "asterisk/unaligned.h"
+#include "asterisk/backtrace.h"
 
 /*!
  * The larger the number the faster memory can be freed.
@@ -78,13 +79,13 @@ static FILE *mmlog;
 
 struct ast_region {
 	AST_LIST_ENTRY(ast_region) node;
+	struct ast_bt *bt;
 	size_t len;
 	unsigned int cache;		/* region was allocated as part of a cache pool */
 	unsigned int lineno;
 	enum func_type which;
 	char file[64];
 	char func[40];
-
 	/*!
 	 * \brief Lower guard fence.
 	 *
@@ -141,6 +142,8 @@ enum summary_opts {
 static enum summary_opts atexit_summary;
 /*! Nonzero if the unfreed regions are listed at exit. */
 static int atexit_list;
+/*! Nonzero if the memory allocation backtrace is enabled. */
+static int backtrace_enabled;
 
 #define HASH(a)		(((unsigned long)(a)) % ARRAY_LEN(regions))
 
@@ -182,6 +185,24 @@ void ast_free_ptr(void *ptr)
 	ast_free(ptr);
 }
 
+static void print_backtrace(struct ast_bt *bt)
+{
+	int i = 0;
+	char **strings;
+
+	if (!bt) {
+		return;
+	}
+
+	if ((strings = ast_bt_get_symbols(bt->addresses, bt->num_frames))) {
+		astmm_log("Memory allocation backtrace:\n");
+		for (i = 3; i < bt->num_frames - 2; i++) {
+			astmm_log("#%d: [%p] %s\n", i - 3, bt->addresses[i], strings[i]);
+		}
+		ast_std_free(strings);
+	}
+}
+
 /*!
  * \internal
  *
@@ -216,6 +237,7 @@ static void *__ast_alloc_region(size_t size, const enum func_type which, const c
 	reg->cache = cache;
 	reg->lineno = lineno;
 	reg->which = which;
+	reg->bt = backtrace_enabled ? ast_bt_create() : NULL;
 	ast_copy_string(reg->file, file, sizeof(reg->file));
 	ast_copy_string(reg->func, func, sizeof(reg->func));
 
@@ -287,6 +309,7 @@ static void region_data_check(struct ast_region *reg)
 		if (*pos != FREED_MAGIC) {
 			astmm_log("WARNING: Memory corrupted after free of %p allocated at %s %s() line %d\n",
 				reg->data, reg->file, reg->func, reg->lineno);
+			print_backtrace(reg->bt);
 			my_do_crash();
 			break;
 		}
@@ -346,6 +369,7 @@ static void region_free(struct ast_freed_regions *freed, struct ast_region *reg)
 
 	if (old) {
 		region_data_check(old);
+		old->bt = ast_bt_destroy(old->bt);
 		free(old);
 	}
 }
@@ -405,12 +429,14 @@ static void region_check_fences(struct ast_region *reg)
 	if (*fence != FENCE_MAGIC) {
 		astmm_log("WARNING: Low fence violation of %p allocated at %s %s() line %d\n",
 			reg->data, reg->file, reg->func, reg->lineno);
+		print_backtrace(reg->bt);
 		my_do_crash();
 	}
 	fence = (unsigned int *) (reg->data + reg->len);
 	if (get_unaligned_uint32(fence) != FENCE_MAGIC) {
 		astmm_log("WARNING: High fence violation of %p allocated at %s %s() line %d\n",
 			reg->data, reg->file, reg->func, reg->lineno);
+		print_backtrace(reg->bt);
 		my_do_crash();
 	}
 }
@@ -646,32 +672,6 @@ int __ast_vasprintf(char **strp, const char *fmt, va_list ap, const char *file,
 	return size;
 }
 
-/*!
- * \internal
- * \brief Count the number of bytes in the specified freed region.
- *
- * \param freed Already freed region blocks storage.
- *
- * \note reglock must be locked before calling.
- *
- * \return Number of bytes in freed region.
- */
-static size_t freed_regions_size(struct ast_freed_regions *freed)
-{
-	size_t total_len = 0;
-	int idx;
-	struct ast_region *old;
-
-	for (idx = 0; idx < ARRAY_LEN(freed->regions); ++idx) {
-		old = freed->regions[idx];
-		if (old) {
-			total_len += old->len;
-		}
-	}
-
-	return total_len;
-}
-
 static char *handle_memory_atexit_list(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	switch (cmd) {
@@ -774,54 +774,12 @@ static char *handle_memory_atexit_summary(struct ast_cli_entry *e, int cmd, stru
 	return CLI_SUCCESS;
 }
 
-/*!
- * \internal
- * \brief Common summary output at the end of the memory show commands.
- *
- * \param fd CLI output file descriptor.
- * \param whales_len Accumulated size of free large allocations.
- * \param minnows_len Accumulated size of free small allocations.
- * \param total_len Accumulated size of all current allocations.
- * \param selected_len Accumulated size of the selected allocations.
- * \param cache_len Accumulated size of the allocations that are part of a cache.
- * \param count Number of selected allocations.
- *
- * \return Nothing
- */
-static void print_memory_show_common_stats(int fd,
-	unsigned int whales_len,
-	unsigned int minnows_len,
-	unsigned int total_len,
-	unsigned int selected_len,
-	unsigned int cache_len,
-	unsigned int count)
-{
-	if (cache_len) {
-		ast_cli(fd, "%10u bytes allocated (%u in caches) in %u selected allocations\n\n",
-			selected_len, cache_len, count);
-	} else {
-		ast_cli(fd, "%10u bytes allocated in %u selected allocations\n\n",
-			selected_len, count);
-	}
-
-	ast_cli(fd, "%10u bytes in all allocations\n", total_len);
-	ast_cli(fd, "%10u bytes in deferred free large allocations\n", whales_len);
-	ast_cli(fd, "%10u bytes in deferred free small allocations\n", minnows_len);
-	ast_cli(fd, "%10u bytes in deferred free allocations\n",
-		whales_len + minnows_len);
-	ast_cli(fd, "%10u bytes in all allocations and deferred free allocations\n",
-		total_len + whales_len + minnows_len);
-}
-
 static char *handle_memory_show_allocations(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	const char *fn = NULL;
 	struct ast_region *reg;
 	unsigned int idx;
-	unsigned int whales_len;
-	unsigned int minnows_len;
-	unsigned int total_len = 0;
-	unsigned int selected_len = 0;
+	unsigned int len = 0;
 	unsigned int cache_len = 0;
 	unsigned int count = 0;
 
@@ -855,7 +813,6 @@ static char *handle_memory_show_allocations(struct ast_cli_entry *e, int cmd, st
 	ast_mutex_lock(&reglock);
 	for (idx = 0; idx < ARRAY_LEN(regions); ++idx) {
 		for (reg = regions[idx]; reg; reg = AST_LIST_NEXT(reg, node)) {
-			total_len += reg->len;
 			if (fn && strcasecmp(fn, reg->file)) {
 				continue;
 			}
@@ -866,21 +823,21 @@ static char *handle_memory_show_allocations(struct ast_cli_entry *e, int cmd, st
 				(unsigned int) reg->len, reg->cache ? " (cache)" : "",
 				reg->func, reg->lineno, reg->file);
 
-			selected_len += reg->len;
+			len += reg->len;
 			if (reg->cache) {
 				cache_len += reg->len;
 			}
 			++count;
 		}
 	}
-
-	whales_len = freed_regions_size(&whales);
-	minnows_len = freed_regions_size(&minnows);
 	ast_mutex_unlock(&reglock);
 
-	print_memory_show_common_stats(a->fd,
-		whales_len, minnows_len, total_len,
-		selected_len, cache_len, count);
+	if (cache_len) {
+		ast_cli(a->fd, "%u bytes allocated (%u in caches) in %u allocations\n",
+			len, cache_len, count);
+	} else {
+		ast_cli(a->fd, "%u bytes allocated in %u allocations\n", len, count);
+	}
 
 	return CLI_SUCCESS;
 }
@@ -893,10 +850,7 @@ static char *handle_memory_show_summary(struct ast_cli_entry *e, int cmd, struct
 	int idx;
 	int cmp;
 	struct ast_region *reg;
-	unsigned int whales_len;
-	unsigned int minnows_len;
-	unsigned int total_len = 0;
-	unsigned int selected_len = 0;
+	unsigned int len = 0;
 	unsigned int cache_len = 0;
 	unsigned int count = 0;
 	struct file_summary {
@@ -914,7 +868,7 @@ static char *handle_memory_show_summary(struct ast_cli_entry *e, int cmd, struct
 		e->usage =
 			"Usage: memory show summary [<file>]\n"
 			"       Summarizes heap memory allocations by file, or optionally\n"
-			"       by line if a file is specified.\n";
+			"       by line, if a file is specified.\n";
 		return NULL;
 	case CLI_GENERATE:
 		return NULL;
@@ -929,7 +883,6 @@ static char *handle_memory_show_summary(struct ast_cli_entry *e, int cmd, struct
 	ast_mutex_lock(&reglock);
 	for (idx = 0; idx < ARRAY_LEN(regions); ++idx) {
 		for (reg = regions[idx]; reg; reg = AST_LIST_NEXT(reg, node)) {
-			total_len += reg->len;
 			if (fn) {
 				if (strcasecmp(fn, reg->file)) {
 					continue;
@@ -988,14 +941,11 @@ static char *handle_memory_show_summary(struct ast_cli_entry *e, int cmd, struct
 			++cur->count;
 		}
 	}
-
-	whales_len = freed_regions_size(&whales);
-	minnows_len = freed_regions_size(&minnows);
 	ast_mutex_unlock(&reglock);
 
 	/* Dump the whole list */
 	for (cur = list; cur; cur = cur->next) {
-		selected_len += cur->len;
+		len += cur->len;
 		cache_len += cur->cache_len;
 		count += cur->count;
 		if (cur->cache_len) {
@@ -1017,9 +967,49 @@ static char *handle_memory_show_summary(struct ast_cli_entry *e, int cmd, struct
 		}
 	}
 
-	print_memory_show_common_stats(a->fd,
-		whales_len, minnows_len, total_len,
-		selected_len, cache_len, count);
+	if (cache_len) {
+		ast_cli(a->fd, "%u bytes allocated (%u in caches) in %u allocations\n",
+			len, cache_len, count);
+	} else {
+		ast_cli(a->fd, "%u bytes allocated in %u allocations\n", len, count);
+	}
+
+	return CLI_SUCCESS;
+}
+
+static char *handle_memory_backtrace(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "memory backtrace";
+		e->usage =
+			"Usage: memory backtrace {on|off}\n"
+			"       Enable dumping an allocation backtrace with memory diagnostics.\n"
+			"       Note that saving the backtrace data for each allocation\n"
+			"       can be CPU intensive.\n";
+		return NULL;
+	case CLI_GENERATE:
+		if (a->pos == 2) {
+			const char * const options[] = { "off", "on", NULL };
+
+			return ast_cli_complete(a->word, options, a->n);
+		}
+		return NULL;
+	}
+
+	if (a->argc != 3) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (ast_true(a->argv[2])) {
+		backtrace_enabled = 1;
+	} else if (ast_false(a->argv[2])) {
+		backtrace_enabled = 0;
+	} else {
+		return CLI_SHOWUSAGE;
+	}
+
+	ast_cli(a->fd, "The memory backtrace is: %s\n", backtrace_enabled ? "On" : "Off");
 
 	return CLI_SUCCESS;
 }
@@ -1029,6 +1019,7 @@ static struct ast_cli_entry cli_memory[] = {
 	AST_CLI_DEFINE(handle_memory_atexit_summary, "Enable memory allocations not freed at exit summary."),
 	AST_CLI_DEFINE(handle_memory_show_allocations, "Display outstanding memory allocations"),
 	AST_CLI_DEFINE(handle_memory_show_summary, "Summarize outstanding memory allocations"),
+	AST_CLI_DEFINE(handle_memory_backtrace, "Enable dumping an allocation backtrace with memory diagnostics."),
 };
 
 AST_LIST_HEAD_NOLOCK(region_list, ast_region);
@@ -1502,7 +1493,7 @@ void __ast_mm_init_phase_2(void)
 		ast_log(LOG_ERROR, "Could not open malloc debug log file: %s\n", filename);
 	}
 
-	ast_register_cleanup(mm_atexit_ast);
+	ast_register_atexit(mm_atexit_ast);
 }
 
 #endif	/* defined(__AST_DEBUG_MALLOC) */
