@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 424506 $")
 
 #include <signal.h>
 
@@ -45,8 +45,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #define AST_AUDIOHOOK_SYNC_TOLERANCE 100 /*!< Tolerance in milliseconds for audiohooks synchronization */
 #define AST_AUDIOHOOK_SMALL_QUEUE_TOLERANCE 100 /*!< When small queue is enabled, this is the maximum amount of audio that can remain queued at a time. */
 
-#define DEFAULT_INTERNAL_SAMPLE_RATE 8000
-
 struct ast_audiohook_translate {
 	struct ast_trans_pvt *trans_pvt;
 	struct ast_format format;
@@ -58,17 +56,7 @@ struct ast_audiohook_list {
 	 * variable will be set and the sample rate will
 	 * be preserved during ast_audiohook_write_list()*/
 	int native_slin_compatible;
-
-	/*
-	 * The internal sample rate is used when writing to the audiohook lists and
-	 * should always be set to the highest value between formats and audiohooks.
-	 *
-	 * During audiohook invocation (see write_list) if native slin is turned off
- 	 * the highest sample rate is determined as each audiohook is processed.
-	 * If it is turned on then the audiohooks are also updated with the current
-	 * highest sample rate.
-	*/
-	int list_internal_samp_rate;
+	int list_internal_samp_rate;/*!< Internal sample rate used when writing to the audiohook list */
 
 	struct ast_audiohook_translate in_translate[2];
 	struct ast_audiohook_translate out_translate[2];
@@ -91,15 +79,12 @@ static int audiohook_set_internal_rate(struct ast_audiohook *audiohook, int rate
 	/* Setup the factories that are needed for this audiohook type */
 	switch (audiohook->type) {
 	case AST_AUDIOHOOK_TYPE_SPY:
-		if (reset) {
-			ast_slinfactory_destroy(&audiohook->read_factory);
-		}
-		ast_slinfactory_init_with_format(&audiohook->read_factory, &slin);
-		/* fall through */
 	case AST_AUDIOHOOK_TYPE_WHISPER:
 		if (reset) {
+			ast_slinfactory_destroy(&audiohook->read_factory);
 			ast_slinfactory_destroy(&audiohook->write_factory);
 		}
+		ast_slinfactory_init_with_format(&audiohook->read_factory, &slin);
 		ast_slinfactory_init_with_format(&audiohook->write_factory, &slin);
 		break;
 	default:
@@ -109,9 +94,11 @@ static int audiohook_set_internal_rate(struct ast_audiohook *audiohook, int rate
 }
 
 /*! \brief Initialize an audiohook structure
+ *
  * \param audiohook Audiohook structure
  * \param type
- * \param source
+ * \param source, init_flags
+ *
  * \return Returns 0 on success, -1 on failure
  */
 int ast_audiohook_init(struct ast_audiohook *audiohook, enum ast_audiohook_type type, const char *source, enum ast_audiohook_init_flags init_flags)
@@ -127,7 +114,7 @@ int ast_audiohook_init(struct ast_audiohook *audiohook, enum ast_audiohook_type
 	audiohook->init_flags = init_flags;
 
 	/* initialize internal rate at 8khz, this will adjust if necessary */
-	audiohook_set_internal_rate(audiohook, DEFAULT_INTERNAL_SAMPLE_RATE, 0);
+	audiohook_set_internal_rate(audiohook, 8000, 0);
 
 	/* Since we are just starting out... this audiohook is new */
 	ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_NEW);
@@ -144,8 +131,8 @@ int ast_audiohook_destroy(struct ast_audiohook *audiohook)
 	/* Drop the factories used by this audiohook type */
 	switch (audiohook->type) {
 	case AST_AUDIOHOOK_TYPE_SPY:
-		ast_slinfactory_destroy(&audiohook->read_factory);
 	case AST_AUDIOHOOK_TYPE_WHISPER:
+		ast_slinfactory_destroy(&audiohook->read_factory);
 		ast_slinfactory_destroy(&audiohook->write_factory);
 		break;
 	default:
@@ -362,34 +349,21 @@ static struct ast_frame *audiohook_read_frame_helper(struct ast_audiohook *audio
 {
 	struct ast_frame *read_frame = NULL, *final_frame = NULL;
 	struct ast_format tmp_fmt;
+	int samples_converted;
 
-	/*
-	 * Update the rate if compatibility mode is turned off or if it is
-	 * turned on and the format rate is higher than the current rate.
-	 *
-	 * This makes it so any unnecessary rate switching/resetting does
-	 * not take place and also any associated audiohook_list's internal
-	 * sample rate maintains the highest sample rate between hooks.
-	 */
-	if (!ast_test_flag(audiohook, AST_AUDIOHOOK_COMPATIBLE) ||
-	    (ast_test_flag(audiohook, AST_AUDIOHOOK_COMPATIBLE) &&
-	     ast_format_rate(format) > audiohook->hook_internal_samp_rate)) {
-		audiohook_set_internal_rate(audiohook, ast_format_rate(format), 1);
-	}
-
-	/* If the sample rate of the requested format differs from that of the underlying audiohook
-	 * sample rate determine how many samples we actually need to get from the audiohook. This
-	 * needs to occur as the signed linear factory stores them at the rate of the audiohook.
-	 * We do this by determining the duration of audio they've requested and then determining
-	 * how many samples that would be in the audiohook format.
-	 */
-	if (ast_format_rate(format) != audiohook->hook_internal_samp_rate) {
-		samples = (audiohook->hook_internal_samp_rate / 1000) * (samples / (ast_format_rate(format) / 1000));
+	/* the number of samples requested is based on the format they are requesting.  Inorder
+	 * to process this correctly samples must be converted to our internal sample rate */
+	if (audiohook->hook_internal_samp_rate == ast_format_rate(format)) {
+		samples_converted = samples;
+	} else if (audiohook->hook_internal_samp_rate > ast_format_rate(format)) {
+		samples_converted = samples * (audiohook->hook_internal_samp_rate / (float) ast_format_rate(format));
+	} else {
+		samples_converted = samples * (ast_format_rate(format) / (float) audiohook->hook_internal_samp_rate);
 	}
 
 	if (!(read_frame = (direction == AST_AUDIOHOOK_DIRECTION_BOTH ?
-		audiohook_read_frame_both(audiohook, samples, read_reference, write_reference) :
-		audiohook_read_frame_single(audiohook, samples, direction)))) {
+		audiohook_read_frame_both(audiohook, samples_converted, read_reference, write_reference) :
+		audiohook_read_frame_single(audiohook, samples_converted, direction)))) {
 		return NULL;
 	}
 
@@ -447,22 +421,6 @@ struct ast_frame *ast_audiohook_read_frame_all(struct ast_audiohook *audiohook,
 static void audiohook_list_set_samplerate_compatibility(struct ast_audiohook_list *audiohook_list)
 {
 	struct ast_audiohook *ah = NULL;
-
-	/*
-	 * Anytime the samplerate compatibility is set (attach/remove an audiohook) the
-	 * list's internal sample rate needs to be reset so that the next time processing
-	 * through write_list, if needed, it will get updated to the correct rate.
-	 *
-	 * A list's internal rate always chooses the higher between its own rate and a
-	 * given rate. If the current rate is being driven by an audiohook that wanted a
-	 * higher rate then when this audiohook is removed the list's rate would remain
-	 * at that level when it should be lower, and with no way to lower it since any
-	 * rate compared against it would be lower.
-	 *
-	 * By setting it back to the lowest rate it can recalulate the new highest rate.
-	 */
-	audiohook_list->list_internal_samp_rate = DEFAULT_INTERNAL_SAMPLE_RATE;
-
 	audiohook_list->native_slin_compatible = 1;
 	AST_LIST_TRAVERSE(&audiohook_list->manipulate_list, ah, list) {
 		if (!(ah->init_flags & AST_AUDIOHOOK_MANIPULATE_ALL_RATES)) {
@@ -493,7 +451,7 @@ int ast_audiohook_attach(struct ast_channel *chan, struct ast_audiohook *audioho
 		AST_LIST_HEAD_INIT_NOLOCK(&ast_channel_audiohooks(chan)->whisper_list);
 		AST_LIST_HEAD_INIT_NOLOCK(&ast_channel_audiohooks(chan)->manipulate_list);
 		/* This sample rate will adjust as necessary when writing to the list. */
-		ast_channel_audiohooks(chan)->list_internal_samp_rate = DEFAULT_INTERNAL_SAMPLE_RATE;
+		ast_channel_audiohooks(chan)->list_internal_samp_rate = 8000;
 	}
 
 	/* Drop into respective list */
@@ -505,16 +463,16 @@ int ast_audiohook_attach(struct ast_channel *chan, struct ast_audiohook *audioho
 		AST_LIST_INSERT_TAIL(&ast_channel_audiohooks(chan)->manipulate_list, audiohook, list);
 
 
-	/*
-	 * Initialize the audiohook's rate to the default. If it needs to be,
-	 * it will get updated later.
-	 */
-	audiohook_set_internal_rate(audiohook, DEFAULT_INTERNAL_SAMPLE_RATE, 1);
+	audiohook_set_internal_rate(audiohook, ast_channel_audiohooks(chan)->list_internal_samp_rate, 1);
 	audiohook_list_set_samplerate_compatibility(ast_channel_audiohooks(chan));
 
 	/* Change status over to running since it is now attached */
 	ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_RUNNING);
 
+	if (ast_channel_is_bridged(chan)) {
+		ast_channel_set_unbridged_nolock(chan, 1);
+	}
+
 	ast_channel_unlock(chan);
 
 	return 0;
@@ -555,14 +513,14 @@ int ast_audiohook_detach(struct ast_audiohook *audiohook)
 	return 0;
 }
 
-/*! \brief Detach audiohooks from list and destroy said list
- * \param audiohook_list List of audiohooks
- * \return Returns 0 on success, -1 on failure
- */
-int ast_audiohook_detach_list(struct ast_audiohook_list *audiohook_list)
+void ast_audiohook_detach_list(struct ast_audiohook_list *audiohook_list)
 {
-	int i = 0;
-	struct ast_audiohook *audiohook = NULL;
+	int i;
+	struct ast_audiohook *audiohook;
+
+	if (!audiohook_list) {
+		return;
+	}
 
 	/* Drop any spies */
 	while ((audiohook = AST_LIST_REMOVE_HEAD(&audiohook_list->spy_list, list))) {
@@ -590,8 +548,6 @@ int ast_audiohook_detach_list(struct ast_audiohook_list *audiohook_list)
 
 	/* Free ourselves */
 	ast_free(audiohook_list);
-
-	return 0;
 }
 
 /*! \brief find an audiohook based on its source
@@ -621,15 +577,10 @@ static struct ast_audiohook *find_audiohook_by_source(struct ast_audiohook_list
 	return NULL;
 }
 
-void ast_audiohook_move_by_source(struct ast_channel *old_chan, struct ast_channel *new_chan, const char *source)
+static void audiohook_move(struct ast_channel *old_chan, struct ast_channel *new_chan, struct ast_audiohook *audiohook)
 {
-	struct ast_audiohook *audiohook;
 	enum ast_audiohook_status oldstatus;
 
-	if (!ast_channel_audiohooks(old_chan) || !(audiohook = find_audiohook_by_source(ast_channel_audiohooks(old_chan), source))) {
-		return;
-	}
-
 	/* By locking both channels and the audiohook, we can assure that
 	 * another thread will not have a chance to read the audiohook's status
 	 * as done, even though ast_audiohook_remove signals the trigger
@@ -645,6 +596,48 @@ void ast_audiohook_move_by_source(struct ast_channel *old_chan, struct ast_chann
 	ast_audiohook_unlock(audiohook);
 }
 
+void ast_audiohook_move_by_source(struct ast_channel *old_chan, struct ast_channel *new_chan, const char *source)
+{
+	struct ast_audiohook *audiohook;
+
+	if (!ast_channel_audiohooks(old_chan)) {
+		return;
+	}
+
+	audiohook = find_audiohook_by_source(ast_channel_audiohooks(old_chan), source);
+	if (!audiohook) {
+		return;
+	}
+
+	audiohook_move(old_chan, new_chan, audiohook);
+}
+
+void ast_audiohook_move_all(struct ast_channel *old_chan, struct ast_channel *new_chan)
+{
+	struct ast_audiohook *audiohook;
+	struct ast_audiohook_list *audiohook_list;
+
+	audiohook_list = ast_channel_audiohooks(old_chan);
+	if (!audiohook_list) {
+		return;
+	}
+
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&audiohook_list->spy_list, audiohook, list) {
+		audiohook_move(old_chan, new_chan, audiohook);
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&audiohook_list->whisper_list, audiohook, list) {
+		audiohook_move(old_chan, new_chan, audiohook);
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&audiohook_list->manipulate_list, audiohook, list) {
+		audiohook_move(old_chan, new_chan, audiohook);
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+}
+
 /*! \brief Detach specified source audiohook from channel
  * \param chan Channel to detach from
  * \param source Name of source to detach
@@ -700,6 +693,11 @@ int ast_audiohook_remove(struct ast_channel *chan, struct ast_audiohook *audioho
 
 	audiohook_list_set_samplerate_compatibility(ast_channel_audiohooks(chan));
 	ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_DONE);
+
+	if (ast_channel_is_bridged(chan)) {
+		ast_channel_set_unbridged_nolock(chan, 1);
+	}
+
 	ast_channel_unlock(chan);
 
 	return 0;
@@ -725,6 +723,9 @@ static struct ast_frame *dtmf_audiohook_write_list(struct ast_channel *chan, str
 			ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_DONE);
 			ast_audiohook_unlock(audiohook);
 			audiohook->manipulate_callback(audiohook, NULL, NULL, 0);
+			if (ast_channel_is_bridged(chan)) {
+				ast_channel_set_unbridged_nolock(chan, 1);
+			}
 			continue;
 		}
 		if (ast_test_flag(audiohook, AST_AUDIOHOOK_WANTS_DTMF))
@@ -749,14 +750,14 @@ static struct ast_frame *audiohook_list_translate_to_slin(struct ast_audiohook_l
 	struct ast_format tmp_fmt;
 	enum ast_format_id slin_id;
 
-	/*
-	 * If we are capable of sample rates other that 8khz, update the internal
-	 * audiohook_list's rate and higher sample rate audio arrives. If native
-	 * slin compatibility is turned on all audiohooks in the list will be
-	 * updated as well during read/write processing.
-	 */
-	audiohook_list->list_internal_samp_rate =
-		MAX(ast_format_rate(&frame->subclass.format), audiohook_list->list_internal_samp_rate);
+	/* If we are capable of maintaining doing samplerates other that 8khz, update
+	 * the internal audiohook_list's rate and higher samplerate audio arrives. By
+	 * updating the list's rate, all the audiohooks in the list will be updated as well
+	 * as the are written and read from. */
+	if (audiohook_list->native_slin_compatible) {
+		audiohook_list->list_internal_samp_rate =
+			MAX(ast_format_rate(&frame->subclass.format), audiohook_list->list_internal_samp_rate);
+	}
 
 	slin_id = ast_format_slin_by_rate(audiohook_list->list_internal_samp_rate);
 
@@ -765,18 +766,12 @@ static struct ast_frame *audiohook_list_translate_to_slin(struct ast_audiohook_l
 	}
 
 	if (ast_format_cmp(&frame->subclass.format, &in_translate->format) == AST_FORMAT_CMP_NOT_EQUAL) {
-		struct ast_trans_pvt *new_trans;
-
-		new_trans = ast_translator_build_path(ast_format_set(&tmp_fmt, slin_id, 0), &frame->subclass.format);
-		if (!new_trans) {
-			return NULL;
-		}
-
 		if (in_translate->trans_pvt) {
 			ast_translator_free_path(in_translate->trans_pvt);
 		}
-		in_translate->trans_pvt = new_trans;
-
+		if (!(in_translate->trans_pvt = ast_translator_build_path(ast_format_set(&tmp_fmt, slin_id, 0), &frame->subclass.format))) {
+			return NULL;
+		}
 		ast_format_copy(&in_translate->format, &frame->subclass.format);
 	}
 	if (!(new_frame = ast_translate(in_translate->trans_pvt, frame, 0))) {
@@ -810,36 +805,6 @@ static struct ast_frame *audiohook_list_translate_to_native(struct ast_audiohook
 	return outframe;
 }
 
-/*!
- *\brief Set the audiohook's internal sample rate to the audiohook_list's rate,
- *       but only when native slin compatibility is turned on.
- *
- * \param audiohook_list audiohook_list data object
- * \param audiohook the audiohook to update
- * \param rate the current max internal sample rate
- */
-static void audiohook_list_set_hook_rate(struct ast_audiohook_list *audiohook_list,
-					 struct ast_audiohook *audiohook, int *rate)
-{
-	/* The rate should always be the max between itself and the hook */
-	if (audiohook->hook_internal_samp_rate > *rate) {
-		*rate = audiohook->hook_internal_samp_rate;
-	}
-
-	/*
-	 * If native slin compatibility is turned on then update the audiohook
-	 * with the audiohook_list's current rate. Note, the audiohook's rate is
-	 * set to the audiohook_list's rate and not the given rate. If there is
-	 * a change in rate the hook's rate is changed on its next check.
-	 */
-	if (audiohook_list->native_slin_compatible) {
-		ast_set_flag(audiohook, AST_AUDIOHOOK_COMPATIBLE);
-		audiohook_set_internal_rate(audiohook, audiohook_list->list_internal_samp_rate, 1);
-	} else {
-		ast_clear_flag(audiohook, AST_AUDIOHOOK_COMPATIBLE);
-	}
-}
-
 /*!
  * \brief Pass an AUDIO frame off to be handled by the audiohook core
  *
@@ -870,7 +835,6 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 	int samples;
 	int middle_frame_manipulated = 0;
 	int removed = 0;
-	int internal_sample_rate;
 
 	/* ---Part_1. translate start_frame to SLINEAR if necessary. */
 	if (!(middle_frame = audiohook_list_translate_to_slin(audiohook_list, direction, start_frame))) {
@@ -878,19 +842,6 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 	}
 	samples = middle_frame->samples;
 
-	/*
-	 * While processing each audiohook check to see if the internal sample rate needs
-	 * to be adjusted (it should be the highest rate specified between formats and
-	 * hooks). The given audiohook_list's internal sample rate is then set to the
-	 * updated value before returning.
-	 *
-	 * If slin compatibility mode is turned on then an audiohook's internal sample
-	 * rate is set to its audiohook_list's rate. If an audiohook_list's rate is
-	 * adjusted during this pass then the change is picked up by the audiohooks
-	 * on the next pass.
-	 */
-	internal_sample_rate = audiohook_list->list_internal_samp_rate;
-
 	/* ---Part_2: Send middle_frame to spy and manipulator lists.  middle_frame is guaranteed to be SLINEAR here.*/
 	/* Queue up signed linear frame to each spy */
 	AST_LIST_TRAVERSE_SAFE_BEGIN(&audiohook_list->spy_list, audiohook, list) {
@@ -900,31 +851,37 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 			removed = 1;
 			ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_DONE);
 			ast_audiohook_unlock(audiohook);
+			if (ast_channel_is_bridged(chan)) {
+				ast_channel_set_unbridged_nolock(chan, 1);
+			}
 			continue;
 		}
-		audiohook_list_set_hook_rate(audiohook_list, audiohook, &internal_sample_rate);
+		audiohook_set_internal_rate(audiohook, audiohook_list->list_internal_samp_rate, 1);
 		ast_audiohook_write_frame(audiohook, direction, middle_frame);
 		ast_audiohook_unlock(audiohook);
 	}
 	AST_LIST_TRAVERSE_SAFE_END;
 
 	/* If this frame is being written out to the channel then we need to use whisper sources */
-	if (direction == AST_AUDIOHOOK_DIRECTION_WRITE && !AST_LIST_EMPTY(&audiohook_list->whisper_list)) {
+	if (!AST_LIST_EMPTY(&audiohook_list->whisper_list)) {
 		int i = 0;
 		short read_buf[samples], combine_buf[samples], *data1 = NULL, *data2 = NULL;
 		memset(&combine_buf, 0, sizeof(combine_buf));
 		AST_LIST_TRAVERSE_SAFE_BEGIN(&audiohook_list->whisper_list, audiohook, list) {
+			struct ast_slinfactory *factory = (direction == AST_AUDIOHOOK_DIRECTION_READ ? &audiohook->read_factory : &audiohook->write_factory);
 			ast_audiohook_lock(audiohook);
 			if (audiohook->status != AST_AUDIOHOOK_STATUS_RUNNING) {
 				AST_LIST_REMOVE_CURRENT(list);
 				removed = 1;
 				ast_audiohook_update_status(audiohook, AST_AUDIOHOOK_STATUS_DONE);
 				ast_audiohook_unlock(audiohook);
+				if (ast_channel_is_bridged(chan)) {
+					ast_channel_set_unbridged_nolock(chan, 1);
+				}
 				continue;
 			}
-			audiohook_list_set_hook_rate(audiohook_list, audiohook, &internal_sample_rate);
-			if (ast_slinfactory_available(&audiohook->write_factory) >= samples &&
-			    ast_slinfactory_read(&audiohook->write_factory, read_buf, samples)) {
+			audiohook_set_internal_rate(audiohook, audiohook_list->list_internal_samp_rate, 1);
+			if (ast_slinfactory_available(factory) >= samples && ast_slinfactory_read(factory, read_buf, samples)) {
 				/* Take audio from this whisper source and combine it into our main buffer */
 				for (i = 0, data1 = combine_buf, data2 = read_buf; i < samples; i++, data1++, data2++)
 					ast_slinear_saturated_add(data1, data2);
@@ -950,22 +907,22 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 				ast_audiohook_unlock(audiohook);
 				/* We basically drop all of our links to the manipulate audiohook and prod it to do it's own destructive things */
 				audiohook->manipulate_callback(audiohook, chan, NULL, direction);
+				if (ast_channel_is_bridged(chan)) {
+					ast_channel_set_unbridged_nolock(chan, 1);
+				}
 				continue;
 			}
-			audiohook_list_set_hook_rate(audiohook_list, audiohook, &internal_sample_rate);
-			/*
-			 * Feed in frame to manipulation.
-			 *
-			 * XXX FAILURES ARE IGNORED XXX
-			 * If the manipulation fails then the frame will be returned in its original state.
-			 * Since there are potentially more manipulator callbacks in the list, no action should
-			 * be taken here to exit early.
-			 */
-			audiohook->manipulate_callback(audiohook, chan, middle_frame, direction);
+			audiohook_set_internal_rate(audiohook, audiohook_list->list_internal_samp_rate, 1);
+			/* Feed in frame to manipulation. */
+			if (!audiohook->manipulate_callback(audiohook, chan, middle_frame, direction)) {
+				/* If the manipulation fails then the frame will be returned in its original state.
+				 * Since there are potentially more manipulator callbacks in the list, no action should
+				 * be taken here to exit early. */
+				 middle_frame_manipulated = 1;
+			}
 			ast_audiohook_unlock(audiohook);
 		}
 		AST_LIST_TRAVERSE_SAFE_END;
-		middle_frame_manipulated = 1;
 	}
 
 	/* ---Part_3: Decide what to do with the end_frame (whether to transcode or not) */
@@ -986,12 +943,6 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 	/* Before returning, if an audiohook got removed, reset samplerate compatibility */
 	if (removed) {
 		audiohook_list_set_samplerate_compatibility(audiohook_list);
-	} else {
-		/*
-		 * Set the audiohook_list's rate to the updated rate. Note that if a hook
-		 * was removed then the list's internal rate is reset to the default.
-		 */
-		audiohook_list->list_internal_samp_rate = internal_sample_rate;
 	}
 
 	return end_frame;
@@ -999,13 +950,10 @@ static struct ast_frame *audio_audiohook_write_list(struct ast_channel *chan, st
 
 int ast_audiohook_write_list_empty(struct ast_audiohook_list *audiohook_list)
 {
-	if (AST_LIST_EMPTY(&audiohook_list->spy_list) &&
-		AST_LIST_EMPTY(&audiohook_list->whisper_list) &&
-		AST_LIST_EMPTY(&audiohook_list->manipulate_list)) {
-
-		return 1;
-	}
-	return 0;
+	return !audiohook_list
+		|| (AST_LIST_EMPTY(&audiohook_list->spy_list)
+			&& AST_LIST_EMPTY(&audiohook_list->whisper_list)
+			&& AST_LIST_EMPTY(&audiohook_list->manipulate_list));
 }
 
 /*! \brief Pass a frame off to be handled by the audiohook core
