@@ -28,7 +28,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include "asterisk/_private.h"
 
@@ -2797,7 +2797,7 @@ static void data_result_print_cli_node(int fd, const struct ast_data *node, uint
 
 	ast_free(tabs);
 
-	ast_term_color_code(&output, COLOR_WHITE, 0);
+	ast_term_color_code(&output, 0, 0);
 
 	ast_cli(fd, "%s", ast_str_buffer(output));
 
@@ -2841,19 +2841,7 @@ static void __data_result_print_cli(int fd, const struct ast_data *root, uint32_
  */
 static void data_result_print_cli(int fd, const struct ast_data *root)
 {
-	struct ast_str *output;
-
-	/* print the initial node. */
-	output = ast_str_create(30);
-	if (!output) {
-		return;
-	}
-
-	ast_term_color_code(&output, data_result_get_color(root->type), 0);
-	ast_str_append(&output, 0, "%s\n", root->name);
-	ast_term_color_code(&output, COLOR_WHITE, 0);
-	ast_cli(fd, "%s", ast_str_buffer(output));
-	ast_free(output);
+	ast_cli(fd, COLORIZE_FMT "\n", COLORIZE(data_result_get_color(root->type), 0, root->name));
 
 	__data_result_print_cli(fd, root, 0);
 
@@ -3315,7 +3303,10 @@ AST_TEST_DEFINE(test_data_get)
 
 #endif
 
-/*! \internal \brief Clean up resources on Asterisk shutdown */
+/*!
+ * \internal
+ * \brief Clean up resources on Asterisk shutdown
+ */
 static void data_shutdown(void)
 {
 	ast_manager_unregister("DataGet");
@@ -3343,7 +3334,7 @@ int ast_data_init(void)
 
 	AST_TEST_REGISTER(test_data_get);
 
-	ast_register_cleanup(data_shutdown);
+	ast_register_atexit(data_shutdown);
 
 	return res;
 }
