@@ -27,6 +27,15 @@
  * \author Steve Underwood <steveu@coppice.org>
  */
 
+/*! \li \ref dsp.c uses the configuration file \ref dsp.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*!
+ * \page dsp.conf dsp.conf
+ * \verbinclude dsp.conf.sample
+ */
+
 /* Some routines from tone_detect.c by Steven Underwood as published under the zapata library */
 /*
 	tone_detect.c - General telephony tone detection, and specific
@@ -46,7 +55,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include <math.h>
 
@@ -102,11 +111,9 @@ static struct progalias {
 	{ "uk", PROG_MODE_UK },
 };
 
-#define FREQ_ARRAY_SIZE 7
-
 static struct progress {
 	enum gsamp_size size;
-	int freqs[FREQ_ARRAY_SIZE];
+	int freqs[7];
 } modes[] = {
 	{ GSAMP_SIZE_NA, { 350, 440, 480, 620, 950, 1400, 1800 } },	/*!< North America */
 	{ GSAMP_SIZE_CR, { 425 } },					/*!< Costa Rica, Brazil */
@@ -127,7 +134,7 @@ enum busy_detect {
 	BUSY_PAT_PERCENT = 7,	/*!< The percentage difference between measured and actual pattern */
 	BUSY_THRESHOLD = 100,	/*!< Max number of ms difference between max and min times in busy */
 	BUSY_MIN = 75,		/*!< Busy must be at least 80 ms in half-cadence */
-	BUSY_MAX =3100		/*!< Busy can't be longer than 3100 ms in half-cadence */
+	BUSY_MAX = 3100		/*!< Busy can't be longer than 3100 ms in half-cadence */
 };
 
 /*! Remember last 15 units */
@@ -161,7 +168,8 @@ enum gsamp_thresh {
  */
 
 #define DTMF_THRESHOLD		8.0e7
-#define TONE_THRESHOLD		7.8e7
+#define FAX_THRESHOLD		8.0e7
+#define FAX_2ND_HARMONIC	2.0     /* 4dB */
 
 #define DEF_DTMF_NORMAL_TWIST		6.31	 /* 8.0dB */
 #define DEF_RELAX_DTMF_NORMAL_TWIST	6.31	 /* 8.0dB */
@@ -176,6 +184,8 @@ enum gsamp_thresh {
 
 #define DTMF_RELATIVE_PEAK_ROW	6.3     /* 8dB */
 #define DTMF_RELATIVE_PEAK_COL	6.3     /* 8dB */
+#define DTMF_2ND_HARMONIC_ROW       (relax ? 1.7 : 2.5)     /* 4dB normal */
+#define DTMF_2ND_HARMONIC_COL	63.1    /* 18dB */
 #define DTMF_TO_TOTAL_ENERGY	42.0
 
 #define BELL_MF_THRESHOLD	1.6e9
@@ -232,7 +242,6 @@ typedef struct {
 	int v3;
 	int chunky;
 	int fac;
-	int samples;
 } goertzel_state_t;
 
 typedef struct {
@@ -329,6 +338,16 @@ static inline void goertzel_sample(goertzel_state_t *s, short sample)
 	}
 }
 
+static inline void goertzel_update(goertzel_state_t *s, short *samps, int count)
+{
+	int i;
+
+	for (i = 0; i < count; i++) {
+		goertzel_sample(s, samps[i]);
+	}
+}
+
+
 static inline float goertzel_result(goertzel_state_t *s)
 {
 	goertzel_result_t r;
@@ -338,11 +357,10 @@ static inline float goertzel_result(goertzel_state_t *s)
 	return (float)r.value * (float)(1 << r.power);
 }
 
-static inline void goertzel_init(goertzel_state_t *s, float freq, int samples, unsigned int sample_rate)
+static inline void goertzel_init(goertzel_state_t *s, float freq, unsigned int sample_rate)
 {
 	s->v2 = s->v3 = s->chunky = 0.0;
 	s->fac = (int)(32768.0 * 2.0 * cos(2.0 * M_PI * freq / sample_rate));
-	s->samples = samples;
 }
 
 static inline void goertzel_reset(goertzel_state_t *s)
@@ -380,7 +398,7 @@ struct ast_dsp {
 	struct ast_dsp_busy_pattern busy_cadence;
 	int historicnoise[DSP_HISTORY];
 	int historicsilence[DSP_HISTORY];
-	goertzel_state_t freqs[FREQ_ARRAY_SIZE];
+	goertzel_state_t freqs[7];
 	int freqcount;
 	int gsamps;
 	enum gsamp_size gsamp_size;
@@ -448,7 +466,7 @@ static void ast_tone_detect_init(tone_detect_state_t *s, int freq, int duration,
 	   and thus no tone will be detected in them */
 	s->hits_required = (duration_samples - (s->block_size - 1)) / s->block_size;
 
-	goertzel_init(&s->tone, freq, s->block_size, sample_rate);
+	goertzel_init(&s->tone, freq, sample_rate);
 
 	s->samples_pending = s->block_size;
 	s->hit_count = 0;
@@ -482,29 +500,30 @@ static void ast_fax_detect_init(struct ast_dsp *s)
 
 }
 
-static void ast_dtmf_detect_init (dtmf_detect_state_t *s, unsigned int sample_rate)
+static void ast_dtmf_detect_init(dtmf_detect_state_t *s, unsigned int sample_rate)
 {
 	int i;
 
+	for (i = 0; i < 4; i++) {
+		goertzel_init(&s->row_out[i], dtmf_row[i], sample_rate);
+		goertzel_init(&s->col_out[i], dtmf_col[i], sample_rate);
+	}
 	s->lasthit = 0;
 	s->current_hit = 0;
-	for (i = 0;  i < 4;  i++) {
-		goertzel_init(&s->row_out[i], dtmf_row[i], DTMF_GSIZE, sample_rate);
-		goertzel_init(&s->col_out[i], dtmf_col[i], DTMF_GSIZE, sample_rate);
-		s->energy = 0.0;
-	}
+	s->energy = 0.0;
 	s->current_sample = 0;
 	s->hits = 0;
 	s->misses = 0;
 }
 
-static void ast_mf_detect_init (mf_detect_state_t *s, unsigned int sample_rate)
+static void ast_mf_detect_init(mf_detect_state_t *s, unsigned int sample_rate)
 {
 	int i;
-	s->hits[0] = s->hits[1] = s->hits[2] = s->hits[3] = s->hits[4] = 0;
-	for (i = 0;  i < 6;  i++) {
-		goertzel_init (&s->tone_out[i], mf_tones[i], MF_GSIZE, sample_rate);
+
+	for (i = 0; i < 6; i++) {
+		goertzel_init(&s->tone_out[i], mf_tones[i], sample_rate);
 	}
+	s->hits[0] = s->hits[1] = s->hits[2] = s->hits[3] = s->hits[4] = 0;
 	s->current_sample = 0;
 	s->current_hit = 0;
 }
@@ -540,7 +559,7 @@ static int tone_detect(struct ast_dsp *dsp, tone_detect_state_t *s, int16_t *amp
 		s->mute_samples -= mute.end;
 	}
 
-	for (start = 0;  start < samples;  start = end) {
+	for (start = 0; start < samples; start = end) {
 		/* Process in blocks. */
 		limit = samples - start;
 		if (limit > s->samples_pending) {
@@ -571,8 +590,7 @@ static int tone_detect(struct ast_dsp *dsp, tone_detect_state_t *s, int16_t *amp
 
 		ast_debug(10, "tone %d, Ew=%.2E, Et=%.2E, s/n=%10.2f\n", s->freq, tone_energy, s->energy, tone_energy / (s->energy - tone_energy));
 		hit = 0;
-		if (TONE_THRESHOLD <= tone_energy
-			&& tone_energy > s->energy * s->threshold) {
+		if (tone_energy > s->energy * s->threshold) {
 			ast_debug(10, "Hit! count=%d\n", s->hit_count);
 			hit = 1;
 		}
@@ -692,15 +710,15 @@ static int dtmf_detect(struct ast_dsp *dsp, digit_detect_state_t *s, int16_t amp
 		}
 		/* We are at the end of a DTMF detection block */
 		/* Find the peak row and the peak column */
-		row_energy[0] = goertzel_result (&s->td.dtmf.row_out[0]);
-		col_energy[0] = goertzel_result (&s->td.dtmf.col_out[0]);
+		row_energy[0] = goertzel_result(&s->td.dtmf.row_out[0]);
+		col_energy[0] = goertzel_result(&s->td.dtmf.col_out[0]);
 
-		for (best_row = best_col = 0, i = 1;  i < 4;  i++) {
-			row_energy[i] = goertzel_result (&s->td.dtmf.row_out[i]);
+		for (best_row = best_col = 0, i = 1; i < 4; i++) {
+			row_energy[i] = goertzel_result(&s->td.dtmf.row_out[i]);
 			if (row_energy[i] > row_energy[best_row]) {
 				best_row = i;
 			}
-			col_energy[i] = goertzel_result (&s->td.dtmf.col_out[i]);
+			col_energy[i] = goertzel_result(&s->td.dtmf.col_out[i]);
 			if (col_energy[i] > col_energy[best_col]) {
 				best_col = i;
 			}
@@ -712,7 +730,7 @@ static int dtmf_detect(struct ast_dsp *dsp, digit_detect_state_t *s, int16_t amp
 		    col_energy[best_col] < row_energy[best_row] * (relax ? relax_dtmf_reverse_twist : dtmf_reverse_twist) &&
 		    row_energy[best_row] < col_energy[best_col] * (relax ? relax_dtmf_normal_twist : dtmf_normal_twist)) {
 			/* Relative peak test */
-			for (i = 0;  i < 4;  i++) {
+			for (i = 0; i < 4; i++) {
 				if ((i != best_col &&
 				    col_energy[i] * DTMF_RELATIVE_PEAK_COL > col_energy[best_col]) ||
 				    (i != best_row
@@ -865,7 +883,7 @@ static int mf_detect(struct ast_dsp *dsp, digit_detect_state_t *s, int16_t amp[]
 	}
 
 	hit = 0;
-	for (sample = 0;  sample < samples;  sample = limit) {
+	for (sample = 0; sample < samples; sample = limit) {
 		/* 80 is optimised to meet the MF specs. */
 		/* XXX So then why is MF_GSIZE defined as 120? */
 		if ((samples - sample) >= (MF_GSIZE - s->td.mf.current_sample)) {
@@ -875,7 +893,7 @@ static int mf_detect(struct ast_dsp *dsp, digit_detect_state_t *s, int16_t amp[]
 		}
 		/* The following unrolled loop takes only 35% (rough estimate) of the
 		   time of a rolled loop on the machine on which it was developed */
-		for (j = sample;  j < limit;  j++) {
+		for (j = sample; j < limit; j++) {
 			/* With GCC 2.95, the following unrolled code seems to take about 35%
 			   (rough estimate) as long as a neat little 0-3 loop */
 			samp = amp[j];
@@ -979,7 +997,7 @@ static int mf_detect(struct ast_dsp *dsp, digit_detect_state_t *s, int16_t amp[]
 		}
 
 		/* Reinitialise the detector for the next block */
-		for (i = 0;  i < 6;  i++) {
+		for (i = 0; i < 6; i++) {
 			goertzel_reset(&s->td.mf.tone_out[i]);
 		}
 		s->td.mf.current_sample = 0;
@@ -1021,13 +1039,12 @@ static inline int pair_there(float p1, float p2, float i1, float i2, float e)
 
 static int __ast_dsp_call_progress(struct ast_dsp *dsp, short *s, int len)
 {
+	short samp;
 	int x;
 	int y;
 	int pass;
 	int newstate = DSP_TONE_STATE_SILENCE;
 	int res = 0;
-	int freqcount = dsp->freqcount > FREQ_ARRAY_SIZE ? FREQ_ARRAY_SIZE : dsp->freqcount;
-
 	while (len) {
 		/* Take the lesser of the number of samples we need and what we have */
 		pass = len;
@@ -1035,17 +1052,18 @@ static int __ast_dsp_call_progress(struct ast_dsp *dsp, short *s, int len)
 			pass = dsp->gsamp_size - dsp->gsamps;
 		}
 		for (x = 0; x < pass; x++) {
-			for (y = 0; y < freqcount; y++) {
-				goertzel_sample(&dsp->freqs[y], s[x]);
+			samp = s[x];
+			dsp->genergy += (int32_t) samp * (int32_t) samp;
+			for (y = 0; y < dsp->freqcount; y++) {
+				goertzel_sample(&dsp->freqs[y], samp);
 			}
-			dsp->genergy += s[x] * s[x];
 		}
 		s += pass;
 		dsp->gsamps += pass;
 		len -= pass;
 		if (dsp->gsamps == dsp->gsamp_size) {
-			float hz[FREQ_ARRAY_SIZE];
-			for (y = 0; y < FREQ_ARRAY_SIZE; y++) {
+			float hz[7];
+			for (y = 0; y < 7; y++) {
 				hz[y] = goertzel_result(&dsp->freqs[y]);
 			}
 			switch (dsp->progmode) {
@@ -1396,13 +1414,13 @@ static int ast_dsp_silence_noise_with_energy(struct ast_dsp *dsp, struct ast_fra
 		switch (f->subclass.format.id) {
 			case AST_FORMAT_ULAW:
 				s = ast_alloca(len * 2);
-				for (x = 0;x < len; x++) {
+				for (x = 0; x < len; x++) {
 					s[x] = AST_MULAW(odata[x]);
 				}
 				break;
 			case AST_FORMAT_ALAW:
 				s = ast_alloca(len * 2);
-				for (x = 0;x < len; x++) {
+				for (x = 0; x < len; x++) {
 					s[x] = AST_ALAW(odata[x]);
 				}
 				break;
@@ -1466,7 +1484,7 @@ struct ast_frame *ast_dsp_process(struct ast_channel *chan, struct ast_dsp *dsp,
 		case AST_FORMAT_ULAW:
 		case AST_FORMAT_TESTLAW:
 			shortdata = ast_alloca(af->datalen * 2);
-			for (x = 0;x < len; x++) {
+			for (x = 0; x < len; x++) {
 				shortdata[x] = AST_MULAW(odata[x]);
 			}
 			break;
@@ -1644,14 +1662,14 @@ static void ast_dsp_prog_reset(struct ast_dsp *dsp)
 
 	dsp->gsamp_size = modes[dsp->progmode].size;
 	dsp->gsamps = 0;
-	for (x = 0; x < FREQ_ARRAY_SIZE; x++) {
+	for (x = 0; x < ARRAY_LEN(modes[dsp->progmode].freqs); x++) {
 		if (modes[dsp->progmode].freqs[x]) {
-			goertzel_init(&dsp->freqs[x], (float)modes[dsp->progmode].freqs[x], dsp->gsamp_size, dsp->sample_rate);
+			goertzel_init(&dsp->freqs[x], (float)modes[dsp->progmode].freqs[x], dsp->sample_rate);
 			max = x + 1;
 		}
 	}
 	dsp->freqcount = max;
-	dsp->ringtimeout= 0;
+	dsp->ringtimeout = 0;
 }
 
 unsigned int ast_dsp_get_sample_rate(const struct ast_dsp *dsp)
@@ -1670,7 +1688,6 @@ static struct ast_dsp *__ast_dsp_new(unsigned int sample_rate)
 		dsp->digitmode = DSP_DIGITMODE_DTMF;
 		dsp->faxmode = DSP_FAXMODE_DETECT_CNG;
 		dsp->sample_rate = sample_rate;
-		dsp->freqcount = 0;
 		/* Initialize digit detector */
 		ast_digit_detect_init(&dsp->digit_state, dsp->digitmode & DSP_DIGITMODE_MF, dsp->sample_rate);
 		dsp->display_inband_dtmf_warning = 1;
@@ -1735,19 +1752,21 @@ void ast_dsp_digitreset(struct ast_dsp *dsp)
 	if (dsp->digitmode & DSP_DIGITMODE_MF) {
 		mf_detect_state_t *s = &dsp->digit_state.td.mf;
 		/* Reinitialise the detector for the next block */
-		for (i = 0;  i < 6;  i++) {
+		for (i = 0; i < 6; i++) {
 			goertzel_reset(&s->tone_out[i]);
 		}
-		s->hits[4] = s->hits[3] = s->hits[2] = s->hits[1] = s->hits[0] = s->current_hit = 0;
+		s->hits[4] = s->hits[3] = s->hits[2] = s->hits[1] = s->hits[0] = 0;
+		s->current_hit = 0;
 		s->current_sample = 0;
 	} else {
 		dtmf_detect_state_t *s = &dsp->digit_state.td.dtmf;
 		/* Reinitialise the detector for the next block */
-		for (i = 0;  i < 4;  i++) {
+		for (i = 0; i < 4; i++) {
 			goertzel_reset(&s->row_out[i]);
 			goertzel_reset(&s->col_out[i]);
 		}
-		s->lasthit = s->current_hit = 0;
+		s->lasthit = 0;
+		s->current_hit = 0;
 		s->energy = 0.0;
 		s->current_sample = 0;
 		s->hits = 0;
@@ -1769,7 +1788,7 @@ void ast_dsp_reset(struct ast_dsp *dsp)
 	}
 	memset(dsp->historicsilence, 0, sizeof(dsp->historicsilence));
 	memset(dsp->historicnoise, 0, sizeof(dsp->historicnoise));
-	dsp->ringtimeout= 0;
+	dsp->ringtimeout = 0;
 }
 
 int ast_dsp_set_digitmode(struct ast_dsp *dsp, int digitmode)
