@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include "asterisk/_private.h"
 #include "asterisk/lock.h"
@@ -351,6 +351,13 @@ void ast_frame_free(struct ast_frame *frame, int cache)
 	}
 }
 
+void ast_frame_dtor(struct ast_frame *f)
+{
+	if (f) {
+		ast_frfree(f);
+	}
+}
+
 /*!
  * \brief 'isolates' a frame by duplicating non-malloc'ed components
  * (header, src, data).
@@ -628,6 +635,14 @@ void ast_frame_subclass2str(struct ast_frame *f, char *subclass, size_t slen, ch
 		/* Should never happen */
 		snprintf(subclass, slen, "IAX Frametype %d", f->subclass.integer);
 		break;
+	case AST_FRAME_BRIDGE_ACTION:
+		/* Should never happen */
+		snprintf(subclass, slen, "Bridge Frametype %d", f->subclass.integer);
+		break;
+	case AST_FRAME_BRIDGE_ACTION_SYNC:
+		/* Should never happen */
+		snprintf(subclass, slen, "Synchronous Bridge Frametype %d", f->subclass.integer);
+		break;
 	case AST_FRAME_TEXT:
 		ast_copy_string(subclass, "N/A", slen);
 		if (moreinfo) {
@@ -692,6 +707,7 @@ void ast_frame_subclass2str(struct ast_frame *f, char *subclass, size_t slen, ch
 		break;
 	default:
 		ast_copy_string(subclass, "Unknown Subclass", slen);
+		break;
 	}
 }
 
@@ -714,6 +730,14 @@ void ast_frame_type2str(enum ast_frame_type frame_type, char *ftype, size_t len)
 		/* Should never happen */
 		ast_copy_string(ftype, "IAX Specific", len);
 		break;
+	case AST_FRAME_BRIDGE_ACTION:
+		/* Should never happen */
+		ast_copy_string(ftype, "Bridge Specific", len);
+		break;
+	case AST_FRAME_BRIDGE_ACTION_SYNC:
+		/* Should never happen */
+		ast_copy_string(ftype, "Bridge Specific", len);
+		break;
 	case AST_FRAME_TEXT:
 		ast_copy_string(ftype, "Text", len);
 		break;
@@ -734,6 +758,7 @@ void ast_frame_type2str(enum ast_frame_type frame_type, char *ftype, size_t len)
 		break;
 	default:
 		snprintf(ftype, len, "Unknown Frametype '%u'", frame_type);
+		break;
 	}
 }
 
@@ -797,10 +822,6 @@ int ast_parse_allow_disallow(struct ast_codec_pref *pref, struct ast_format_cap
 	char *parse = NULL, *this = NULL, *psize = NULL;
 	struct ast_format format;
 
-	if (!allowing && ast_strlen_zero(list)) {
-		return 0;
-	}
-
 	parse = ast_strdupa(list);
 	while ((this = strsep(&parse, ","))) {
 		iter_allowing = allowing;
@@ -853,6 +874,8 @@ int ast_parse_allow_disallow(struct ast_codec_pref *pref, struct ast_format_cap
 				}
 			} else if (!iter_allowing) {
 				memset(pref, 0, sizeof(*pref));
+			} else {
+				ast_codec_pref_append_all(pref);
 			}
 		}
 	}
@@ -1084,9 +1107,13 @@ int ast_codec_get_samples(struct ast_frame *f)
 			return 160;
 		}
 	case AST_FORMAT_CELT:
-		/* TODO The assumes 20ms delivery right now, which is incorrect */
+		/* TODO This assumes 20ms delivery right now, which is incorrect */
 		samples = ast_format_rate(&f->subclass.format) / 50;
 		break;
+	case AST_FORMAT_OPUS:
+		/* TODO This assumes 20ms delivery right now, which is incorrect */
+		samples = 960;
+		break;
 	default:
 		ast_log(LOG_WARNING, "Unable to calculate samples for format %s\n", ast_getformatname(&f->subclass.format));
 	}
