@@ -33,7 +33,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_MODULE_DIR */
@@ -50,12 +50,13 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/enum.h"
 #include "asterisk/http.h"
 #include "asterisk/lock.h"
-#include "asterisk/features.h"
+#include "asterisk/features_config.h"
 #include "asterisk/dsp.h"
 #include "asterisk/udptl.h"
 #include "asterisk/heap.h"
 #include "asterisk/app.h"
 #include "asterisk/test.h"
+#include "asterisk/sounds_index.h"
 
 #include <dlfcn.h>
 
@@ -63,6 +64,30 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/utils.h"
 
 /*** DOCUMENTATION
+	<managerEvent language="en_US" name="Reload">
+		<managerEventInstance class="EVENT_FLAG_SYSTEM">
+			<synopsis>Raised when a module has been reloaded in Asterisk.</synopsis>
+			<syntax>
+				<parameter name="Module">
+					<para>The name of the module that was reloaded, or
+					<literal>All</literal> if all modules were reloaded</para>
+				</parameter>
+				<parameter name="Status">
+					<para>The numeric status code denoting the success or failure
+					of the reload request.</para>
+					<enumlist>
+						<enum name="0"><para>Success</para></enum>
+						<enum name="1"><para>Request queued</para></enum>
+						<enum name="2"><para>Module not found</para></enum>
+						<enum name="3"><para>Error</para></enum>
+						<enum name="4"><para>Reload already in progress</para></enum>
+						<enum name="5"><para>Module uninitialized</para></enum>
+						<enum name="6"><para>Reload not supported</para></enum>
+					</enumlist>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
  ***/
 
 #ifndef RTLD_NOW
@@ -92,17 +117,12 @@ static unsigned int embedding = 1; /* we always start out by registering embedde
 
 struct ast_module {
 	const struct ast_module_info *info;
-#ifdef REF_DEBUG
-	/* Used to get module references into REF_DEBUG logs */
-	void *ref_debug;
-#endif
 	void *lib;					/* the shared lib, or NULL if embedded */
 	int usecount;					/* the number of 'users' currently in this module */
 	struct module_user_list users;			/* the list of users in the module */
 	struct {
 		unsigned int running:1;
 		unsigned int declined:1;
-		unsigned int keepuntilshutdown:1;
 	} flags;
 	AST_LIST_ENTRY(ast_module) entry;
 	char resource[0];
@@ -165,10 +185,9 @@ void ast_module_register(const struct ast_module_info *info)
 		mod = resource_being_loaded;
 	}
 
+	ast_verb(5, "Registering module %s\n", info->name);
+
 	mod->info = info;
-#ifdef REF_DEBUG
-	mod->ref_debug = ao2_t_alloc(0, NULL, info->name);
-#endif
 	AST_LIST_HEAD_INIT(&mod->users);
 
 	/* during startup, before the loader has been initialized,
@@ -213,10 +232,8 @@ void ast_module_unregister(const struct ast_module_info *info)
 	AST_LIST_UNLOCK(&module_list);
 
 	if (mod) {
+		ast_verb(5, "Unregistering module %s\n", info->name);
 		AST_LIST_HEAD_DESTROY(&mod->users);
-#ifdef REF_DEBUG
-		ao2_cleanup(mod->ref_debug);
-#endif
 		ast_free(mod);
 	}
 }
@@ -236,10 +253,6 @@ struct ast_module_user *__ast_module_user_add(struct ast_module *mod, struct ast
 	AST_LIST_INSERT_HEAD(&mod->users, u, entry);
 	AST_LIST_UNLOCK(&mod->users);
 
-#ifdef REF_DEBUG
-	ao2_ref(mod->ref_debug, +1);
-#endif
-
 	ast_atomic_fetchadd_int(&mod->usecount, +1);
 
 	ast_update_use_count();
@@ -264,10 +277,6 @@ void __ast_module_user_remove(struct ast_module *mod, struct ast_module_user *u)
 		return;
 	}
 
-#ifdef REF_DEBUG
-	ao2_ref(mod->ref_debug, -1);
-#endif
-
 	ast_atomic_fetchadd_int(&mod->usecount, -1);
 	ast_free(u);
 
@@ -283,11 +292,6 @@ void __ast_module_user_hangup_all(struct ast_module *mod)
 		if (u->chan) {
 			ast_softhangup(u->chan, AST_SOFTHANGUP_APPUNLOAD);
 		}
-
-#ifdef REF_DEBUG
-		ao2_ref(mod->ref_debug, -1);
-#endif
-
 		ast_atomic_fetchadd_int(&mod->usecount, -1);
 		ast_free(u);
 	}
@@ -313,12 +317,13 @@ static struct reload_classes {
 	{ "manager",	reload_manager },
 	{ "http",	ast_http_reload },
 	{ "logger",	logger_reload },
-	{ "features",	ast_features_reload },
+	{ "features",	ast_features_config_reload },
 	{ "dsp",	ast_dsp_reload},
 	{ "udptl",	ast_udptl_reload },
 	{ "indications", ast_indications_reload },
 	{ "cel",        ast_cel_engine_reload },
 	{ "plc",        ast_plc_reload },
+	{ "sounds",     ast_sounds_reindex },
 	{ NULL, 	NULL }
 };
 
@@ -328,7 +333,7 @@ static int printdigest(const unsigned char *d)
 	char buf[256]; /* large enough so we don't have to worry */
 
 	for (pos = 0, x = 0; x < 16; x++)
-		pos += sprintf(buf + pos, " %02hhx", *d++);
+		pos += sprintf(buf + pos, " %02x", (unsigned)*d++);
 
 	ast_debug(1, "Unexpected signature:%s\n", buf);
 
@@ -402,7 +407,10 @@ static struct ast_module *find_resource(const char *resource, int do_lock)
 
 #ifdef LOADABLE_MODULES
 
-static void close_lib(const char *name, void *lib)
+/*!
+ * \brief dlclose(), with failure logging.
+ */
+static void logged_dlclose(const char *name, void *lib)
 {
 	char *error;
 
@@ -419,14 +427,66 @@ static void close_lib(const char *name, void *lib)
 	}
 }
 
+#if defined(HAVE_RTLD_NOLOAD)
+/*!
+ * \brief Check to see if the given resource is loaded.
+ *
+ * \param resource_name Name of the resource, including .so suffix.
+ * \return False (0) if module is not loaded.
+ * \return True (non-zero) if module is loaded.
+ */
+static int is_module_loaded(const char *resource_name)
+{
+	char fn[PATH_MAX] = "";
+	void *lib;
+
+	ast_verb(10, "Checking if %s is loaded\n", resource_name);
+
+	snprintf(fn, sizeof(fn), "%s/%s", ast_config_AST_MODULE_DIR,
+		resource_name);
+
+	lib = dlopen(fn, RTLD_LAZY | RTLD_NOLOAD);
+
+	if (lib) {
+		ast_verb(10, "  %s loaded\n", resource_name);
+		logged_dlclose(resource_name, lib);
+		return 1;
+	}
+
+	ast_verb(10, "  %s not loaded\n", resource_name);
+	return 0;
+}
+#endif
+
 static void unload_dynamic_module(struct ast_module *mod)
 {
+	char *name = ast_strdupa(ast_module_name(mod));
 	void *lib = mod->lib;
 
 	/* WARNING: the structure pointed to by mod is going to
 	   disappear when this operation succeeds, so we can't
 	   dereference it */
-	close_lib(ast_module_name(mod), lib);
+	logged_dlclose(ast_module_name(mod), lib);
+
+	/* There are several situations where the module might still be resident
+	 * in memory.
+	 *
+	 * If somehow there was another dlopen() on the same module (unlikely,
+	 * since that all is supposed to happen in loader.c).
+	 *
+	 * Or the lazy resolution of a global symbol (very likely, since that is
+	 * how we load all of our modules that export global symbols).
+	 *
+	 * Avoid the temptation of repeating the dlclose(). The other code that
+	 * dlopened the module still has its module reference, and should close
+	 * it itself. In other situations, dlclose() will happily return success
+	 * for as many times as you wish to call it.
+	 */
+#if defined(HAVE_RTLD_NOLOAD)
+	if (is_module_loaded(name)) {
+		ast_log(LOG_WARNING, "Module '%s' could not be completely unloaded\n", name);
+	}
+#endif
 }
 
 static enum ast_module_load_result load_resource(const char *resource_name, unsigned int global_symbols_only, struct ast_heap *resource_heap, int required);
@@ -475,7 +535,7 @@ static struct ast_module *load_dynamic_module(const char *resource_in, unsigned
 	if (resource_being_loaded != (mod = AST_LIST_LAST(&module_list))) {
 		ast_log(LOG_WARNING, "Module '%s' did not register itself during load\n", resource_in);
 		/* no, it did not, so close it and return */
-		close_lib(resource_in, lib);
+		logged_dlclose(resource_in, lib);
 		/* note that the module's destructor will call ast_module_unregister(),
 		   which will free the structure we allocated in resource_being_loaded */
 		return NULL;
@@ -486,32 +546,11 @@ static struct ast_module *load_dynamic_module(const char *resource_in, unsigned
 	/* if we are being asked only to load modules that provide global symbols,
 	   and this one does not, then close it and return */
 	if (global_symbols_only && !wants_global) {
-		close_lib(resource_in, lib);
+		logged_dlclose(resource_in, lib);
 		return NULL;
 	}
 
-	/* This section is a workaround for a gcc 4.1 bug that has already been
-	 * fixed in later versions.  Unfortunately, some distributions, such as
-	 * RHEL/CentOS 5, distribute gcc 4.1, so we're stuck with having to deal
-	 * with this issue.  This basically ensures that optional_api modules are
-	 * loaded before any module which requires their functionality. */
-#if !defined(HAVE_ATTRIBUTE_weak_import) && !defined(HAVE_ATTRIBUTE_weakref)
-	if (!ast_strlen_zero(mod->info->nonoptreq)) {
-		/* Force any required dependencies to load */
-		char *each, *required_resource = ast_strdupa(mod->info->nonoptreq);
-		while ((each = strsep(&required_resource, ","))) {
-			struct ast_module *dependency;
-			each = ast_strip(each);
-			dependency = find_resource(each, 0);
-			/* Is it already loaded? */
-			if (!dependency) {
-				load_resource(each, global_symbols_only, resource_heap, 1);
-			}
-		}
-	}
-#endif
-
-	close_lib(resource_in, lib);
+	logged_dlclose(resource_in, lib);
 	resource_being_loaded = NULL;
 
 	/* start the load process again */
@@ -537,6 +576,7 @@ static struct ast_module *load_dynamic_module(const char *resource_in, unsigned
 
 	return AST_LIST_LAST(&module_list);
 }
+
 #endif
 
 void ast_module_shutdown(void)
@@ -565,25 +605,14 @@ void ast_module_shutdown(void)
 			}
 			AST_LIST_REMOVE_CURRENT(entry);
 			if (mod->flags.running && !mod->flags.declined && mod->info->unload) {
+				ast_verb(1, "Unloading %s\n", mod->resource);
 				mod->info->unload();
 			}
 			AST_LIST_HEAD_DESTROY(&mod->users);
-#ifdef REF_DEBUG
-			ao2_cleanup(mod->ref_debug);
-#endif
 			free(mod);
 			somethingchanged = 1;
 		}
 		AST_LIST_TRAVERSE_SAFE_END;
-		if (!somethingchanged) {
-			AST_LIST_TRAVERSE(&module_list, mod, entry) {
-				if (mod->flags.keepuntilshutdown) {
-					ast_module_unref(mod);
-					mod->flags.keepuntilshutdown = 0;
-					somethingchanged = 1;
-				}
-			}
-		}
 	} while (somethingchanged && !final);
 
 	AST_LIST_UNLOCK(&module_list);
@@ -623,6 +652,7 @@ int ast_unload_resource(const char *resource_name, enum ast_module_unload_mode f
 		/* Request any channels attached to the module to hangup. */
 		__ast_module_user_hangup_all(mod);
 
+		ast_verb(1, "Unloading %s\n", mod->resource);
 		res = mod->info->unload();
 		if (res) {
 			ast_log(LOG_WARNING, "Firm unload failed for %s\n", resource_name);
@@ -759,22 +789,63 @@ static void queue_reload_request(const char *module)
 	AST_LIST_UNLOCK(&reload_queue);
 }
 
-int ast_module_reload(const char *name)
+/*!
+ * \since 12
+ * \internal
+ * \brief Publish a \ref stasis message regarding the reload result
+ */
+static void publish_reload_message(const char *name, enum ast_module_reload_result result)
+{
+	RAII_VAR(struct stasis_message *, message, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_json_payload *, payload, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
+	RAII_VAR(struct ast_json *, event_object, NULL, ast_json_unref);
+	char res_buffer[8];
+
+	snprintf(res_buffer, sizeof(res_buffer), "%u", result);
+	event_object = ast_json_pack("{s: s, s: s}",
+			"Module", S_OR(name, "All"),
+			"Status", res_buffer);
+	json_object = ast_json_pack("{s: s, s: i, s: O}",
+			"type", "Reload",
+			"class_type", EVENT_FLAG_SYSTEM,
+			"event", event_object);
+
+	if (!json_object) {
+		return;
+	}
+
+	payload = ast_json_payload_create(json_object);
+	if (!payload) {
+		return;
+	}
+
+	message = stasis_message_create(ast_manager_get_generic_type(), payload);
+	if (!message) {
+		return;
+	}
+
+	stasis_publish(ast_manager_get_topic(), message);
+}
+
+enum ast_module_reload_result ast_module_reload(const char *name)
 {
 	struct ast_module *cur;
-	int res = 0; /* return value. 0 = not found, others, see below */
+	enum ast_module_reload_result res = AST_MODULE_RELOAD_NOT_FOUND;
 	int i;
 
 	/* If we aren't fully booted, we just pretend we reloaded but we queue this
 	   up to run once we are booted up. */
 	if (!ast_fully_booted) {
 		queue_reload_request(name);
-		return 0;
+		res = AST_MODULE_RELOAD_QUEUED;
+		goto module_reload_exit;
 	}
 
 	if (ast_mutex_trylock(&reloadlock)) {
 		ast_verbose("The previous reload command didn't finish yet\n");
-		return -1;	/* reload already in progress */
+		res = AST_MODULE_RELOAD_IN_PROGRESS;
+		goto module_reload_exit;
 	}
 	ast_lastreloadtime = ast_tvnow();
 
@@ -790,26 +861,26 @@ int ast_module_reload(const char *name)
 		if (res != AST_LOCK_SUCCESS) {
 			ast_verbose("Cannot grab lock on %s\n", ast_config_AST_CONFIG_DIR);
 			ast_mutex_unlock(&reloadlock);
-			return -1;
+			res = AST_MODULE_RELOAD_ERROR;
+			goto module_reload_exit;
 		}
 	}
 
 	/* Call "predefined" reload here first */
 	for (i = 0; reload_classes[i].name; i++) {
 		if (!name || !strcasecmp(name, reload_classes[i].name)) {
-			if (!reload_classes[i].reload_fn()) {
-				ast_test_suite_event_notify("MODULE_RELOAD", "Message: %s", name);
+			if (reload_classes[i].reload_fn() == AST_MODULE_LOAD_SUCCESS) {
+				res = AST_MODULE_RELOAD_SUCCESS;
 			}
-			res = 2;	/* found and reloaded */
 		}
 	}
 
-	if (name && res) {
+	if (name && res == AST_MODULE_RELOAD_SUCCESS) {
 		if (ast_opt_lock_confdir) {
 			ast_unlock_path(ast_config_AST_CONFIG_DIR);
 		}
 		ast_mutex_unlock(&reloadlock);
-		return res;
+		goto module_reload_exit;
 	}
 
 	AST_LIST_LOCK(&module_list);
@@ -820,28 +891,30 @@ int ast_module_reload(const char *name)
 			continue;
 
 		if (!cur->flags.running || cur->flags.declined) {
-			if (!name)
+			if (res == AST_MODULE_RELOAD_NOT_FOUND) {
+				res = AST_MODULE_RELOAD_UNINITIALIZED;
+			}
+			if (!name) {
 				continue;
-			ast_log(LOG_NOTICE, "The module '%s' was not properly initialized.  "
-				"Before reloading the module, you must run \"module load %s\" "
-				"and fix whatever is preventing the module from being initialized.\n",
-				name, name);
-			res = 2; /* Don't report that the module was not found */
+			}
 			break;
 		}
 
 		if (!info->reload) {	/* cannot be reloaded */
-			/* Nothing to reload, so reload is successful */
-			ast_test_suite_event_notify("MODULE_RELOAD", "Message: %s", cur->resource);
-			if (res < 1)	/* store result if possible */
-				res = 1;	/* 1 = no reload() method */
-			continue;
+			if (res == AST_MODULE_RELOAD_NOT_FOUND) {
+				res = AST_MODULE_RELOAD_NOT_IMPLEMENTED;
+			}
+			if (!name) {
+				continue;
+			}
+			break;
 		}
-
-		res = 2;
 		ast_verb(3, "Reloading module '%s' (%s)\n", cur->resource, info->description);
-		if (!info->reload()) {
-			ast_test_suite_event_notify("MODULE_RELOAD", "Message: %s", cur->resource);
+		if (info->reload() == AST_MODULE_LOAD_SUCCESS) {
+			res = AST_MODULE_RELOAD_SUCCESS;
+		}
+		if (name) {
+			break;
 		}
 	}
 	AST_LIST_UNLOCK(&module_list);
@@ -851,6 +924,8 @@ int ast_module_reload(const char *name)
 	}
 	ast_mutex_unlock(&reloadlock);
 
+module_reload_exit:
+	publish_reload_message(name, res);
 	return res;
 }
 
@@ -894,6 +969,9 @@ static enum ast_module_load_result start_resource(struct ast_module *mod)
 		return AST_MODULE_LOAD_FAILURE;
 	}
 
+	if (!ast_fully_booted) {
+		ast_verb(1, "Loading %s.\n", mod->resource);
+	}
 	res = mod->info->load();
 
 	switch (res) {
@@ -1042,16 +1120,17 @@ static int mod_load_cmp(void *a, void *b)
 {
 	struct ast_module *a_mod = (struct ast_module *) a;
 	struct ast_module *b_mod = (struct ast_module *) b;
-	int res = -1;
-	/* if load_pri is not set, default is 128.  Lower is better*/
-	unsigned char a_pri = ast_test_flag(a_mod->info, AST_MODFLAG_LOAD_ORDER) ? a_mod->info->load_pri : 128;
-	unsigned char b_pri = ast_test_flag(b_mod->info, AST_MODFLAG_LOAD_ORDER) ? b_mod->info->load_pri : 128;
-	if (a_pri == b_pri) {
-		res = 0;
-	} else if (a_pri < b_pri) {
-		res = 1;
-	}
-	return res;
+	/* if load_pri is not set, default is 128.  Lower is better */
+	int a_pri = ast_test_flag(a_mod->info, AST_MODFLAG_LOAD_ORDER) ? a_mod->info->load_pri : 128;
+	int b_pri = ast_test_flag(b_mod->info, AST_MODFLAG_LOAD_ORDER) ? b_mod->info->load_pri : 128;
+
+	/*
+	 * Returns comparison values for a min-heap
+	 * <0 a_pri > b_pri
+	 * =0 a_pri == b_pri
+	 * >0 a_pri < b_pri
+	 */
+	return b_pri - a_pri;
 }
 
 /*! loads modules in order by load_pri, updates mod_count
@@ -1260,25 +1339,6 @@ done:
 	}
 
 	AST_LIST_UNLOCK(&module_list);
-
-	/* Tell manager clients that are aggressive at logging in that we're done
-	   loading modules. If there's a DNS problem in chan_sip, we might not
-	   even reach this */
-	/*** DOCUMENTATION
-		<managerEventInstance>
-			<synopsis>Raised when all dynamic modules have finished their initial loading.</synopsis>
-			<syntax>
-				<parameter name="ModuleSelection">
-					<enumlist>
-						<enum name="Preload"/>
-						<enum name="All"/>
-					</enumlist>
-				</parameter>
-			</syntax>
-		</managerEventInstance>
-	***/
-	manager_event(EVENT_FLAG_SYSTEM, "ModuleLoadReport", "ModuleLoadStatus: Done\r\nModuleSelection: %s\r\nModuleCount: %d\r\n", preload_only ? "Preload" : "All", modulecount);
-
 	return res;
 }
 
@@ -1294,7 +1354,7 @@ void ast_update_use_count(void)
 	AST_LIST_UNLOCK(&updaters);
 }
 
-int ast_update_module_list(int (*modentry)(const char *module, const char *description, int usecnt, const char *like),
+int ast_update_module_list(int (*modentry)(const char *module, const char *description, int usecnt, const char *status, const char *like),
 			   const char *like)
 {
 	struct ast_module *cur;
@@ -1305,7 +1365,8 @@ int ast_update_module_list(int (*modentry)(const char *module, const char *descr
 		unlock = 0;
 
 	AST_LIST_TRAVERSE(&module_list, cur, entry) {
-		total_mod_loaded += modentry(cur->resource, cur->info->description, cur->usecount, like);
+		total_mod_loaded += modentry(cur->resource, cur->info->description, cur->usecount,
+						cur->flags.running ? "Running" : "Not Running", like);
 	}
 
 	if (unlock)
@@ -1360,62 +1421,24 @@ int ast_loader_unregister(int (*v)(void))
 	return cur ? 0 : -1;
 }
 
-struct ast_module *__ast_module_ref(struct ast_module *mod, const char *file, int line, const char *func)
+struct ast_module *ast_module_ref(struct ast_module *mod)
 {
 	if (!mod) {
 		return NULL;
 	}
 
-#ifdef REF_DEBUG
-	__ao2_ref_debug(mod->ref_debug, +1, "", file, line, func);
-#endif
-
 	ast_atomic_fetchadd_int(&mod->usecount, +1);
 	ast_update_use_count();
 
 	return mod;
 }
 
-void __ast_module_shutdown_ref(struct ast_module *mod, const char *file, int line, const char *func)
-{
-	if (!mod || mod->flags.keepuntilshutdown) {
-		return;
-	}
-
-	__ast_module_ref(mod, file, line, func);
-	mod->flags.keepuntilshutdown = 1;
-}
-
-void __ast_module_unref(struct ast_module *mod, const char *file, int line, const char *func)
+void ast_module_unref(struct ast_module *mod)
 {
 	if (!mod) {
 		return;
 	}
 
-#ifdef REF_DEBUG
-	__ao2_ref_debug(mod->ref_debug, -1, "", file, line, func);
-#endif
-
 	ast_atomic_fetchadd_int(&mod->usecount, -1);
 	ast_update_use_count();
 }
-
-
-
-/* The following exists for ABI compatibility */
-#undef ast_module_ref
-#undef ast_module_unref
-
-struct ast_module *ast_module_ref(struct ast_module *mod);
-void ast_module_unref(struct ast_module *mod);
-
-struct ast_module *ast_module_ref(struct ast_module *mod)
-{
-	return __ast_module_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
-void ast_module_unref(struct ast_module *mod)
-{
-	__ast_module_unref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
