@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 425503 $")
 
 #ifdef DEBUG_SCHEDULER
 #define DEBUG(a) do { \
@@ -65,32 +65,22 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 AST_THREADSTORAGE(last_del_id);
 
-/*!
- * \brief Scheduler ID holder
- *
- * These form a queue on a scheduler context. When a new
- * scheduled item is created, a sched_id is popped off the
- * queue and its id is assigned to the new scheduled item.
- * When the scheduled task is complete, the sched_id on that
- * task is then pushed to the back of the queue to be re-used
- * on some future scheduled item.
- */
-struct sched_id {
-	/*! Immutable ID number that is copied onto the scheduled task */
-	int id;
-	AST_LIST_ENTRY(sched_id) list;
-};
-
 struct sched {
 	AST_LIST_ENTRY(sched) list;
-	/*! The ID that has been popped off the scheduler context's queue */
-	struct sched_id *sched_id;
+	int id;                       /*!< ID number of event */
 	struct timeval when;          /*!< Absolute time event should take place */
 	int resched;                  /*!< When to reschedule */
 	int variable;                 /*!< Use return value from callback to reschedule */
 	const void *data;             /*!< Data */
 	ast_sched_cb callback;        /*!< Callback */
 	ssize_t __heap_index;
+	/*!
+	 * Used to synchronize between thread running a task and thread
+	 * attempting to delete a task
+	 */
+	ast_cond_t cond;
+	/*! Indication that a running task was deleted. */
+	unsigned int deleted:1;
 };
 
 struct sched_thread {
@@ -107,15 +97,13 @@ struct ast_sched_context {
 	struct ast_hashtab *schedq_ht;             /*!< hash table for fast searching */
 	struct ast_heap *sched_heap;
 	struct sched_thread *sched_thread;
+	/*! The scheduled task that is currently executing */
+	struct sched *currently_executing;
 
 #ifdef SCHED_MAX_CACHE
 	AST_LIST_HEAD_NOLOCK(, sched) schedc;   /*!< Cache of unused schedule structures and how many */
 	unsigned int schedccnt;
 #endif
-	/*! Queue of scheduler task IDs to assign */
-	AST_LIST_HEAD_NOLOCK(, sched_id) id_queue;
-	/*! The number of IDs in the id_queue */
-	int id_queue_size;
 };
 
 static void *sched_run(void *data)
@@ -213,13 +201,13 @@ static int sched_cmp(const void *a, const void *b)
 {
 	const struct sched *as = a;
 	const struct sched *bs = b;
-	return as->sched_id->id != bs->sched_id->id; /* return 0 on a match like strcmp would */
+	return as->id != bs->id; /* return 0 on a match like strcmp would */
 }
 
 static unsigned int sched_hash(const void *obj)
 {
 	const struct sched *s = obj;
-	unsigned int h = s->sched_id->id;
+	unsigned int h = s->id;
 	return h;
 }
 
@@ -240,7 +228,6 @@ struct ast_sched_context *ast_sched_context_create(void)
 	tmp->eventcnt = 1;
 
 	tmp->schedq_ht = ast_hashtab_create(23, sched_cmp, ast_hashtab_resize_java, ast_hashtab_newsize_java, sched_hash, 1);
-	AST_LIST_HEAD_INIT_NOLOCK(&tmp->id_queue);
 
 	if (!(tmp->sched_heap = ast_heap_create(8, sched_time_cmp,
 			offsetof(struct sched, __heap_index)))) {
@@ -253,18 +240,13 @@ struct ast_sched_context *ast_sched_context_create(void)
 
 static void sched_free(struct sched *task)
 {
-	/* task->sched_id will be NULL most of the time, but when the
-	 * scheduler context shuts down, it will free all scheduled
-	 * tasks, and in that case, the task->sched_id will be non-NULL
-	 */
-	ast_free(task->sched_id);
+	ast_cond_destroy(&task->cond);
 	ast_free(task);
 }
 
 void ast_sched_context_destroy(struct ast_sched_context *con)
 {
 	struct sched *s;
-	struct sched_id *sid;
 
 	sched_thread_destroy(con);
 	con->sched_thread = NULL;
@@ -287,9 +269,6 @@ void ast_sched_context_destroy(struct ast_sched_context *con)
 
 	ast_hashtab_destroy(con->schedq_ht, NULL);
 	con->schedq_ht = NULL;
-	while ((sid = AST_LIST_REMOVE_HEAD(&con->id_queue, list))) {
-		ast_free(sid);
-	}
 
 	ast_mutex_unlock(&con->lock);
 	ast_mutex_destroy(&con->lock);
@@ -297,81 +276,6 @@ void ast_sched_context_destroy(struct ast_sched_context *con)
 	ast_free(con);
 }
 
-#define ID_QUEUE_INCREMENT 16
-
-/*!
- * \brief Add new scheduler IDs to the queue.
- *
- * \retval The number of IDs added to the queue
- */
-static int add_ids(struct ast_sched_context *con)
-{
-	int new_size;
-	int original_size;
-	int i;
-
-	original_size = con->id_queue_size;
-	/* So we don't go overboard with the mallocs here, we'll just up
-	 * the size of the list by a fixed amount each time instead of
-	 * multiplying the size by any particular factor
-	 */
-	new_size = original_size + ID_QUEUE_INCREMENT;
-	if (new_size < 0) {
-		/* Overflow. Cap it at INT_MAX. */
-		new_size = INT_MAX;
-	}
-	for (i = original_size; i < new_size; ++i) {
-		struct sched_id *new_id;
-
-		new_id = ast_calloc(1, sizeof(*new_id));
-		if (!new_id) {
-			break;
-		}
-
-		/*
-		 * According to the API doxygen a sched ID of 0 is valid.
-		 * Unfortunately, 0 was never returned historically and
-		 * several users incorrectly coded usage of the returned
-		 * sched ID assuming that 0 was invalid.
-		 */
-		new_id->id = ++con->id_queue_size;
-
-		AST_LIST_INSERT_TAIL(&con->id_queue, new_id, list);
-	}
-
-	return con->id_queue_size - original_size;
-}
-
-static int set_sched_id(struct ast_sched_context *con, struct sched *new_sched)
-{
-	if (AST_LIST_EMPTY(&con->id_queue) && (add_ids(con) == 0)) {
-		return -1;
-	}
-
-	new_sched->sched_id = AST_LIST_REMOVE_HEAD(&con->id_queue, list);
-	return 0;
-}
-
-static void sched_release(struct ast_sched_context *con, struct sched *tmp)
-{
-	if (tmp->sched_id) {
-		AST_LIST_INSERT_TAIL(&con->id_queue, tmp->sched_id, list);
-		tmp->sched_id = NULL;
-	}
-
-	/*
-	 * Add to the cache, or just free() if we
-	 * already have too many cache entries
-	 */
-#ifdef SCHED_MAX_CACHE
-	if (con->schedccnt < SCHED_MAX_CACHE) {
-		AST_LIST_INSERT_HEAD(&con->schedc, tmp, list);
-		con->schedccnt++;
-	} else
-#endif
-		ast_free(tmp);
-}
-
 static struct sched *sched_alloc(struct ast_sched_context *con)
 {
 	struct sched *tmp;
@@ -383,47 +287,30 @@ static struct sched *sched_alloc(struct ast_sched_context *con)
 #ifdef SCHED_MAX_CACHE
 	if ((tmp = AST_LIST_REMOVE_HEAD(&con->schedc, list))) {
 		con->schedccnt--;
-	} else
+	} else 
 #endif
 	{
 		tmp = ast_calloc(1, sizeof(*tmp));
-		if (!tmp) {
-			return NULL;
-		}
-	}
-
-	if (set_sched_id(con, tmp)) {
-		sched_release(con, tmp);
-		return NULL;
+		ast_cond_init(&tmp->cond, NULL);
 	}
 
 	return tmp;
 }
 
-void ast_sched_clean_by_callback(struct ast_sched_context *con, ast_sched_cb match, ast_sched_cb cleanup_cb)
+static void sched_release(struct ast_sched_context *con, struct sched *tmp)
 {
-	int i = 1;
-	struct sched *current;
+	/*
+	 * Add to the cache, or just free() if we
+	 * already have too many cache entries
+	 */
 
-	ast_mutex_lock(&con->lock);
-	while ((current = ast_heap_peek(con->sched_heap, i))) {
-		if (current->callback != match) {
-			i++;
-			continue;
-		}
-
-		ast_heap_remove(con->sched_heap, current);
-		if (!ast_hashtab_remove_this_object(con->schedq_ht, current)) {
-			ast_log(LOG_ERROR,"Sched entry %d was in the schedq list but not in the hashtab???\n",
-				current->sched_id->id);
-		}
-
-		con->schedcnt--;
-
-		cleanup_cb(current->data);
-		sched_release(con, current);
-	}
-	ast_mutex_unlock(&con->lock);
+#ifdef SCHED_MAX_CACHE
+	if (con->schedccnt < SCHED_MAX_CACHE) {
+		AST_LIST_INSERT_HEAD(&con->schedc, tmp, list);
+		con->schedccnt++;
+	} else
+#endif
+		sched_free(tmp);
 }
 
 /*! \brief
@@ -462,8 +349,7 @@ static void schedule(struct ast_sched_context *con, struct sched *s)
 	ast_heap_push(con->sched_heap, s);
 
 	if (!ast_hashtab_insert_safe(con->schedq_ht, s)) {
-		ast_log(LOG_WARNING,"Schedule Queue entry %d is already in table!\n",
-			s->sched_id->id);
+		ast_log(LOG_WARNING,"Schedule Queue entry %d is already in table!\n", s->id);
 	}
 
 	con->schedcnt++;
@@ -481,17 +367,6 @@ static int sched_settime(struct timeval *t, int when)
 {
 	struct timeval now = ast_tvnow();
 
-	if (when < 0) {
-		/*
-		 * A negative when value is likely a bug as it
-		 * represents a VERY large timeout time.
-		 */
-		ast_log(LOG_WARNING,
-			"Bug likely: Negative time interval %d (interpreted as %u ms) requested!\n",
-			when, (unsigned int) when);
-		ast_assert(0);
-	}
-
 	/*ast_debug(1, "TV -> %lu,%lu\n", tv->tv_sec, tv->tv_usec);*/
 	if (ast_tvzero(*t))	/* not supplied, default to now */
 		*t = now;
@@ -523,17 +398,18 @@ int ast_sched_add_variable(struct ast_sched_context *con, int when, ast_sched_cb
 
 	ast_mutex_lock(&con->lock);
 	if ((tmp = sched_alloc(con))) {
-		con->eventcnt++;
+		tmp->id = con->eventcnt++;
 		tmp->callback = callback;
 		tmp->data = data;
 		tmp->resched = when;
 		tmp->variable = variable;
 		tmp->when = ast_tv(0, 0);
+		tmp->deleted = 0;
 		if (sched_settime(&tmp->when, when)) {
 			sched_release(con, tmp);
 		} else {
 			schedule(con, tmp);
-			res = tmp->sched_id->id;
+			res = tmp->id;
 		}
 	}
 #ifdef DUMP_SCHEDULER
@@ -564,10 +440,8 @@ int ast_sched_add(struct ast_sched_context *con, int when, ast_sched_cb callback
 
 const void *ast_sched_find_data(struct ast_sched_context *con, int id)
 {
-	struct sched_id tmp_id;
 	struct sched tmp,*res;
-	tmp_id.id = id;
-	tmp.sched_id = &tmp_id;
+	tmp.id = id;
 	res = ast_hashtab_lookup(con->schedq_ht, &tmp);
 	if (res)
 		return res->data;
@@ -586,11 +460,8 @@ int ast_sched_del(struct ast_sched_context *con, int id)
 int _ast_sched_del(struct ast_sched_context *con, int id, const char *file, int line, const char *function)
 #endif
 {
-	struct sched_id tmp_id = {
-		.id = id,
-	};
 	struct sched *s, tmp = {
-		.sched_id = &tmp_id,
+		.id = id,
 	};
 	int *last_id = ast_threadstorage_get(&last_del_id, sizeof(int));
 
@@ -604,17 +475,22 @@ int _ast_sched_del(struct ast_sched_context *con, int id, const char *file, int
 	s = ast_hashtab_lookup(con->schedq_ht, &tmp);
 	if (s) {
 		if (!ast_heap_remove(con->sched_heap, s)) {
-			ast_log(LOG_WARNING,"sched entry %d not in the sched heap?\n", s->sched_id->id);
+			ast_log(LOG_WARNING,"sched entry %d not in the sched heap?\n", s->id);
 		}
 
 		if (!ast_hashtab_remove_this_object(con->schedq_ht, s)) {
-			ast_log(LOG_WARNING,"Found sched entry %d, then couldn't remove it?\n",
-				s->sched_id->id);
+			ast_log(LOG_WARNING,"Found sched entry %d, then couldn't remove it?\n", s->id);
 		}
-
 		con->schedcnt--;
-
 		sched_release(con, s);
+	} else if (con->currently_executing && (id == con->currently_executing->id)) {
+		s = con->currently_executing;
+		s->deleted = 1;
+		/* Wait for executing task to complete so that caller of ast_sched_del() does not
+		 * free memory out from under the task.
+		 */
+		ast_cond_wait(&s->cond, &con->lock);
+		/* Do not sched_release() here because ast_sched_runq() will do it */
 	}
 
 #ifdef DUMP_SCHEDULER
@@ -707,7 +583,7 @@ void ast_sched_dump(struct ast_sched_context *con)
 		q = ast_heap_peek(con->sched_heap, x);
 		delta = ast_tvsub(q->when, when);
 		ast_debug(1, "|%.4d | %-15p | %-15p | %.6ld : %.6ld |\n",
-			q->sched_id->id,
+			q->id,
 			q->callback,
 			q->data,
 			(long)delta.tv_sec,
@@ -745,8 +621,7 @@ int ast_sched_runq(struct ast_sched_context *con)
 		current = ast_heap_pop(con->sched_heap);
 
 		if (!ast_hashtab_remove_this_object(con->schedq_ht, current)) {
-			ast_log(LOG_ERROR,"Sched entry %d was in the schedq list but not in the hashtab???\n",
-				current->sched_id->id);
+			ast_log(LOG_ERROR,"Sched entry %d was in the schedq list but not in the hashtab???\n", current->id);
 		}
 
 		con->schedcnt--;
@@ -760,11 +635,14 @@ int ast_sched_runq(struct ast_sched_context *con)
 		 * should return 0.
 		 */
 
+		con->currently_executing = current;
 		ast_mutex_unlock(&con->lock);
 		res = current->callback(current->data);
 		ast_mutex_lock(&con->lock);
+		con->currently_executing = NULL;
+		ast_cond_signal(&current->cond);
 
-		if (res) {
+		if (res && !current->deleted) {
 			/*
 			 * If they return non-zero, we should schedule them to be
 			 * run again.
@@ -787,16 +665,14 @@ int ast_sched_runq(struct ast_sched_context *con)
 
 long ast_sched_when(struct ast_sched_context *con,int id)
 {
-	struct sched_id tmp_id;
 	struct sched *s, tmp;
 	long secs = -1;
 	DEBUG(ast_debug(1, "ast_sched_when()\n"));
 
 	ast_mutex_lock(&con->lock);
 
-	/* these next 3 lines replace a lookup loop */
-	tmp_id.id = id;
-	tmp.sched_id = &tmp_id;
+	/* these next 2 lines replace a lookup loop */
+	tmp.id = id;
 	s = ast_hashtab_lookup(con->schedq_ht, &tmp);
 
 	if (s) {
