@@ -31,7 +31,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 434388 $")
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -400,11 +400,7 @@ static int tcptls_stream_close(void *cookie)
 
 			if (!stream->ssl->server) {
 				/* For client threads, ensure that the error stack is cleared */
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-				ERR_remove_thread_state(NULL);
-#else
 				ERR_remove_state(0);
-#endif	/* OPENSSL_VERSION_NUMBER >= 0x10000000L */
 			}
 
 			SSL_free(stream->ssl);
@@ -552,6 +548,7 @@ static void session_instance_destructor(void *obj)
 		i->stream_cookie = NULL;
 	}
 	ast_free(i->overflow_buf);
+	ao2_cleanup(i->private_data);
 }
 
 /*! \brief
@@ -717,9 +714,8 @@ void *ast_tcptls_server_root(void *data)
 		}
 		fd = ast_accept(desc->accept_fd, &addr);
 		if (fd < 0) {
-			if ((errno != EAGAIN) && (errno != EWOULDBLOCK) && (errno != EINTR) && (errno != ECONNABORTED)) {
+			if ((errno != EAGAIN) && (errno != EINTR)) {
 				ast_log(LOG_WARNING, "Accept failed: %s\n", strerror(errno));
-				break;
 			}
 			continue;
 		}
@@ -758,8 +754,7 @@ static int __ssl_setup(struct ast_tls_config *cfg, int client)
 	return 0;
 #else
 	int disable_ssl = 0;
-	long ssl_opts = 0;
-
+ 
 	if (!cfg->enabled) {
 		return 0;
 	}
@@ -779,13 +774,10 @@ static int __ssl_setup(struct ast_tls_config *cfg, int client)
 			cfg->ssl_ctx = SSL_CTX_new(SSLv2_client_method());
 		} else
 #endif
-#ifndef OPENSSL_NO_SSL3_METHOD
 		if (ast_test_flag(&cfg->flags, AST_SSL_SSLV3_CLIENT)) {
 			ast_log(LOG_WARNING, "Usage of SSLv3 is discouraged due to known vulnerabilities. Please use 'tlsv1' or leave the TLS method unspecified!\n");
 			cfg->ssl_ctx = SSL_CTX_new(SSLv3_client_method());
-		} else
-#endif
-		if (ast_test_flag(&cfg->flags, AST_SSL_TLSV1_CLIENT)) {
+		} else if (ast_test_flag(&cfg->flags, AST_SSL_TLSV1_CLIENT)) {
 			cfg->ssl_ctx = SSL_CTX_new(TLSv1_client_method());
 		} else {
 			disable_ssl = 1;
@@ -807,30 +799,12 @@ static int __ssl_setup(struct ast_tls_config *cfg, int client)
 	 * them. SSLv23_*_method supports TLSv1+.
 	 */
 	if (disable_ssl) {
-		ssl_opts |= SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
-	}
+		long ssl_opts;
 
-	if (ast_test_flag(&cfg->flags, AST_SSL_SERVER_CIPHER_ORDER)) {
-		ssl_opts |= SSL_OP_CIPHER_SERVER_PREFERENCE;
+		ssl_opts = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
+		SSL_CTX_set_options(cfg->ssl_ctx, ssl_opts);
 	}
 
-	if (ast_test_flag(&cfg->flags, AST_SSL_DISABLE_TLSV1)) {
-		ssl_opts |= SSL_OP_NO_TLSv1;
-	}
-#if defined(HAVE_SSL_OP_NO_TLSV1_1) && defined(HAVE_SSL_OP_NO_TLSV1_2)
-	if (ast_test_flag(&cfg->flags, AST_SSL_DISABLE_TLSV11)) {
-		ssl_opts |= SSL_OP_NO_TLSv1_1;
-	}
-	if (ast_test_flag(&cfg->flags, AST_SSL_DISABLE_TLSV12)) {
-		ssl_opts |= SSL_OP_NO_TLSv1_2;
-	}
-#else
-	ast_log(LOG_WARNING, "Your version of OpenSSL leaves you potentially vulnerable "
-			"to the SSL BEAST attack. Please upgrade to OpenSSL 1.0.1 or later\n");
-#endif
-
-	SSL_CTX_set_options(cfg->ssl_ctx, ssl_opts);
-
 	SSL_CTX_set_verify(cfg->ssl_ctx,
 		ast_test_flag(&cfg->flags, AST_SSL_VERIFY_CLIENT) ? SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT : SSL_VERIFY_NONE,
 		NULL);
@@ -1146,14 +1120,6 @@ int ast_tls_read_conf(struct ast_tls_config *tls_cfg, struct ast_tcptls_session_
 			ast_clear_flag(&tls_cfg->flags, AST_SSL_TLSV1_CLIENT);
 			ast_clear_flag(&tls_cfg->flags, AST_SSL_SSLV3_CLIENT);
 		}
-	} else if (!strcasecmp(varname, "tlsservercipherorder")) {
-		ast_set2_flag(&tls_cfg->flags, ast_true(value), AST_SSL_SERVER_CIPHER_ORDER);
-	} else if (!strcasecmp(varname, "tlsdisablev1")) {
-		ast_set2_flag(&tls_cfg->flags, ast_true(value), AST_SSL_DISABLE_TLSV1);
-	} else if (!strcasecmp(varname, "tlsdisablev11")) {
-		ast_set2_flag(&tls_cfg->flags, ast_true(value), AST_SSL_DISABLE_TLSV11);
-	} else if (!strcasecmp(varname, "tlsdisablev12")) {
-		ast_set2_flag(&tls_cfg->flags, ast_true(value), AST_SSL_DISABLE_TLSV12);
 	} else {
 		return -1;
 	}
