@@ -31,7 +31,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 377884 $")
 
 #include "asterisk/_private.h"
 
@@ -313,7 +313,7 @@ int ast_timing_init(void)
 		return -1;
 	}
 
-	ast_register_cleanup(timing_shutdown);
+	ast_register_atexit(timing_shutdown);
 
 	return ast_cli_register_multiple(cli_timing, ARRAY_LEN(cli_timing));
 }
