@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -43,7 +43,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/sched.h"
 #include "asterisk/cli.h"
 #include "asterisk/term.h"
-#include "asterisk/linkedlists.h"
 
 /*! \todo
  * TODO: sample frames for each supported input format.
@@ -264,6 +263,9 @@ static void matrix_clear(void)
  */
 static struct translator_path *matrix_get(unsigned int x, unsigned int y)
 {
+	if (!(x >= 0 && y >= 0)) {
+		return NULL;
+	}
 	return __matrix[x] + y;
 }
 
@@ -328,6 +330,9 @@ static void destroy(struct ast_trans_pvt *pvt)
 /*! \brief framein wrapper, deals with bound checks.  */
 static int framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
 {
+	int ret;
+	int samples = pvt->samples;	/* initial value */
+
 	/* Copy the last in jb timing info to the pvt */
 	ast_copy_flags(&pvt->f, f, AST_FRFLAG_HAS_TIMING_INFO);
 	pvt->f.ts = f->ts;
@@ -351,7 +356,12 @@ static int framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
 	/* we require a framein routine, wouldn't know how to do
 	 * it otherwise.
 	 */
-	return pvt->t->framein(pvt, f);
+	ret = pvt->t->framein(pvt, f);
+	/* diagnostic ... */
+	if (pvt->samples == samples)
+		ast_log(LOG_WARNING, "%s did not update samples %d\n",
+			pvt->t->name, pvt->samples);
+	return ret;
 }
 
 /*! \brief generic frameout routine.
@@ -505,12 +515,7 @@ struct ast_frame *ast_translate(struct ast_trans_pvt *path, struct ast_frame *f,
 	}
 	delivery = f->delivery;
 	for (out = f; out && p ; p = p->next) {
-		struct ast_frame *current = out;
-
-		do {
-			framein(p, current);
-			current = AST_LIST_NEXT(current, frame_list);
-		} while (current);
+		framein(p, out);
 		if (out != f) {
 			ast_frfree(out);
 		}
@@ -519,32 +524,21 @@ struct ast_frame *ast_translate(struct ast_trans_pvt *path, struct ast_frame *f,
 	if (out) {
 		/* we have a frame, play with times */
 		if (!ast_tvzero(delivery)) {
-			struct ast_frame *current = out;
+			/* Regenerate prediction after a discontinuity */
+			if (ast_tvzero(path->nextout)) {
+				path->nextout = ast_tvnow();
+			}
 
-			do {
-				/* Regenerate prediction after a discontinuity */
-				if (ast_tvzero(path->nextout)) {
-					path->nextout = ast_tvnow();
-				}
+			/* Use next predicted outgoing timestamp */
+			out->delivery = path->nextout;
 
-				/* Use next predicted outgoing timestamp */
-				current->delivery = path->nextout;
-
-				/* Invalidate prediction if we're entering a silence period */
-				if (out->frametype == AST_FRAME_CNG) {
-					path->nextout = ast_tv(0, 0);
-				/* Predict next outgoing timestamp from samples in this
-				   frame. */
-				} else {
-					path->nextout = ast_tvadd(path->nextout, ast_samp2tv(current->samples, ast_format_rate(&current->subclass.format)));
-				}
-
-				if (f->samples != current->samples && ast_test_flag(current, AST_FRFLAG_HAS_TIMING_INFO)) {
-					ast_debug(4, "Sample size different %d vs %d\n", f->samples, current->samples);
-					ast_clear_flag(current, AST_FRFLAG_HAS_TIMING_INFO);
-				}
-				current = AST_LIST_NEXT(current, frame_list);
-			} while (current);
+			/* Predict next outgoing timestamp from samples in this
+			   frame. */
+			path->nextout = ast_tvadd(path->nextout, ast_samp2tv(out->samples, ast_format_rate(&out->subclass.format)));
+			if (f->samples != out->samples && ast_test_flag(out, AST_FRFLAG_HAS_TIMING_INFO)) {
+				ast_debug(4, "Sample size different %d vs %d\n", f->samples, out->samples);
+				ast_clear_flag(out, AST_FRFLAG_HAS_TIMING_INFO);
+			}
 		} else {
 			out->delivery = ast_tv(0, 0);
 			ast_set2_flag(out, has_timing_info, AST_FRFLAG_HAS_TIMING_INFO);
@@ -553,10 +547,10 @@ struct ast_frame *ast_translate(struct ast_trans_pvt *path, struct ast_frame *f,
 				out->len = len;
 				out->seqno = seqno;
 			}
-			/* Invalidate prediction if we're entering a silence period */
-			if (out->frametype == AST_FRAME_CNG) {
-				path->nextout = ast_tv(0, 0);
-			}
+		}
+		/* Invalidate prediction if we're entering a silence period */
+		if (out->frametype == AST_FRAME_CNG) {
+			path->nextout = ast_tv(0, 0);
 		}
 	}
 	if (consume) {
@@ -1201,87 +1195,72 @@ int ast_translator_best_choice(struct ast_format_cap *dst_cap,
 {
 	unsigned int besttablecost = INT_MAX;
 	unsigned int beststeps = INT_MAX;
-	struct ast_format cur_dst;
-	struct ast_format cur_src;
 	struct ast_format best;
 	struct ast_format bestdst;
-	struct ast_format_cap *joint_cap;
-
+	struct ast_format_cap *joint_cap = ast_format_cap_joint(dst_cap, src_cap);
 	ast_format_clear(&best);
+	ast_format_clear(&bestdst);
 
-	joint_cap = ast_format_cap_joint(dst_cap, src_cap);
 	if (joint_cap) { /* yes, pick one and return */
 		struct ast_format tmp_fmt;
-
 		ast_format_cap_iter_start(joint_cap);
 		while (!ast_format_cap_iter_next(joint_cap, &tmp_fmt)) {
-			if (AST_FORMAT_GET_TYPE(tmp_fmt.id) != AST_FORMAT_TYPE_AUDIO) {
+			/* We are guaranteed to find one common format. */
+			if (!best.id) {
+				ast_format_copy(&best, &tmp_fmt);
+				continue;
+			}
+			/* If there are multiple common formats, pick the one with the highest sample rate */
+			if (ast_format_rate(&best) < ast_format_rate(&tmp_fmt)) {
+				ast_format_copy(&best, &tmp_fmt);
 				continue;
 			}
 
-			if (!best.id
-				|| ast_format_rate(&best) < ast_format_rate(&tmp_fmt)) {
-				ast_format_copy(&best, &tmp_fmt);
-			}
 		}
 		ast_format_cap_iter_end(joint_cap);
-		ast_format_cap_destroy(joint_cap);
 
+		/* We are done, this is a common format to both. */
+		ast_format_copy(dst_fmt_out, &best);
+		ast_format_copy(src_fmt_out, &best);
+		ast_format_cap_destroy(joint_cap);
+		return 0;
+	} else {      /* No, we will need to translate */
+		struct ast_format cur_dst;
+		struct ast_format cur_src;
+		AST_RWLIST_RDLOCK(&translators);
+
+		ast_format_cap_iter_start(dst_cap);
+		while (!ast_format_cap_iter_next(dst_cap, &cur_dst)) {
+			ast_format_cap_iter_start(src_cap);
+			while (!ast_format_cap_iter_next(src_cap, &cur_src)) {
+				int x = format2index(cur_src.id);
+				int y = format2index(cur_dst.id);
+				if (x < 0 || y < 0) {
+					continue;
+				}
+				if (!matrix_get(x, y) || !(matrix_get(x, y)->step)) {
+					continue;
+				}
+				if (((matrix_get(x, y)->table_cost < besttablecost) || (matrix_get(x, y)->multistep < beststeps))) {
+					/* better than what we have so far */
+					ast_format_copy(&best, &cur_src);
+					ast_format_copy(&bestdst, &cur_dst);
+					besttablecost = matrix_get(x, y)->table_cost;
+					beststeps = matrix_get(x, y)->multistep;
+				}
+			}
+			ast_format_cap_iter_end(src_cap);
+		}
+
+		ast_format_cap_iter_end(dst_cap);
+		AST_RWLIST_UNLOCK(&translators);
 		if (best.id) {
-			/* We are done, this is a common format to both. */
-			ast_format_copy(dst_fmt_out, &best);
+			ast_format_copy(dst_fmt_out, &bestdst);
 			ast_format_copy(src_fmt_out, &best);
 			return 0;
 		}
-	}
-
-	/* No, we will need to translate */
-	ast_format_clear(&bestdst);
-
-	AST_RWLIST_RDLOCK(&translators);
-	ast_format_cap_iter_start(dst_cap);
-	while (!ast_format_cap_iter_next(dst_cap, &cur_dst)) {
-		if (AST_FORMAT_GET_TYPE(cur_dst.id) != AST_FORMAT_TYPE_AUDIO) {
-			continue;
-		}
-
-		ast_format_cap_iter_start(src_cap);
-		while (!ast_format_cap_iter_next(src_cap, &cur_src)) {
-			int x;
-			int y;
-
-			if (AST_FORMAT_GET_TYPE(cur_src.id) != AST_FORMAT_TYPE_AUDIO) {
-				continue;
-			}
-
-			x = format2index(cur_src.id);
-			y = format2index(cur_dst.id);
-			if (x < 0 || y < 0) {
-				continue;
-			}
-			if (!matrix_get(x, y) || !(matrix_get(x, y)->step)) {
-				continue;
-			}
-			if (matrix_get(x, y)->table_cost < besttablecost
-				|| matrix_get(x, y)->multistep < beststeps) {
-				/* better than what we have so far */
-				ast_format_copy(&best, &cur_src);
-				ast_format_copy(&bestdst, &cur_dst);
-				besttablecost = matrix_get(x, y)->table_cost;
-				beststeps = matrix_get(x, y)->multistep;
-			}
-		}
-		ast_format_cap_iter_end(src_cap);
-	}
-	ast_format_cap_iter_end(dst_cap);
-	AST_RWLIST_UNLOCK(&translators);
-
-	if (!best.id) {
 		return -1;
 	}
-	ast_format_copy(dst_fmt_out, &bestdst);
-	ast_format_copy(src_fmt_out, &best);
-	return 0;
 }
 
 unsigned int ast_translate_path_steps(struct ast_format *dst_format, struct ast_format *src_format)
@@ -1441,6 +1420,6 @@ int ast_translate_init(void)
 	ast_rwlock_init(&tablelock);
 	res = matrix_resize(1);
 	res |= ast_cli_register_multiple(cli_translate, ARRAY_LEN(cli_translate));
-	ast_register_cleanup(translate_shutdown);
+	ast_register_atexit(translate_shutdown);
 	return res;
 }
