@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 427275 $")
 
 #include <ctype.h>
 
@@ -80,6 +80,8 @@ static char *complete_dialplan_remove_context(struct ast_cli_args *);
 
 static char *handle_cli_dialplan_remove_context(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
+	struct ast_context *con;
+
 	switch (cmd) {
 	case CLI_INIT:
 		e->command = "dialplan remove context";
@@ -95,11 +97,16 @@ static char *handle_cli_dialplan_remove_context(struct ast_cli_entry *e, int cmd
 		return CLI_SHOWUSAGE;
 	}
 
-	if (ast_context_destroy_by_name(a->argv[3], NULL)) {
-		ast_cli(a->fd, "There is no such context as '%s'\n", a->argv[3]);
-		return CLI_SUCCESS;
+	con = ast_context_find(a->argv[3]);
+
+	if (!con) {
+		ast_cli(a->fd, "There is no such context as '%s'\n",
+                        a->argv[3]);
+                return CLI_SUCCESS;
 	} else {
-		ast_cli(a->fd, "Removed context '%s'\n", a->argv[3]);
+		ast_context_destroy(con, registrar);
+		ast_cli(a->fd, "Removing context '%s'\n",
+			a->argv[3]);
 		return CLI_SUCCESS;
 	}
 }
@@ -848,9 +855,7 @@ static char *handle_cli_dialplan_save(struct ast_cli_entry *e, int cmd, struct a
 					const char *el = ast_get_extension_label(p);
 					char label[128] = "";
 					char *appdata = ast_get_extension_app_data(p);
-
-					int escaped_len = (!ast_strlen_zero(appdata)) ? 2 * strlen(appdata) + 1 : 1;
-					char escaped[escaped_len];
+					char *escaped;
 
 					if (ast_get_extension_matchcid(p)) {
 						sep = "/";
@@ -864,9 +869,12 @@ static char *handle_cli_dialplan_save(struct ast_cli_entry *e, int cmd, struct a
 					}
 
 					if (!ast_strlen_zero(appdata)) {
+						int escaped_len = 2 * strlen(appdata) + 1;
+						char escaped[escaped_len];
+
 						ast_escape_semicolons(appdata, escaped, escaped_len);
 					} else {
-						escaped[0] = '\0';
+						escaped = "";
 					}
 
 					fprintf(output, "exten => %s%s%s,%d%s,%s(%s)\n",
