@@ -28,7 +28,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 410157 $")
 
 #include <sys/stat.h>
 #include <time.h>
@@ -100,14 +100,6 @@ struct outgoing {
 };
 
 #if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
-struct direntry {
-	AST_LIST_ENTRY(direntry) list;
-	time_t mtime;
-	char name[0];
-};
-
-static AST_LIST_HEAD_STATIC(dirlist, direntry);
-
 static void queue_file(const char *filename, time_t when);
 #endif
 
@@ -152,7 +144,7 @@ static struct outgoing *new_outgoing(const char *fn)
 		return NULL;
 	}
 
-	o->capabilities = ast_format_cap_alloc_nolock();
+	o->capabilities = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_NOLOCK);
 	if (!o->capabilities) {
 		free_outgoing(o);
 		return NULL;
@@ -330,10 +322,6 @@ static int remove_from_queue(struct outgoing *o, const char *status)
 	char newfn[256];
 	const char *bname;
 
-#if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
-	struct direntry *cur;
-#endif
-
 	if (!ast_test_flag(&o->options, SPOOL_FLAG_ALWAYS_DELETE)) {
 		struct stat current_file_status;
 
@@ -344,19 +332,6 @@ static int remove_from_queue(struct outgoing *o, const char *status)
 		}
 	}
 
-#if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
-	AST_LIST_LOCK(&dirlist);
-	AST_LIST_TRAVERSE_SAFE_BEGIN(&dirlist, cur, list) {
-		if (!strcmp(cur->name, o->fn)) {
-			AST_LIST_REMOVE_CURRENT(list);
-			ast_free(cur);
-			break;
-		}
-	}
-	AST_LIST_TRAVERSE_SAFE_END;
-	AST_LIST_UNLOCK(&dirlist);
-#endif
-
 	if (!ast_test_flag(&o->options, SPOOL_FLAG_ARCHIVE)) {
 		unlink(o->fn);
 		return 0;
@@ -400,14 +375,14 @@ static void *attempt_thread(void *data)
 		ast_verb(3, "Attempting call on %s/%s for application %s(%s) (Retry %d)\n", o->tech, o->dest, o->app, o->data, o->retries);
 		res = ast_pbx_outgoing_app(o->tech, o->capabilities, o->dest, o->waittime * 1000,
 			o->app, o->data, &reason, 2 /* wait to finish */, o->cid_num, o->cid_name,
-			o->vars, o->account, NULL);
+			o->vars, o->account, NULL, NULL);
 		o->vars = NULL;
 	} else {
 		ast_verb(3, "Attempting call on %s/%s for %s@%s:%d (Retry %d)\n", o->tech, o->dest, o->exten, o->context,o->priority, o->retries);
 		res = ast_pbx_outgoing_exten(o->tech, o->capabilities, o->dest,
 			o->waittime * 1000, o->context, o->exten, o->priority, &reason,
 			2 /* wait to finish */, o->cid_num, o->cid_name, o->vars, o->account, NULL,
-			ast_test_flag(&o->options, SPOOL_FLAG_EARLY_MEDIA));
+			ast_test_flag(&o->options, SPOOL_FLAG_EARLY_MEDIA), NULL);
 		o->vars = NULL;
 	}
 	if (res) {
@@ -512,6 +487,14 @@ static int scan_service(const char *fn, time_t now)
 	return 0;
 }
 
+#if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
+struct direntry {
+	AST_LIST_ENTRY(direntry) list;
+	time_t mtime;
+	char name[0];
+};
+
+static AST_LIST_HEAD_STATIC(dirlist, direntry);
 
 #if defined(HAVE_INOTIFY)
 /* Only one thread is accessing this list, so no lock is necessary */
@@ -519,8 +502,6 @@ static AST_LIST_HEAD_NOLOCK_STATIC(createlist, direntry);
 static AST_LIST_HEAD_NOLOCK_STATIC(openlist, direntry);
 #endif
 
-#if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
-
 static void queue_file(const char *filename, time_t when)
 {
 	struct stat st;
