@@ -23,15 +23,25 @@
  * \todo Support writing attendees
  */
 
+/*! \li \ref res_calendar.c uses the configuration file \ref calendar.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*! 
+ * \page calendar.conf calendar.conf
+ * \verbinclude calendar.conf.sample
+ */
+
 /*** MODULEINFO
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 428245 $")
 
 #include "asterisk/_private.h"
+#include "asterisk/channel.h"
 #include "asterisk/calendar.h"
 #include "asterisk/utils.h"
 #include "asterisk/astobj2.h"
@@ -736,7 +746,7 @@ static void *do_notify(void *data)
 		goto notify_cleanup;
 	}
 
-	if (ast_dial_append(dial, tech, dest) < 0) {
+	if (ast_dial_append(dial, tech, dest, NULL) < 0) {
 		ast_log(LOG_ERROR, "Could not append channel\n");
 		goto notify_cleanup;
 	}
@@ -744,7 +754,7 @@ static void *do_notify(void *data)
 	ast_dial_set_global_timeout(dial, event->owner->notify_waittime);
 	generate_random_string(buf, sizeof(buf));
 
-	if (!(chan = ast_channel_alloc(1, AST_STATE_DOWN, 0, 0, 0, 0, 0, 0, 0, "Calendar/%s-%s", event->owner->name, buf))) {
+	if (!(chan = ast_channel_alloc(1, AST_STATE_DOWN, 0, 0, 0, 0, 0, NULL, NULL, 0, "Calendar/%s-%s", event->owner->name, buf))) {
 		ast_log(LOG_ERROR, "Could not allocate notification channel\n");
 		goto notify_cleanup;
 	}
@@ -757,6 +767,8 @@ static void *do_notify(void *data)
 	/* clear native formats and set to slinear. write format is signlear so just use that to set it */
 	ast_format_cap_set(ast_channel_nativeformats(chan), ast_channel_writeformat(chan));
 
+	ast_channel_unlock(chan);
+
 	if (!(datastore = ast_datastore_alloc(&event_notification_datastore, NULL))) {
 		ast_log(LOG_ERROR, "Could not allocate datastore, notification not being sent!\n");
 		goto notify_cleanup;
@@ -1080,8 +1092,8 @@ static struct ast_custom_function calendar_busy_function = {
 static int add_event_to_list(struct eventlist *events, struct ast_calendar_event *event, time_t start, time_t end)
 {
 	struct evententry *entry, *iter;
-	long event_startdiff = labs(start - event->start);
-	long event_enddiff = labs(end - event->end);
+	int event_startdiff = abs(start - event->start);
+	int event_enddiff = abs(end - event->end);
 	int i = 0;
 
 	if (!(entry = ast_calloc(1, sizeof(*entry)))) {
@@ -1094,16 +1106,16 @@ static int add_event_to_list(struct eventlist *events, struct ast_calendar_event
 
 	if (start == end) {
 		AST_LIST_TRAVERSE_SAFE_BEGIN(events, iter, list) {
-			long startdiff = labs(iter->event->start - start);
+			int startdiff = abs(iter->event->start - start);
 
-			ast_debug(10, "Comparing %s with startdiff %ld to %s with startdiff %ld\n", event->summary, event_startdiff, iter->event->summary, startdiff);
+			ast_debug(10, "Comparing %s with startdiff %d to %s with startdiff %d\n", event->summary, event_startdiff, iter->event->summary, startdiff);
 			++i;
 			if (startdiff > event_startdiff) {
 				AST_LIST_INSERT_BEFORE_CURRENT(entry, list);
 				return i;
 			}
 			if (startdiff == event_startdiff) {
-				long enddiff = labs(iter->event->end - end);
+				int enddiff = abs(iter->event->end - end);
 
 				if (enddiff > event_enddiff) {
 					AST_LIST_INSERT_BEFORE_CURRENT(entry, list);
@@ -1827,8 +1839,6 @@ static int unload_module(void)
 	ast_mutex_unlock(&refreshlock);
 	pthread_join(refresh_thread, NULL);
 
-	ast_sched_context_destroy(sched);
-
 	AST_LIST_LOCK(&techs);
 	AST_LIST_TRAVERSE_SAFE_BEGIN(&techs, tech, list) {
 		ast_unload_resource(tech->module, 0);
@@ -1842,6 +1852,16 @@ static int unload_module(void)
 	return 0;
 }
 
+/*!
+ * \brief Load the module
+ *
+ * Module loading including tests for configuration or dependencies.
+ * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
+ * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
+ * configuration file or other non-critical problem return 
+ * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
+ */
 static int load_module(void)
 {
 	if (!(calendars = ao2_container_alloc(CALENDAR_BUCKETS, calendar_hash_fn, calendar_cmp_fn))) {
