@@ -22,7 +22,7 @@
  *
  * \author Mark Spencer <markster@digium.com>
  *
- * \extref Uses the OpenSSL library, available at
+ * Uses the OpenSSL library, available at
  *	http://www.openssl.org/
  */
 
@@ -33,7 +33,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413588 $")
 
 #include "asterisk/paths.h"	/* use ast_config_AST_KEY_DIR */
 #include <openssl/ssl.h>
@@ -532,7 +532,7 @@ static void md52sum(char *sum, unsigned char *md5)
 {
 	int x;
 	for (x = 0; x < 16; x++) {
-		sum += sprintf(sum, "%02hhx", *(md5++));
+		sum += sprintf(sum, "%02x", (unsigned)*(md5++));
 	}
 }
 
@@ -652,17 +652,13 @@ static int load_module(void)
 	} else {
 		crypto_load(-1, -1);
 	}
-
-	/* This prevents dlclose from ever running, but allows CLI cleanup at shutdown. */
-	ast_module_shutdown_ref(ast_module_info->self);
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 static int unload_module(void)
 {
-	ast_cli_unregister_multiple(cli_crypto, ARRAY_LEN(cli_crypto));
-
-	return 0;
+	/* Can't unload this once we're loaded */
+	return -1;
 }
 
 /* needs usecount semantics defined */
