@@ -22,10 +22,19 @@
  *
  * \author Tilghman Lesher <res_curl_v1@the-tilghman.com>
  *
- * \extref Depends on the CURL library  - http://curl.haxx.se/
+ * Depends on the CURL library  - http://curl.haxx.se/
  * 
  */
 
+/*! \li \ref res_curl.c uses the configuration file \ref res_curl.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*! 
+ * \page res_curl.conf res_curl.conf
+ * \verbinclude res_curl.conf.sample
+ */
+
 /*** MODULEINFO
 	<depend>curl</depend>
 	<support_level>core</support_level>
@@ -33,38 +42,54 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 389251 $")
 
 #include <curl/curl.h>
 
 #include "asterisk/module.h"
 
+static const char *dependents[] = {
+	"func_curl.so",
+	"res_config_curl.so",
+};
+
 static int unload_module(void)
 {
 	int res = 0;
+	size_t i;
 
 	/* If the dependent modules are still in memory, forbid unload */
-	if (ast_module_check("func_curl.so")) {
-		ast_log(LOG_ERROR, "func_curl.so (dependent module) is still loaded.  Cannot unload res_curl.so\n");
-		return -1;
+	for (i = 0; i < ARRAY_LEN(dependents); i++) {
+		if (ast_module_check(dependents[i])) {
+			ast_log(LOG_ERROR, "%s (dependent module) is still loaded.  Cannot unload res_curl.so\n", dependents[i]);
+			res = -1;
+		}
 	}
 
-	if (ast_module_check("res_config_curl.so")) {
-		ast_log(LOG_ERROR, "res_config_curl.so (dependent module) is still loaded.  Cannot unload res_curl.so\n");
+	if (res)
 		return -1;
-	}
 
 	curl_global_cleanup();
 
 	return res;
 }
 
+/*!
+ * \brief Load the module
+ *
+ * Module loading including tests for configuration or dependencies.
+ * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
+ * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
+ * configuration file or other non-critical problem return 
+ * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
+ */
 static int load_module(void)
 {
 	int res = 0;
 
 	if (curl_global_init(CURL_GLOBAL_ALL)) {
-		ast_log(LOG_ERROR, "Unable to initialize the CURL library. Cannot load res_curl\n");
+		ast_log(LOG_ERROR, "Unable to initialize the cURL library. Cannot load res_curl.so\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
