@@ -50,7 +50,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 423365 $")
 
 #define SPANDSP_EXPOSE_INTERNAL_STRUCTURES
 #include <spandsp.h>
@@ -498,7 +498,7 @@ static int spandsp_modems(struct ast_fax_session_details *details)
 	if (AST_FAX_MODEM_V17 & details->modems) {
 		modems |= T30_SUPPORT_V17;
 	}
-	if (AST_FAX_MODEM_V27TER & details->modems) {
+	if (AST_FAX_MODEM_V27 & details->modems) {
 		modems |= T30_SUPPORT_V27TER;
 	}
 	if (AST_FAX_MODEM_V29 & details->modems) {
@@ -587,10 +587,8 @@ e_return:
 	return NULL;
 }
 
-static void spandsp_v21_cleanup(struct ast_fax_session *s)
-{
+static void spandsp_v21_cleanup(struct ast_fax_session *s) {
 	struct spandsp_pvt *p = s->tech_pvt;
-
 	modem_connect_tones_rx_free(p->tone_state);
 }
 
@@ -666,8 +664,7 @@ static void spandsp_v21_tone(void *data, int code, int level, int delay)
 	}
 }
 
-static int spandsp_v21_detect(struct ast_fax_session *s, const struct ast_frame *f)
-{
+static int spandsp_v21_detect(struct ast_fax_session *s, const struct ast_frame *f) {
 	struct spandsp_pvt *p = s->tech_pvt;
 	int16_t *slndata;
 	g711_state_t *decoder;
@@ -791,26 +788,23 @@ static int spandsp_fax_gw_t30_gen(struct ast_channel *chan, void *data, int len,
  * \param chan channel
  * \param params generator data
  * \return data to use in generator call*/
-static void *spandsp_fax_gw_gen_alloc(struct ast_channel *chan, void *params)
-{
+static void *spandsp_fax_gw_gen_alloc(struct ast_channel *chan, void *params) {
 	ao2_ref(params, +1);
 	return params;
 }
 
-static void spandsp_fax_gw_gen_release(struct ast_channel *chan, void *data)
-{
+static void spandsp_fax_gw_gen_release(struct ast_channel *chan, void *data) {
 	ao2_ref(data, -1);
 }
 
 /*! \brief activate a spandsp gateway based on the information in the given fax session
  * \param s fax session
  * \return -1 on error 0 on sucess*/
-static int spandsp_fax_gateway_start(struct ast_fax_session *s)
-{
+static int spandsp_fax_gateway_start(struct ast_fax_session *s) {
 	struct spandsp_pvt *p = s->tech_pvt;
 	struct ast_fax_t38_parameters *t38_param;
 	int i;
-	struct ast_channel *peer;
+	RAII_VAR(struct ast_channel *, peer, NULL, ao2_cleanup);
 	static struct ast_generator t30_gen = {
 		.alloc = spandsp_fax_gw_gen_alloc,
 		.release = spandsp_fax_gw_gen_release,
@@ -831,14 +825,10 @@ static int spandsp_fax_gateway_start(struct ast_fax_session *s)
 
 	p->ist38 = 1;
 	p->ast_t38_state = ast_channel_get_t38_state(s->chan);
-	ast_channel_lock(s->chan);
-	peer = ast_bridged_channel(s->chan);
-	if (!peer) {
+	if (!(peer = ast_channel_bridge_peer(s->chan))) {
 		ast_channel_unlock(s->chan);
 		return -1;
 	}
-	ao2_ref(peer, +1);
-	ast_channel_unlock(s->chan);
 
 	/* we can be in T38_STATE_NEGOTIATING or T38_STATE_NEGOTIATED when the
 	 * gateway is started. We treat both states the same. */
@@ -847,7 +837,6 @@ static int spandsp_fax_gateway_start(struct ast_fax_session *s)
 	}
 
 	ast_activate_generator(p->ast_t38_state == T38_STATE_NEGOTIATED ? peer : s->chan, &t30_gen , s);
-	ao2_ref(peer, -1);
 
 	set_logging(&p->t38_gw_state.logging, s->details);
 	set_logging(&p->t38_core_state->logging, s->details);
