@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 429316 $")
 
 #include "asterisk/module.h"
 #include "asterisk/http.h"
@@ -48,7 +48,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #define MAX_PROTOCOL_BUCKETS 7
 
 /*! \brief Size of the pre-determined buffer for WebSocket frames */
-#define MAXIMUM_FRAME_SIZE 16384
+#define MAXIMUM_FRAME_SIZE 8192
 
 /*! \brief Default reconstruction size for multi-frame payload reconstruction. If exceeded the next frame will start a
  *         payload.
@@ -89,9 +89,6 @@ struct websocket_protocol {
 	ast_websocket_callback callback; /*!< Callback called when a new session is established */
 };
 
-/*! \brief Container for registered protocols */
-static struct ao2_container *protocols;
-
 /*! \brief Hashing function for protocols */
 static int protocol_hash_fn(const void *obj, const int flags)
 {
@@ -117,6 +114,53 @@ static void protocol_destroy_fn(void *obj)
 	ast_free(protocol->name);
 }
 
+/*! \brief Structure for a WebSocket server */
+struct ast_websocket_server {
+	struct ao2_container *protocols; /*!< Container for registered protocols */
+};
+
+static void websocket_server_internal_dtor(void *obj)
+{
+	struct ast_websocket_server *server = obj;
+	ao2_cleanup(server->protocols);
+	server->protocols = NULL;
+}
+
+static void websocket_server_dtor(void *obj)
+{
+	websocket_server_internal_dtor(obj);
+	ast_module_unref(ast_module_info->self);
+}
+
+static struct ast_websocket_server *websocket_server_create_impl(void (*dtor)(void *))
+{
+	RAII_VAR(struct ast_websocket_server *, server, NULL, ao2_cleanup);
+
+	server = ao2_alloc(sizeof(*server), dtor);
+	if (!server) {
+		return NULL;
+	}
+
+	server->protocols = ao2_container_alloc(MAX_PROTOCOL_BUCKETS, protocol_hash_fn, protocol_cmp_fn);
+	if (!server->protocols) {
+		return NULL;
+	}
+
+	ao2_ref(server, +1);
+	return server;
+}
+
+static struct ast_websocket_server *websocket_server_internal_create(void)
+{
+	return websocket_server_create_impl(websocket_server_internal_dtor);
+}
+
+struct ast_websocket_server *AST_OPTIONAL_API_NAME(ast_websocket_server_create)(void)
+{
+	ast_module_ref(ast_module_info->self);
+	return websocket_server_create_impl(websocket_server_dtor);
+}
+
 /*! \brief Destructor function for sessions */
 static void session_destroy_fn(void *obj)
 {
@@ -132,34 +176,38 @@ static void session_destroy_fn(void *obj)
 	ast_free(session->payload);
 }
 
-int AST_OPTIONAL_API_NAME(ast_websocket_add_protocol)(const char *name, ast_websocket_callback callback)
+int AST_OPTIONAL_API_NAME(ast_websocket_server_add_protocol)(struct ast_websocket_server *server, const char *name, ast_websocket_callback callback)
 {
 	struct websocket_protocol *protocol;
 
-	ao2_lock(protocols);
+	if (!server->protocols) {
+		return -1;
+	}
+
+	ao2_lock(server->protocols);
 
 	/* Ensure a second protocol handler is not registered for the same protocol */
-	if ((protocol = ao2_find(protocols, name, OBJ_KEY | OBJ_NOLOCK))) {
+	if ((protocol = ao2_find(server->protocols, name, OBJ_KEY | OBJ_NOLOCK))) {
 		ao2_ref(protocol, -1);
-		ao2_unlock(protocols);
+		ao2_unlock(server->protocols);
 		return -1;
 	}
 
 	if (!(protocol = ao2_alloc(sizeof(*protocol), protocol_destroy_fn))) {
-		ao2_unlock(protocols);
+		ao2_unlock(server->protocols);
 		return -1;
 	}
 
 	if (!(protocol->name = ast_strdup(name))) {
 		ao2_ref(protocol, -1);
-		ao2_unlock(protocols);
+		ao2_unlock(server->protocols);
 		return -1;
 	}
 
 	protocol->callback = callback;
 
-	ao2_link_flags(protocols, protocol, OBJ_NOLOCK);
-	ao2_unlock(protocols);
+	ao2_link_flags(server->protocols, protocol, OBJ_NOLOCK);
+	ao2_unlock(server->protocols);
 	ao2_ref(protocol, -1);
 
 	ast_verb(2, "WebSocket registered sub-protocol '%s'\n", name);
@@ -167,11 +215,11 @@ int AST_OPTIONAL_API_NAME(ast_websocket_add_protocol)(const char *name, ast_webs
 	return 0;
 }
 
-int AST_OPTIONAL_API_NAME(ast_websocket_remove_protocol)(const char *name, ast_websocket_callback callback)
+int AST_OPTIONAL_API_NAME(ast_websocket_server_remove_protocol)(struct ast_websocket_server *server, const char *name, ast_websocket_callback callback)
 {
 	struct websocket_protocol *protocol;
 
-	if (!(protocol = ao2_find(protocols, name, OBJ_KEY))) {
+	if (!(protocol = ao2_find(server->protocols, name, OBJ_KEY))) {
 		return -1;
 	}
 
@@ -180,7 +228,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_remove_protocol)(const char *name, ast_w
 		return -1;
 	}
 
-	ao2_unlink(protocols, protocol);
+	ao2_unlink(server->protocols, protocol);
 	ao2_ref(protocol, -1);
 
 	ast_verb(2, "WebSocket unregistered sub-protocol '%s'\n", name);
@@ -209,34 +257,21 @@ int AST_OPTIONAL_API_NAME(ast_websocket_close)(struct ast_websocket *session, ui
 
 	ao2_lock(session);
 	res = ast_careful_fwrite(session->f, session->fd, frame, 4, session->timeout);
-
-	/* If an error occurred when trying to close this connection explicitly terminate it now.
-	 * Doing so will cause the thread polling on it to wake up and terminate.
-	 */
-	if (res) {
-		fclose(session->f);
-		session->f = NULL;
-		ast_verb(2, "WebSocket connection from '%s' forcefully closed due to fatal write error\n",
-			ast_sockaddr_stringify(&session->address));
-	}
-
 	ao2_unlock(session);
-
 	return res;
 }
 
 
 /*! \brief Write function for websocket traffic */
-int AST_OPTIONAL_API_NAME(ast_websocket_write)(struct ast_websocket *session, enum ast_websocket_opcode opcode, char *payload, uint64_t payload_size)
+int AST_OPTIONAL_API_NAME(ast_websocket_write)(struct ast_websocket *session, enum ast_websocket_opcode opcode, char *payload, uint64_t actual_length)
 {
 	size_t header_size = 2; /* The minimum size of a websocket frame is 2 bytes */
 	char *frame;
-	uint64_t length;
-	uint64_t frame_size;
+	uint64_t length = 0;
 
-	if (payload_size < 126) {
-		length = payload_size;
-	} else if (payload_size < (1 << 16)) {
+	if (actual_length < 126) {
+		length = actual_length;
+	} else if (actual_length < (1 << 16)) {
 		length = 126;
 		/* We need an additional 2 bytes to store the extended length */
 		header_size += 2;
@@ -246,37 +281,33 @@ int AST_OPTIONAL_API_NAME(ast_websocket_write)(struct ast_websocket *session, en
 		header_size += 8;
 	}
 
-	frame_size = header_size + payload_size;
-
-	frame = ast_alloca(frame_size + 1);
-	memset(frame, 0, frame_size + 1);
+	frame = ast_alloca(header_size);
+	memset(frame, 0, sizeof(*frame));
 
 	frame[0] = opcode | 0x80;
 	frame[1] = length;
 
 	/* Use the additional available bytes to store the length */
 	if (length == 126) {
-		put_unaligned_uint16(&frame[2], htons(payload_size));
+		put_unaligned_uint16(&frame[2], htons(actual_length));
 	} else if (length == 127) {
-		put_unaligned_uint64(&frame[2], htonll(payload_size));
+		put_unaligned_uint64(&frame[2], htonl(actual_length));
 	}
 
-	memcpy(&frame[header_size], payload, payload_size);
-
 	ao2_lock(session);
 	if (session->closing) {
 		ao2_unlock(session);
 		return -1;
 	}
-
-	if (ast_careful_fwrite(session->f, session->fd, frame, frame_size, session->timeout)) {
+	if (ast_careful_fwrite(session->f, session->fd, frame, header_size, session->timeout)) {
 		ao2_unlock(session);
-		/* 1011 - server terminating connection due to not being able to fulfill the request */
-		ast_debug(1, "Closing WS with 1011 because we can't fulfill a write request\n");
-		ast_websocket_close(session, 1011);
 		return -1;
 	}
 
+	if (ast_careful_fwrite(session->f, session->fd, payload, actual_length, session->timeout)) {
+		ao2_unlock(session);
+		return -1;
+	}
 	fflush(session->f);
 	ao2_unlock(session);
 
@@ -300,7 +331,7 @@ void AST_OPTIONAL_API_NAME(ast_websocket_ref)(struct ast_websocket *session)
 
 void AST_OPTIONAL_API_NAME(ast_websocket_unref)(struct ast_websocket *session)
 {
-	ao2_ref(session, -1);
+	ao2_cleanup(session);
 }
 
 int AST_OPTIONAL_API_NAME(ast_websocket_fd)(struct ast_websocket *session)
@@ -372,45 +403,18 @@ int AST_OPTIONAL_API_NAME(ast_websocket_set_timeout)(struct ast_websocket *sessi
  */
 static inline int ws_safe_read(struct ast_websocket *session, char *buf, int len, enum ast_websocket_opcode *opcode)
 {
+	int sanity;
 	size_t rlen;
 	int xlen = len;
 	char *rbuf = buf;
-	int sanity = 10;
-
-	ao2_lock(session);
-	if (!session->f) {
-		ao2_unlock(session);
-		errno = ECONNABORTED;
-		return -1;
-	}
-
-	for (;;) {
+	for (sanity = 10; sanity; sanity--) {
 		clearerr(session->f);
 		rlen = fread(rbuf, 1, xlen, session->f);
-		if (!rlen) {
-			if (feof(session->f)) {
-				ast_log(LOG_WARNING, "Web socket closed abruptly\n");
-				*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
-				session->closing = 1;
-				ao2_unlock(session);
-				return -1;
-			}
-
-			if (ferror(session->f) && errno != EAGAIN) {
-				ast_log(LOG_ERROR, "Error reading from web socket: %s\n", strerror(errno));
-				*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
-				session->closing = 1;
-				ao2_unlock(session);
-				return -1;
-			}
-
-			if (!--sanity) {
-				ast_log(LOG_WARNING, "Websocket seems unresponsive, disconnecting ...\n");
-				*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
-				session->closing = 1;
-				ao2_unlock(session);
-				return -1;
-			}
+		if (!rlen && ferror(session->f) && errno != EAGAIN) {
+			ast_log(LOG_ERROR, "Error reading from web socket: %s\n", strerror(errno));
+			*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
+			session->closing = 1;
+			return -1;
 		}
 		xlen = xlen - rlen;
 		rbuf = rbuf + rlen;
@@ -421,12 +425,15 @@ static inline int ws_safe_read(struct ast_websocket *session, char *buf, int len
 			ast_log(LOG_ERROR, "ast_wait_for_input returned err: %s\n", strerror(errno));
 			*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
 			session->closing = 1;
-			ao2_unlock(session);
 			return -1;
 		}
 	}
-
-	ao2_unlock(session);
+	if (!sanity) {
+		ast_log(LOG_WARNING, "Websocket seems unresponsive, disconnecting ...\n");
+		*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
+		session->closing = 1;
+		return -1;
+	}
 	return 0;
 }
 
@@ -443,7 +450,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 	*fragmented = 0;
 
 	if (ws_safe_read(session, &buf[0], MIN_WS_HDR_SZ, opcode)) {
-		return -1;
+		return 0;
 	}
 	frame_size += MIN_WS_HDR_SZ;
 
@@ -461,7 +468,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 		if (options_len) {
 			/* read the rest of the header options */
 			if (ws_safe_read(session, &buf[frame_size], options_len, opcode)) {
-				return -1;
+				return 0;
 			}
 			frame_size += options_len;
 		}
@@ -490,7 +497,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 		}
 
 		if (ws_safe_read(session, *payload, *payload_len, opcode)) {
-			return -1;
+			return 0;
 		}
 
 		/* If a mask is present unmask the payload */
@@ -514,7 +521,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 					session->payload, session->payload_len, *payload_len);
 				*payload_len = 0;
 				ast_websocket_close(session, 1009);
-				return -1;
+				return 0;
 			}
 
 			session->payload = new_payload;
@@ -551,7 +558,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 		/* Make the payload available so the user can look at the reason code if they so desire */
 		if ((*payload_len) && (new_payload = ast_realloc(session->payload, *payload_len))) {
 			if (ws_safe_read(session, &buf[frame_size], (*payload_len), opcode)) {
-				return -1;
+				return 0;
 			}
 			session->payload = new_payload;
 			memcpy(session->payload, &buf[frame_size], *payload_len);
@@ -570,21 +577,53 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 	return 0;
 }
 
-/*! \brief Callback that is executed everytime an HTTP request is received by this module */
-static int websocket_callback(struct ast_tcptls_session_instance *ser, const struct ast_http_uri *urih, const char *uri, enum ast_http_method method, struct ast_variable *get_vars, struct ast_variable *headers)
+/*!
+ * \brief If the server has exactly one configured protocol, return it.
+ */
+static struct websocket_protocol *one_protocol(
+	struct ast_websocket_server *server)
+{
+	SCOPED_AO2LOCK(lock, server->protocols);
+
+	if (ao2_container_count(server->protocols) != 1) {
+		return NULL;
+	}
+
+	return ao2_callback(server->protocols, OBJ_NOLOCK, NULL, NULL);
+}
+
+static void websocket_bad_request(struct ast_tcptls_session_instance *ser)
+{
+	struct ast_str *http_header = ast_str_create(64);
+
+	if (!http_header) {
+		ast_http_request_close_on_completion(ser);
+		ast_http_error(ser, 500, "Server Error", "Out of memory");
+		return;
+	}
+	ast_str_set(&http_header, 0, "Sec-WebSocket-Version: 7, 8, 13\r\n");
+	ast_http_send(ser, AST_HTTP_UNKNOWN, 400, "Bad Request", http_header, NULL, 0, 0);
+}
+
+int AST_OPTIONAL_API_NAME(ast_websocket_uri_cb)(struct ast_tcptls_session_instance *ser, const struct ast_http_uri *urih, const char *uri, enum ast_http_method method, struct ast_variable *get_vars, struct ast_variable *headers)
 {
 	struct ast_variable *v;
 	char *upgrade = NULL, *key = NULL, *key1 = NULL, *key2 = NULL, *protos = NULL, *requested_protocols = NULL, *protocol = NULL;
 	int version = 0, flags = 1;
 	struct websocket_protocol *protocol_handler = NULL;
 	struct ast_websocket *session;
+	struct ast_websocket_server *server;
+
+	SCOPED_MODULE_USE(ast_module_info->self);
 
 	/* Upgrade requests are only permitted on GET methods */
 	if (method != AST_HTTP_GET) {
 		ast_http_error(ser, 501, "Not Implemented", "Attempt to use unimplemented / unsupported method");
-		return -1;
+		return 0;
 	}
 
+	server = urih->data;
+
 	/* Get the minimum headers required to satisfy our needs */
 	for (v = headers; v; v = v->next) {
 		if (!strcasecmp(v->name, "Upgrade")) {
@@ -610,36 +649,38 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - did not request WebSocket\n",
 			ast_sockaddr_stringify(&ser->remote_address));
 		ast_http_error(ser, 426, "Upgrade Required", NULL);
-		return -1;
+		return 0;
 	} else if (ast_strlen_zero(requested_protocols)) {
-		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - no protocols requested\n",
-			ast_sockaddr_stringify(&ser->remote_address));
-		fputs("HTTP/1.1 400 Bad Request\r\n"
-		      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
-		return -1;
+		/* If there's only a single protocol registered, and the
+		 * client doesn't specify what protocol it's using, go ahead
+		 * and accept the connection */
+		protocol_handler = one_protocol(server);
+		if (!protocol_handler) {
+			/* Multiple registered subprotocols; client must specify */
+			ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - no protocols requested\n",
+				ast_sockaddr_stringify(&ser->remote_address));
+			websocket_bad_request(ser);
+			return 0;
+		}
 	} else if (key1 && key2) {
 		/* Specification defined in http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-76 and
 		 * http://tools.ietf.org/html/draft-ietf-hybi-thewebsocketprotocol-00 -- not currently supported*/
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - unsupported version '00/76' chosen\n",
 			ast_sockaddr_stringify(&ser->remote_address));
-		fputs("HTTP/1.1 400 Bad Request\r\n"
-		      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+		websocket_bad_request(ser);
 		return 0;
 	}
 
 	/* Iterate through the requested protocols trying to find one that we have a handler for */
-	while ((protocol = strsep(&requested_protocols, ","))) {
-		if ((protocol_handler = ao2_find(protocols, ast_strip(protocol), OBJ_KEY))) {
-			break;
-		}
+	while (!protocol_handler && (protocol = strsep(&requested_protocols, ","))) {
+		protocol_handler = ao2_find(server->protocols, ast_strip(protocol), OBJ_KEY);
 	}
 
 	/* If no protocol handler exists bump this back to the requester */
 	if (!protocol_handler) {
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - no protocols out of '%s' supported\n",
 			ast_sockaddr_stringify(&ser->remote_address), protos);
-		fputs("HTTP/1.1 400 Bad Request\r\n"
-		      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+		websocket_bad_request(ser);
 		return 0;
 	}
 
@@ -654,8 +695,13 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 
 		combined_length = (key ? strlen(key) : 0) + strlen(WEBSOCKET_GUID) + 1;
 		if (!key || combined_length > 8192) { /* no stack overflows please */
-			fputs("HTTP/1.1 400 Bad Request\r\n"
-			      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+			websocket_bad_request(ser);
+			ao2_ref(protocol_handler, -1);
+			return 0;
+		}
+
+		if (ast_http_body_discard(ser)) {
+			websocket_bad_request(ser);
 			ao2_ref(protocol_handler, -1);
 			return 0;
 		}
@@ -663,12 +709,11 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 		if (!(session = ao2_alloc(sizeof(*session), session_destroy_fn))) {
 			ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted\n",
 				ast_sockaddr_stringify(&ser->remote_address));
-			fputs("HTTP/1.1 400 Bad Request\r\n"
-			      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+			websocket_bad_request(ser);
 			ao2_ref(protocol_handler, -1);
 			return 0;
 		}
-		session->timeout = AST_DEFAULT_WEBSOCKET_WRITE_TIMEOUT;
+		session->timeout =  AST_DEFAULT_WEBSOCKET_WRITE_TIMEOUT;
 
 		combined = ast_alloca(combined_length);
 		snprintf(combined, combined_length, "%s%s", key, WEBSOCKET_GUID);
@@ -678,19 +723,32 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 		fprintf(ser->f, "HTTP/1.1 101 Switching Protocols\r\n"
 			"Upgrade: %s\r\n"
 			"Connection: Upgrade\r\n"
-			"Sec-WebSocket-Accept: %s\r\n"
-			"Sec-WebSocket-Protocol: %s\r\n\r\n",
+			"Sec-WebSocket-Accept: %s\r\n",
 			upgrade,
-			base64,
-			protocol);
+			base64);
+
+		/* RFC 6455, Section 4.1:
+		 *
+		 * 6. If the response includes a |Sec-WebSocket-Protocol| header
+		 *    field and this header field indicates the use of a
+		 *    subprotocol that was not present in the client's handshake
+		 *    (the server has indicated a subprotocol not requested by
+		 *    the client), the client MUST _Fail the WebSocket
+		 *    Connection_.
+		 */
+		if (protocol) {
+			fprintf(ser->f, "Sec-WebSocket-Protocol: %s\r\n",
+				protocol);
+		}
+
+		fprintf(ser->f, "\r\n");
 		fflush(ser->f);
 	} else {
 
 		/* Specification defined in http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-75 or completely unknown */
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - unsupported version '%d' chosen\n",
 			ast_sockaddr_stringify(&ser->remote_address), version ? version : 75);
-		fputs("HTTP/1.1 400 Bad Request\r\n"
-		      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+		websocket_bad_request(ser);
 		ao2_ref(protocol_handler, -1);
 		return 0;
 	}
@@ -699,14 +757,13 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 	if (setsockopt(ser->fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - failed to enable keepalive\n",
 			ast_sockaddr_stringify(&ser->remote_address));
-		fputs("HTTP/1.1 400 Bad Request\r\n"
-		      "Sec-WebSocket-Version: 7, 8, 13\r\n\r\n", ser->f);
+		websocket_bad_request(ser);
 		ao2_ref(session, -1);
 		ao2_ref(protocol_handler, -1);
 		return 0;
 	}
 
-	ast_verb(2, "WebSocket connection from '%s' for protocol '%s' accepted using version '%d'\n", ast_sockaddr_stringify(&ser->remote_address), protocol, version);
+	ast_verb(2, "WebSocket connection from '%s' for protocol '%s' accepted using version '%d'\n", ast_sockaddr_stringify(&ser->remote_address), protocol ? : "", version);
 
 	/* Populate the session with all the needed details */
 	session->f = ser->f;
@@ -717,6 +774,7 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 	session->secure = ser->ssl ? 1 : 0;
 
 	/* Give up ownership of the socket and pass it to the protocol handler */
+	ast_tcptls_stream_set_exclusive_input(ser->stream_cookie, 0);
 	protocol_handler->callback(session, get_vars, headers);
 	ao2_ref(protocol_handler, -1);
 
@@ -732,7 +790,7 @@ static int websocket_callback(struct ast_tcptls_session_instance *ser, const str
 }
 
 static struct ast_http_uri websocketuri = {
-	.callback = websocket_callback,
+	.callback = AST_OPTIONAL_API_NAME(ast_websocket_uri_cb),
 	.description = "Asterisk HTTP WebSocket",
 	.uri = "ws",
 	.has_subtree = 0,
@@ -783,20 +841,60 @@ end:
 	ast_websocket_unref(session);
 }
 
+static int websocket_add_protocol_internal(const char *name, ast_websocket_callback callback)
+{
+	struct ast_websocket_server *ws_server = websocketuri.data;
+	if (!ws_server) {
+		return -1;
+	}
+	return ast_websocket_server_add_protocol(ws_server, name, callback);
+}
+
+int AST_OPTIONAL_API_NAME(ast_websocket_add_protocol)(const char *name, ast_websocket_callback callback)
+{
+	int res = websocket_add_protocol_internal(name, callback);
+	if (res == 0) {
+		ast_module_ref(ast_module_info->self);
+	}
+	return res;
+}
+
+static int websocket_remove_protocol_internal(const char *name, ast_websocket_callback callback)
+{
+	struct ast_websocket_server *ws_server = websocketuri.data;
+	if (!ws_server) {
+		return -1;
+	}
+	return ast_websocket_server_remove_protocol(ws_server, name, callback);
+}
+
+int AST_OPTIONAL_API_NAME(ast_websocket_remove_protocol)(const char *name, ast_websocket_callback callback)
+{
+	int res = websocket_remove_protocol_internal(name, callback);
+	if (res == 0) {
+		ast_module_unref(ast_module_info->self);
+	}
+	return res;
+}
+
 static int load_module(void)
 {
-	protocols = ao2_container_alloc(MAX_PROTOCOL_BUCKETS, protocol_hash_fn, protocol_cmp_fn);
+	websocketuri.data = websocket_server_internal_create();
+	if (!websocketuri.data) {
+		return AST_MODULE_LOAD_FAILURE;
+	}
 	ast_http_uri_link(&websocketuri);
-	ast_websocket_add_protocol("echo", websocket_echo_callback);
+	websocket_add_protocol_internal("echo", websocket_echo_callback);
 
 	return 0;
 }
 
 static int unload_module(void)
 {
-	ast_websocket_remove_protocol("echo", websocket_echo_callback);
+	websocket_remove_protocol_internal("echo", websocket_echo_callback);
 	ast_http_uri_unlink(&websocketuri);
-	ao2_ref(protocols, -1);
+	ao2_ref(websocketuri.data, -1);
+	websocketuri.data = NULL;
 
 	return 0;
 }
