@@ -29,7 +29,7 @@
  
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 429032 $")
 
 #include <sys/stat.h>
 #include <libgen.h>
@@ -40,8 +40,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
-#include "asterisk/manager.h"
 #include "asterisk/cli.h"
+#include "asterisk/manager.h"
+#include "asterisk/stasis.h"
+#include "asterisk/stasis_channels.h"
 #define AST_API_MODULE
 #include "asterisk/monitor.h"
 #include "asterisk/app.h"
@@ -57,17 +59,17 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		<syntax>
 			<parameter name="file_format" argsep=":">
 				<argument name="file_format" required="true">
-					<para>Optional.  If not set, defaults to <literal>wav</literal></para>
+					<para>optional, if not set, defaults to <literal>wav</literal></para>
 				</argument>
 				<argument name="urlbase" />
 			</parameter>
 			<parameter name="fname_base">
-				<para>If set, changes the filename used to the one specified.</para>
+				<para>if set, changes the filename used to the one specified.</para>
 			</parameter>
 			<parameter name="options">
 				<optionlist>
 					<option name="m">
-						<para>When the recording ends mix the two leg files into one and
+						<para>when the recording ends mix the two leg files into one and
 						delete the two leg files. If the variable <variable>MONITOR_EXEC</variable>
 						is set, the application referenced in it will be executed instead of
 						soxmix/sox and the raw leg files will NOT be deleted automatically.
@@ -78,13 +80,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 						will be passed on as additional arguments to <variable>MONITOR_EXEC</variable>.
 						Both <variable>MONITOR_EXEC</variable> and the Mix flag can be set from the
 						administrator interface.</para>
-						<warning><para>Do not use untrusted strings such as
-						<variable>CALLERID(num)</variable> or <variable>CALLERID(name)</variable>
-						as part of <variable>MONITOR_EXEC</variable> or
-						<variable>MONITOR_EXEC_ARGS</variable>.  You risk a command injection
-						attack executing arbitrary commands if the untrusted strings aren't
-						filtered to remove dangerous characters.  See function
-						<variable>FILTER()</variable>.</para></warning>
 					</option>
 					<option name="b">
 						<para>Don't begin recording unless a call is bridged to another channel.</para>
@@ -298,6 +293,7 @@ int AST_OPTIONAL_API_NAME(ast_monitor_start)(struct ast_channel *chan, const cha
 					     const char *fname_base, int need_lock, int stream_action)
 {
 	int res = 0;
+	RAII_VAR(struct stasis_message *, message, NULL, ao2_cleanup);
 
 	LOCK_IF_NEEDED(chan, need_lock);
 
@@ -402,11 +398,12 @@ int AST_OPTIONAL_API_NAME(ast_monitor_start)(struct ast_channel *chan, const cha
 		/* so we know this call has been monitored in case we need to bill for it or something */
 		pbx_builtin_setvar_helper(chan, "__MONITORED","true");
 
-		ast_manager_event(chan, EVENT_FLAG_CALL, "MonitorStart",
-			                "Channel: %s\r\n"
-					        "Uniqueid: %s\r\n",
-	                        ast_channel_name(chan),
-			                ast_channel_uniqueid(chan));
+		message = ast_channel_blob_create_from_cache(ast_channel_uniqueid(chan),
+				ast_channel_monitor_start_type(),
+				NULL);
+		if (message) {
+			stasis_publish(ast_channel_topic(chan), message);
+		}
 	} else {
 		ast_debug(1,"Cannot start monitoring %s, already monitored\n", ast_channel_name(chan));
 		res = -1;
@@ -446,6 +443,7 @@ static const char *get_soxmix_format(const char *format)
 int AST_OPTIONAL_API_NAME(ast_monitor_stop)(struct ast_channel *chan, int need_lock)
 {
 	int delfiles = 0;
+	RAII_VAR(struct stasis_message *, message, NULL, ao2_cleanup);
 
 	LOCK_IF_NEEDED(chan, need_lock);
 
@@ -460,28 +458,24 @@ int AST_OPTIONAL_API_NAME(ast_monitor_stop)(struct ast_channel *chan, int need_l
 		}
 
 		if (ast_channel_monitor(chan)->filename_changed && !ast_strlen_zero(ast_channel_monitor(chan)->filename_base)) {
-			if (ast_channel_monitor(chan)->read_stream) {
-				if (ast_fileexists(ast_channel_monitor(chan)->read_filename,NULL,NULL) > 0) {
-					snprintf(filename, FILENAME_MAX, "%s-in", ast_channel_monitor(chan)->filename_base);
-					if (ast_fileexists(filename, NULL, NULL) > 0) {
-						ast_filedelete(filename, NULL);
-					}
-					ast_filerename(ast_channel_monitor(chan)->read_filename, filename, ast_channel_monitor(chan)->format);
-				} else {
-					ast_log(LOG_WARNING, "File %s not found\n", ast_channel_monitor(chan)->read_filename);
+			if (ast_fileexists(ast_channel_monitor(chan)->read_filename,NULL,NULL) > 0) {
+				snprintf(filename, FILENAME_MAX, "%s-in", ast_channel_monitor(chan)->filename_base);
+				if (ast_fileexists(filename, NULL, NULL) > 0) {
+					ast_filedelete(filename, NULL);
 				}
+				ast_filerename(ast_channel_monitor(chan)->read_filename, filename, ast_channel_monitor(chan)->format);
+			} else {
+				ast_log(LOG_WARNING, "File %s not found\n", ast_channel_monitor(chan)->read_filename);
 			}
 
-			if (ast_channel_monitor(chan)->write_stream) {
-				if (ast_fileexists(ast_channel_monitor(chan)->write_filename,NULL,NULL) > 0) {
-					snprintf(filename, FILENAME_MAX, "%s-out", ast_channel_monitor(chan)->filename_base);
-					if (ast_fileexists(filename, NULL, NULL) > 0) {
-						ast_filedelete(filename, NULL);
-					}
-					ast_filerename(ast_channel_monitor(chan)->write_filename, filename, ast_channel_monitor(chan)->format);
-				} else {
-					ast_log(LOG_WARNING, "File %s not found\n", ast_channel_monitor(chan)->write_filename);
+			if (ast_fileexists(ast_channel_monitor(chan)->write_filename,NULL,NULL) > 0) {
+				snprintf(filename, FILENAME_MAX, "%s-out", ast_channel_monitor(chan)->filename_base);
+				if (ast_fileexists(filename, NULL, NULL) > 0) {
+					ast_filedelete(filename, NULL);
 				}
+				ast_filerename(ast_channel_monitor(chan)->write_filename, filename, ast_channel_monitor(chan)->format);
+			} else {
+				ast_log(LOG_WARNING, "File %s not found\n", ast_channel_monitor(chan)->write_filename);
 			}
 		}
 
@@ -524,12 +518,12 @@ int AST_OPTIONAL_API_NAME(ast_monitor_stop)(struct ast_channel *chan, int need_l
 		ast_free(ast_channel_monitor(chan));
 		ast_channel_monitor_set(chan, NULL);
 
-		ast_manager_event(chan, EVENT_FLAG_CALL, "MonitorStop",
-			                "Channel: %s\r\n"
-	                        "Uniqueid: %s\r\n",
-	                        ast_channel_name(chan),
-	                        ast_channel_uniqueid(chan)
-	                        );
+		message = ast_channel_blob_create_from_cache(ast_channel_uniqueid(chan),
+				ast_channel_monitor_stop_type(),
+				NULL);
+		if (message) {
+			stasis_publish(ast_channel_topic(chan), message);
+		}
 		pbx_builtin_setvar_helper(chan, "MONITORED", NULL);
 	}
 	pbx_builtin_setvar_helper(chan, "AUTO_MONITOR", NULL);
@@ -700,18 +694,10 @@ static int start_monitor_exec(struct ast_channel *chan, const char *data)
 	}
 
 	if (!ast_strlen_zero(urlprefix) && !ast_strlen_zero(args.fname_base)) {
-		struct ast_cdr *chan_cdr;
 		snprintf(tmp, sizeof(tmp), "%s/%s.%s", urlprefix, args.fname_base,
 			((strcmp(args.format, "gsm")) ? "wav" : "gsm"));
 		ast_channel_lock(chan);
-		if (!ast_channel_cdr(chan)) {
-			if (!(chan_cdr = ast_cdr_alloc())) {
-				ast_channel_unlock(chan);
-				return -1;
-			}
-			ast_channel_cdr_set(chan, chan_cdr);
-		}
-		ast_cdr_setuserfield(chan, tmp);
+		ast_cdr_setuserfield(ast_channel_name(chan), tmp);
 		ast_channel_unlock(chan);
 	}
 	if (waitforbridge) {
