@@ -19,12 +19,19 @@
 /*! \file
  *
  * \brief Routines implementing music on hold
- *
- * \arg See also \ref Config_moh
  * 
  * \author Mark Spencer <markster@digium.com>
  */
 
+/*! \li \ref res_musiconhold.c uses the configuration file \ref musiconhold.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*! 
+ * \page musiconhold.conf musiconhold.conf
+ * \verbinclude musiconhold.conf.sample
+ */
+
 /*** MODULEINFO
 	<conflict>win32</conflict>
 	<support_level>core</support_level>
@@ -32,7 +39,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 421978 $")
 
 #include <ctype.h>
 #include <signal.h>
@@ -60,16 +67,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/cli.h"
 #include "asterisk/stringfields.h"
 #include "asterisk/linkedlists.h"
-#include "asterisk/manager.h"
+#include "asterisk/stasis.h"
+#include "asterisk/stasis_channels.h"
 #include "asterisk/paths.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/timing.h"
 #include "asterisk/time.h"
 #include "asterisk/poll-compat.h"
 
-/* In order to gain access to ast_module_reload in 11 you need to include _private.h */
-#include "asterisk/_private.h"
-
 #define INITIAL_NUM_FILES   8
 #define HANDLE_REF	1
 #define DONT_UNREF	0
@@ -255,6 +260,48 @@ static struct mohclass *_mohclass_unref(struct mohclass *class, const char *tag,
 }
 #endif
 
+static void moh_post_start(struct ast_channel *chan, const char *moh_class_name)
+{
+	struct stasis_message *message;
+	struct ast_json *json_object;
+
+	ast_verb(3, "Started music on hold, class '%s', on channel '%s'\n",
+		moh_class_name, ast_channel_name(chan));
+
+	json_object = ast_json_pack("{s: s}", "class", moh_class_name);
+	if (!json_object) {
+		return;
+	}
+
+	message = ast_channel_blob_create_from_cache(ast_channel_uniqueid(chan),
+		ast_channel_moh_start_type(), json_object);
+	if (message) {
+		/* A channel snapshot must have been in the cache. */
+		ast_assert(((struct ast_channel_blob *) stasis_message_data(message))->snapshot != NULL);
+
+		stasis_publish(ast_channel_topic(chan), message);
+	}
+	ao2_cleanup(message);
+	ast_json_unref(json_object);
+}
+
+static void moh_post_stop(struct ast_channel *chan)
+{
+	struct stasis_message *message;
+
+	ast_verb(3, "Stopped music on hold on %s\n", ast_channel_name(chan));
+
+	message = ast_channel_blob_create_from_cache(ast_channel_uniqueid(chan),
+		ast_channel_moh_stop_type(), NULL);
+	if (message) {
+		/* A channel snapshot must have been in the cache. */
+		ast_assert(((struct ast_channel_blob *) stasis_message_data(message))->snapshot != NULL);
+
+		stasis_publish(ast_channel_topic(chan), message);
+	}
+	ao2_cleanup(message);
+}
+
 static void moh_files_release(struct ast_channel *chan, void *data)
 {
 	struct moh_files_state *state;
@@ -269,8 +316,8 @@ static void moh_files_release(struct ast_channel *chan, void *data)
 		ast_closestream(ast_channel_stream(chan));
 		ast_channel_stream_set(chan, NULL);
 	}
-	
-	ast_verb(3, "Stopped music on hold on %s\n", ast_channel_name(chan));
+
+	moh_post_stop(chan);
 
 	ast_format_clear(&state->mohwfmt); /* make sure to clear this format before restoring the original format. */
 	if (state->origwfmt.id && ast_set_write_format(chan, &state->origwfmt)) {
@@ -443,11 +490,11 @@ static void *moh_files_alloc(struct ast_channel *chan, void *params)
 	struct moh_files_state *state;
 	struct mohclass *class = params;
 
-	if (!ast_channel_music_state(chan) && (state = ast_calloc(1, sizeof(*state)))) {
+	state = ast_channel_music_state(chan);
+	if (!state && (state = ast_calloc(1, sizeof(*state)))) {
 		ast_channel_music_state_set(chan, state);
 		ast_module_ref(ast_module_info->self);
 	} else {
-		state = ast_channel_music_state(chan);
 		if (!state) {
 			return NULL;
 		}
@@ -473,10 +520,9 @@ static void *moh_files_alloc(struct ast_channel *chan, void *params)
 	ast_copy_string(state->name, class->name, sizeof(state->name));
 	state->save_total = class->total_files;
 
+	moh_post_start(chan, class->name);
 
-	ast_verb(3, "Started music on hold, class '%s', on %s\n", class->name, ast_channel_name(chan));
-	
-	return ast_channel_music_state(chan);
+	return state;
 }
 
 static int moh_digit_match(void *obj, void *arg, int flags)
@@ -507,8 +553,7 @@ static void moh_handle_digit(struct ast_channel *chan, char digit)
 	}
 }
 
-static struct ast_generator moh_file_stream = 
-{
+static struct ast_generator moh_file_stream = {
 	.alloc    = moh_files_alloc,
 	.release  = moh_files_release,
 	.generate = moh_files_generator,
@@ -896,7 +941,7 @@ static struct mohclass *_get_mohbyname(const char *name, int warn, int flags, co
 #endif
 
 	if (!moh && warn) {
-		ast_log(LOG_WARNING, "Music on Hold class '%s' not found in memory. Verify your configuration.\n", name);
+		ast_debug(1, "Music on Hold class '%s' not found in memory\n", name);
 	}
 
 	return moh;
@@ -966,7 +1011,7 @@ static void moh_release(struct ast_channel *chan, void *data)
 					ast_channel_name(chan), ast_getformatname(&oldwfmt));
 		}
 
-		ast_verb(3, "Stopped music on hold on %s\n", ast_channel_name(chan));
+		moh_post_stop(chan);
 	}
 }
 
@@ -977,11 +1022,11 @@ static void *moh_alloc(struct ast_channel *chan, void *params)
 	struct moh_files_state *state;
 
 	/* Initiating music_state for current channel. Channel should know name of moh class */
-	if (!ast_channel_music_state(chan) && (state = ast_calloc(1, sizeof(*state)))) {
+	state = ast_channel_music_state(chan);
+	if (!state && (state = ast_calloc(1, sizeof(*state)))) {
 		ast_channel_music_state_set(chan, state);
 		ast_module_ref(ast_module_info->self);
 	} else {
-		state = ast_channel_music_state(chan);
 		if (!state) {
 			return NULL;
 		}
@@ -1000,8 +1045,8 @@ static void *moh_alloc(struct ast_channel *chan, void *params)
 			res = NULL;
 		} else {
 			state->class = mohclass_ref(class, "Placing reference into state container");
+			moh_post_start(chan, class->name);
 		}
-		ast_verb(3, "Started music on hold, class '%s', on channel '%s'\n", class->name, ast_channel_name(chan));
 	}
 	return res;
 }
@@ -1258,7 +1303,7 @@ static int init_app_class(struct mohclass *class)
 /*!
  * \note This function owns the reference it gets to moh if unref is true
  */
-#define moh_register(a,b,c)	_moh_register(a,b,c,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+#define moh_register(moh, reload, unref) _moh_register(moh, reload, unref, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 static int _moh_register(struct mohclass *moh, int reload, int unref, const char *file, int line, const char *funcname)
 {
 	struct mohclass *mohclass = NULL;
@@ -1357,18 +1402,6 @@ static struct mohclass *_moh_class_malloc(const char *file, int line, const char
 	return class;
 }
 
-static struct ast_variable *load_realtime_musiconhold(const char *name)
-{
-	struct ast_variable *var = ast_load_realtime("musiconhold", "name", name, SENTINEL);
-	if (!var) {
-		ast_log(LOG_WARNING,
-			"Music on Hold class '%s' not found in memory/database. "
-			"Verify your configuration.\n",
-			name);
-	}
-	return var;
-}
-
 static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, const char *interpclass)
 {
 	struct mohclass *mohclass = NULL;
@@ -1376,7 +1409,6 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 	struct ast_variable *var = NULL;
 	int res = 0;
 	int realtime_possible = ast_check_realtime("musiconhold");
-	int warn_if_not_in_memory = !realtime_possible;
 
 	/* The following is the order of preference for which class to use:
 	 * 1) The channels explicitly set musicclass, which should *only* be
@@ -1390,28 +1422,28 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 	 * 4) The default class.
 	 */
 	if (!ast_strlen_zero(ast_channel_musicclass(chan))) {
-		mohclass = get_mohbyname(ast_channel_musicclass(chan), warn_if_not_in_memory, 0);
+		mohclass = get_mohbyname(ast_channel_musicclass(chan), 1, 0);
 		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(ast_channel_musicclass(chan));
+			var = ast_load_realtime("musiconhold", "name", ast_channel_musicclass(chan), SENTINEL);
 		}
 	}
 	if (!mohclass && !var && !ast_strlen_zero(mclass)) {
-		mohclass = get_mohbyname(mclass, warn_if_not_in_memory, 0);
+		mohclass = get_mohbyname(mclass, 1, 0);
 		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(mclass);
+			var = ast_load_realtime("musiconhold", "name", mclass, SENTINEL);
 		}
 	}
 	if (!mohclass && !var && !ast_strlen_zero(interpclass)) {
-		mohclass = get_mohbyname(interpclass, warn_if_not_in_memory, 0);
+		mohclass = get_mohbyname(interpclass, 1, 0);
 		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(interpclass);
+			var = ast_load_realtime("musiconhold", "name", interpclass, SENTINEL);
 		}
 	}
 
 	if (!mohclass && !var) {
-		mohclass = get_mohbyname("default", warn_if_not_in_memory, 0);
+		mohclass = get_mohbyname("default", 1, 0);
 		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold("default");
+			var = ast_load_realtime("musiconhold", "name", "default", SENTINEL);
 		}
 	}
 
@@ -1563,16 +1595,6 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 		}
 	}
 
-	ast_manager_event(chan, EVENT_FLAG_CALL, "MusicOnHold",
-		"State: Start\r\n"
-		"Channel: %s\r\n"
-		"UniqueID: %s\r\n"
-		"Class: %s\r\n",
-		ast_channel_name(chan), ast_channel_uniqueid(chan),
-		mohclass->name);
-
-	ast_set_flag(ast_channel_flags(chan), AST_FLAG_MOH);
-
 	if (!state || !state->class || strcmp(mohclass->name, state->class->name)) {
 		if (mohclass->total_files) {
 			res = ast_activate_generator(chan, &moh_file_stream, mohclass);
@@ -1580,6 +1602,10 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 			res = ast_activate_generator(chan, &mohgen, mohclass);
 		}
 	}
+	if (!res) {
+		ast_channel_latest_musicclass_set(chan, mohclass->name);
+		ast_set_flag(ast_channel_flags(chan), AST_FLAG_MOH);
+	}
 
 	mohclass = mohclass_unref(mohclass, "unreffing local reference to mohclass in local_ast_moh_start");
 
@@ -1598,12 +1624,6 @@ static void local_ast_moh_stop(struct ast_channel *chan)
 			ast_channel_stream_set(chan, NULL);
 		}
 	}
-
-	ast_manager_event(chan, EVENT_FLAG_CALL, "MusicOnHold",
-		"State: Stop\r\n"
-		"Channel: %s\r\n"
-		"UniqueID: %s\r\n",
-		ast_channel_name(chan), ast_channel_uniqueid(chan));
 	ast_channel_unlock(chan);
 }
 
@@ -1852,8 +1872,7 @@ static char *handle_cli_moh_reload(struct ast_cli_entry *e, int cmd, struct ast_
 	if (a->argc != e->args)
 		return CLI_SHOWUSAGE;
 
-	/* The module loader will prevent concurrent reloads from occurring, so we delegate */
-	ast_module_reload("res_musiconhold");
+	reload();
 
 	return CLI_SUCCESS;
 }
@@ -1954,6 +1973,16 @@ static int moh_class_cmp(void *obj, void *arg, int flags)
 		CMP_MATCH | CMP_STOP;
 }
 
+/*!
+ * \brief Load the module
+ *
+ * Module loading including tests for configuration or dependencies.
+ * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
+ * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
+ * configuration file or other non-critical problem return 
+ * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
+ */
 static int load_module(void)
 {
 	int res;
