@@ -18,9 +18,11 @@
 
 /*!
  * \file
- * \author Tilghman Lesher <tlesher AT digium DOT com>
+ * \author Tilghman Lesher \verbatim <tlesher AT digium DOT com> \endverbatim
  *
  * \brief kqueue timing interface
+ *
+ * \ingroup resource
  */
 
 /*** MODULEINFO
@@ -93,9 +95,7 @@ static int kqueue_timer_cmp(void *obj, void *args, int flags)
 static void timer_destroy(void *obj)
 {
 	struct kqueue_timer *timer = obj;
-	if (timer->handle > -1) {
-		close(timer->handle);
-	}
+	close(timer->handle);
 }
 
 #define lookup_timer(a)	_lookup_timer(a, __FILE__, __LINE__, __PRETTY_FUNCTION__)
@@ -123,12 +123,13 @@ static int kqueue_timer_open(void)
 		ast_log(LOG_ERROR, "Could not allocate memory for kqueue_timer structure\n");
 		return -1;
 	}
-	if ((timer->handle = handle = kqueue()) < 0) {
+	if ((handle = kqueue()) < 0) {
 		ast_log(LOG_ERROR, "Failed to create kqueue timer: %s\n", strerror(errno));
 		ao2_ref(timer, -1);
 		return -1;
 	}
 
+	timer->handle = handle;
 	ao2_link(kqueue_timers, timer);
 	/* Get rid of the reference from the allocation */
 	ao2_ref(timer, -1);
@@ -366,6 +367,16 @@ AST_TEST_DEFINE(test_kqueue_timing)
 }
 #endif
 
+/*!
+ * \brief Load the module
+ *
+ * Module loading including tests for configuration or dependencies.
+ * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
+ * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
+ * configuration file or other non-critical problem return 
+ * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
+ */
 static int load_module(void)
 {
 	if (!(kqueue_timers = ao2_container_alloc(563, kqueue_timer_hash, kqueue_timer_cmp))) {
