@@ -20,7 +20,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 396126 $")
 
 /*
  * There is some collision collision between netsmp and asterisk names,
@@ -298,10 +298,11 @@ static u_char *ast_var_channels_table(struct variable *vp, oid *name, size_t *le
 		}
 		break;
 	case ASTCHANBRIDGE:
-		if ((bridge = ast_bridged_channel(chan)) != NULL) {
+		if ((bridge = ast_channel_bridge_peer(chan)) != NULL) {
 			ast_copy_string(string_ret, ast_channel_name(bridge), sizeof(string_ret));
 			*var_len = strlen(string_ret);
 			ret = (u_char *)string_ret;
+			ast_channel_unref(bridge);
 		}
 		break;
 	case ASTCHANMASQ:
@@ -610,7 +611,7 @@ static u_char *ast_var_channel_bridge(struct variable *vp, oid *name, size_t *le
 
 	while ((chan = ast_channel_iterator_next(iter))) {
 		ast_channel_lock(chan);
-		if (ast_bridged_channel(chan)) {
+		if (ast_channel_is_bridged(chan)) {
 			long_ret++;
 		}
 		ast_channel_unlock(chan);
@@ -760,7 +761,7 @@ static u_char *ast_var_indications_table(struct variable *vp, oid *name, size_t
 	return NULL;
 }
 
-static int countmodule(const char *mod, const char *desc, int use, const char *like)
+static int countmodule(const char *mod, const char *desc, int use, const char *status, const char *like)
 {
 	return 1;
 }
