@@ -15,9 +15,10 @@
  * the GNU General Public License Version 2. See the LICENSE file
  * at the top of the source tree.
  */
+
 /*! \file
  *
- *  \brief A program to read in the /tmp/refs file generated
+ * \brief A program to read in the /tmp/refs file generated
  *         by astobj2 code when the REF_DEBUG macro is defined.
  *         It will read in the file line by line, and
  *         sort the data out by object, and check to see
@@ -28,15 +29,15 @@
  *         this program reads in the /tmp/refs file and 
  *         generates no output. No news is good news.
  *  The contents of the /tmp/refs file looks like this:
+ * \verbatim
+ * 0x84fd718 -1   astobj2.c:926:cd_cb_debug (deref object via container destroy) [@1]
+ * 0x84fd718 =1   chan_sip.c:19760:build_user (allocate a user struct)
+ * 0x84fd718 +1   chan_sip.c:21558:reload_config (link user into users table) [@1]
+ * 0x84fd718 -1   chan_sip.c:2376:unref_user (Unref the result of build_user. Now, the table link is the only one left.) [@2]
+ * 0x84fd718 **call destructor** astobj2.c:926:cd_cb_debug (deref object via container destroy)
+ * \endverbatim
  *
-0x84fd718 -1   astobj2.c:926:cd_cb_debug (deref object via container destroy) [@1]
-0x84fd718 =1   chan_sip.c:19760:build_user (allocate a user struct)
-0x84fd718 +1   chan_sip.c:21558:reload_config (link user into users table) [@1]
-0x84fd718 -1   chan_sip.c:2376:unref_user (Unref the result of build_user. Now, the table link is the only one left.) [@2]
-0x84fd718 **call destructor** astobj2.c:926:cd_cb_debug (deref object via container destroy)
- *
- *
- *  \author Steve Murphy <murf@digium.com>
+ * \author Steve Murphy <murf@digium.com>
  */
 
 /*** MODULEINFO
@@ -44,7 +45,7 @@
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 401936 $")
 
 #include <pthread.h>
 #include <sys/stat.h>
@@ -259,18 +260,6 @@ void ast_log(int level, const char *file, int line, const char *function, const
 	va_end(vars);
 }
 
-void ast_log_safe(int level, const char *file, int line, const char *function, const char *fmt, ...)
-{
-	va_list vars;
-	va_start(vars,fmt);
-	printf("LOG: lev:%d file:%s  line:%d func: %s  ",
-		   level, file, line, function);
-	vprintf(fmt, vars);
-	fflush(stdout);
-	va_end(vars);
-}
-
-
 void __ast_verbose(const char *file, int line, const char *func, int level, const char *fmt, ...)
 {
         va_list vars;
@@ -291,20 +280,20 @@ void ast_unregister_thread(void *id)
 {
 }
 #ifdef HAVE_BKTR
-struct ast_bt *ast_bt_create(void);
-struct ast_bt *ast_bt_create(void) 
+struct ast_bt *__ast_bt_create(void);
+struct ast_bt *__ast_bt_create(void)
 {
 	return NULL;
 }
 
-int ast_bt_get_addresses(struct ast_bt *bt);
-int ast_bt_get_addresses(struct ast_bt *bt)
+int __ast_bt_get_addresses(struct ast_bt *bt);
+int __ast_bt_get_addresses(struct ast_bt *bt)
 {
 	return 0;
 }
 
-char **ast_bt_get_symbols(void **addresses, size_t num_frames);
-char **ast_bt_get_symbols(void **addresses, size_t num_frames)
+char **__ast_bt_get_symbols(void **addresses, size_t num_frames);
+char **__ast_bt_get_symbols(void **addresses, size_t num_frames)
 {
 	char **foo = calloc(num_frames, sizeof(char *) + 1);
 	if (foo) {
@@ -316,9 +305,13 @@ char **ast_bt_get_symbols(void **addresses, size_t num_frames)
 	return foo;
 }
 
-void *ast_bt_destroy(struct ast_bt *bt);
-void *ast_bt_destroy(struct ast_bt *bt)
+void *__ast_bt_destroy(struct ast_bt *bt);
+void *__ast_bt_destroy(struct ast_bt *bt)
 {
 	return NULL;
 }
+
+void ast_log_backtrace(void)
+{
+}
 #endif
